<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 13c670923ab6 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-07-31 16:35 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
define('WE_DOCUMENT_ROOT', (string) realpath(__DIR__ . '/../'));
require_once('we/include/we.inc.php');

abstract class rpc{

	public static function processRPC(): void{
		$isAllow = we_html_tools::protect([], false);

		$protocol = 'json';


		if(!$isAllow){
			self::dieWithError('Not logged in', $protocol);
		}
		$cmd = we_base_request::string('cmd');
		//newer versions may distinguish the commands by the module name and give the specific command afterwards
		$mod = we_base_request::string('mod', $cmd);


		if(!($mod)){
			self::dieWithError('The Request is not well formed!', $protocol);
		}

		$shell = new we_rpc_cmdShell($protocol, $mod, $cmd);

		if($shell->getStatus() === we_rpc_cmd::STATUS_OK){
			$shell->executeCommand();
			echo $shell->getResponse();
			we_base_request::finishRequest();
			$shell->cleanUp();
		} else { // there was an error in initializing the command
			self::dieWithError($shell->getErrorOut(), $protocol);
		}
	}

	private static function dieWithError(string $text, string $protocol): void{
		switch($protocol){
			case 'json':
				$resp = new we_rpc_response();
				$resp->setStatus(false);
				$resp->setData('data', $text);
				$errorView = new we_rpc_genericJSONView($protocol);
				echo $errorView->getResponse($resp);
				exit;
			default:
				exit($text);
		}
	}

}

rpc::processRPC();
