<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 562fb8bcf3a8 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-06-30 12:28 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_base_ContentTypes{
	public const CUSTOMERGROUP = 'we/customerGroup';
	public const USERGROUP = 'we/userGroup';
	public const USERALIAS = 'we/alias';
	public const CUSTOMER = 'we/customer';
	public const USER = 'we/user';
	public const APPLICATION = 'application/*';
	public const AUDIO = 'audio/*';
	public const BACKUP_BINARY = 'weBackup/Binary';
	public const CATEGORIES = 'we/categories';
	public const CATEGORY = 'we/category';
	public const COLLECTION = 'we/collection';
	public const CSS = 'text/css';
	public const DOCTYPE = 'we/doctype';
	public const FOLDER_COMMON = 'folder';
	public const FOLDER_DOCUMENT = 'folder/document';
	public const FOLDER_OBJECTFILE = 'folder/objectFile';
	public const FOLDER_TEMPLATE = 'folder/template';
	public const FOLDER_PHPCLASS = 'folder/phpClass';
	public const FOLDER_VFILE = 'folder/vFile';
	public const GLOSSAR = 'we/glossar';
	public const HTACCESS = 'text/htaccess';
	public const HTML = 'text/html';
	public const IMAGE = 'image/*';
	public const JS = 'text/js';
	public const NAVIGATION = 'weNavigation';
	public const NAVIGATIONRULE = 'weNavigationRule';
	public const NEWSLETTER = 'we/newsletter';
	public const OBJECT = 'object';
	public const OBJECT_FILE = 'objectFile';
	public const PDF = 'application/pdf';
	public const TEMPLATE = 'text/weTmpl';
	public const PHPCLASS = 'text/phpClass';
	public const TEXT = 'text/plain';
	public const THUMBNAIL = 'weThumbnail';
	public const VERSION = 'we/version';
	public const VIDEO = 'video/*';
	public const VOTING = 'we/voting';
	public const WEDOCUMENT = 'text/webedition';
	public const XML = 'text/xml';
	private const _CHARSET = '###CHARSET###';
	//all content types which should be accessed via link-element (href)
	public const LINK_ABLE = [
		self::APPLICATION,
		self::AUDIO,
		self::CSS,
		self::HTML,
		self::IMAGE,
		self::JS,
		self::PDF,
		self::TEXT,
		self::VIDEO,
		self::WEDOCUMENT,
		self::XML,
	];
	public const FOLDER_TYPES = [
		self::FOLDER_COMMON,
		self::FOLDER_DOCUMENT,
		self::FOLDER_OBJECTFILE,
		self::FOLDER_TEMPLATE,
		self::FOLDER_PHPCLASS,
		self::FOLDER_VFILE,
	];

	/** @var array<string,array{Extension:array,ExtensionIsFilename:bool,Exclude:array,ContentTypes:array,Permission:array,IsRealFile:bool,IsWebEditionFile:bool,Table:array,Class:string,IsBinaryFile:bool}>	 */
	private const CONTENT_TYPES = [
		self::IMAGE => [
			'Class' => 'we_document_image',
			'ContentTypes' => ['image/jpeg', 'image/gif', 'image/png', 'image/svg+xml', 'image/svg-xml', 'image/webp'],
			'Exclude' => [],
			'Extension' => ['.gif', '.jpg', '.jpeg', '.png', '.svg', '.svgz', '.jfif', '.webp'],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => true,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Permission' => [we_base_perm::WORK_IMAGE_NEW],
			'Table' => [FILE_TABLE],
		],
		self::XML => [//this entry must stay before text/html, text/we because fileextensions are not distinct
			'Class' => 'we_document_text',
			'ContentTypes' => ['text/xml'],
			'Exclude' => [],
			'Extension' => ['.xml'],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Permission' => [we_base_perm::WORK_TEXT_NEW],
			'Table' => [FILE_TABLE],
		],
		self::HTML => [
			'Class' => 'we_document_html',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => ['.html', '.htm', '.shtm', '.shtml', '.stm', '.php', '.jsp', '.asp', '.aspx', '.pl', '.cgi', '.xml', '.xsl'],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Permission' => [we_base_perm::WORK_HTML_NEW],
			'Table' => [FILE_TABLE],
		],
		self::WEDOCUMENT => [
			'Class' => 'we_document_webEdition',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => ['.html', '.htm', '.shtm', '.shtml', '.stm', '.php', '.jsp', '.asp', '.aspx', '.pl', '.cgi', '.xml'],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false,
			'IsWebEditionFile' => true,
			'Permission' => [we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW],
			'Table' => [FILE_TABLE],
		],
		self::TEMPLATE => [
			'Class' => 'we_contents_template',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => ['.tmpl'],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Permission' => [we_base_perm::WORK_TEMPLATE_NEW],
			'Table' => [TEMPLATES_TABLE],
		],
		self::JS => [
			'Class' => 'we_document_text',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => ['.js', '.mjs'],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Permission' => [we_base_perm::WORK_JS_NEW],
			'Table' => [FILE_TABLE],
		],
		self::CSS => [
			'Class' => 'we_document_text',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => ['.css', '.less', '.scss', '.sass'],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Permission' => [we_base_perm::WORK_CSS_NEW],
			'Table' => [FILE_TABLE],
		],
		self::HTACCESS => [
			'Class' => 'we_document_text',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => ['.htaccess', '.htpasswd'],
			'ExtensionIsFilename' => true,
			'IsBinaryFile' => false,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Permission' => [we_base_perm::WORK_HTACCESS_NEW],
			'Table' => [FILE_TABLE],
		],
		self::TEXT => [
			'Class' => 'we_document_text',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => ['.txt', '.csv'],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Permission' => [we_base_perm::WORK_TEXT_NEW],
			'Table' => [FILE_TABLE],
		],
		self::FOLDER_DOCUMENT => [
			'Class' => 'we_folder_document',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => [],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Permission' => [],
			'Table' => [FILE_TABLE],
		],
		self::FOLDER_OBJECTFILE => [
			'Class' => 'we_folder_objectFile',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => [],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Permission' => [],
			'Table' => [OBJECT_FILES_TABLE],
		],
		self::FOLDER_TEMPLATE => [
			'Class' => 'we_folder_template',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => [],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Permission' => [],
			'Table' => [TEMPLATES_TABLE],
		],
		self::FOLDER_VFILE => [
			'Class' => 'we_folder_vfile',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => [],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Permission' => [],
			'Table' => [VFILE_TABLE],
		],
		self::VIDEO => [
			'Class' => 'we_document_video',
			'ContentTypes' => ['video/mp4', 'video/webm', 'application/ogg', 'video/ogg',],
			'Exclude' => [],
			'Extension' => ['.mp4', '.m4v', '.ogg', '.webm'],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => true,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Permission' => [we_base_perm::WORK_VIDEO_NEW],
			'Table' => [FILE_TABLE],
		],
		self::AUDIO => [
			'Class' => 'we_document_audio',
			'ContentTypes' => ['audio/mp3', 'audio/ogg', 'audio/wav'],
			'Exclude' => [],
			'Extension' => ['.mp3', '.wav', '.ogg'],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => true,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Permission' => [we_base_perm::WORK_VIDEO_NEW],
			'Table' => [FILE_TABLE],
		],
		self::PDF => [
			'Class' => 'we_document_pdf',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => ['.pdf'],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => true,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Permission' => [we_base_perm::WORK_OTHER_NEW],
			'Table' => [FILE_TABLE],
		],
		self::APPLICATION => [
			'Class' => 'we_document_other',
			'ContentTypes' => [],
			'Exclude' => [self::AUDIO, self::IMAGE, self::PDF, self::VIDEO],
			'Extension' => [],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => true,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Permission' => [we_base_perm::WORK_OTHER_NEW],
			'Table' => [FILE_TABLE],
		],
		self::OBJECT => [
			'Class' => 'we_contents_object',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => [],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Permission' => [],
			'Table' => [OBJECT_TABLE],
		],
		self::OBJECT_FILE => [
			'Class' => 'we_contents_objectFile',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => [],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Permission' => [],
			'Table' => [OBJECT_FILES_TABLE],
		],
		self::COLLECTION => [
			'Class' => 'we_contents_collection',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => [],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false, //TODO: use this when saving
			'IsWebEditionFile' => false,
			'Permission' => [],
			'Table' => [VFILE_TABLE],
		],
		self::CATEGORY => [
			'Class' => 'we_category_category',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => [],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Permission' => [],
			'Table' => [CATEGORY_TABLE],
		],
		self::GLOSSAR => [
			'Class' => 'we_glossary_glossary',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => [],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Permission' => [],
			'Table' => [GLOSSARY_TABLE],
		],
		self::CUSTOMER => [
			'Class' => 'we_customer_customer',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => [],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Permission' => [],
			'Table' => [CUSTOMER_TABLE],
		],
		self::NEWSLETTER => [
			'Class' => 'we_newsletter_newsletter',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => [],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Permission' => [],
			'Table' => [NEWSLETTER_TABLE],
		],
		self::NAVIGATION => [
			'Class' => 'we_navigation_navigation',
			'ContentTypes' => [],
			'Exclude' => [],
			'Extension' => [],
			'ExtensionIsFilename' => false,
			'IsBinaryFile' => false,
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Permission' => [],
			'Table' => [NAVIGATION_TABLE],
		],
	];

	public static function hasContentType(string $name): bool{
		return array_key_exists($name, self::CONTENT_TYPES);
	}

	/**
	 *
	 * @param string $filter
	 * @param bool $filterOmitFolder
	 * @return string[]
	 */
	public static function getContentTypes(string $filter = '', bool $filterOmitFolder = false): array{
		if($filter){
			$ret = [];
			foreach(self::CONTENT_TYPES as $k => $v){
				switch($k){
					case self::FOLDER_DOCUMENT:
					case self::FOLDER_OBJECTFILE:
					case self::FOLDER_TEMPLATE:
					case self::FOLDER_VFILE:
						if($filterOmitFolder){
							continue 2;
						}
					default:
						if(in_array($filter, $v['Table'], true)){
							$ret[] = $k;
						}
				}
			}

			return $ret;
		}
		return array_keys(self::CONTENT_TYPES);
	}

	/**
	 * @param $name
	 * @param bool $ignoreIsFilename
	 * @return string[]
	 */
	public static function getExtension(string $name, bool $ignoreIsFilename = false): array{
		return array_key_exists($name, self::CONTENT_TYPES) && ($ignoreIsFilename || !self::CONTENT_TYPES[$name]['ExtensionIsFilename']) ?
			self::CONTENT_TYPES[$name]['Extension'] : [];
	}

	/**
	 * @param string $name
	 * @return string[]
	 */
	public static function getExcluded(string $name): array{
		if(!array_key_exists($name, self::CONTENT_TYPES)){
			return [];
		}

		$extensions = [];
		foreach(self::CONTENT_TYPES[$name]['Exclude'] as $excluded){
			if(array_key_exists($excluded, self::CONTENT_TYPES) && ($exts = self::CONTENT_TYPES[$excluded]['Extension'])){
				$extensions = array_merge($extensions, $exts);
			} elseif($excluded[0] === '.'){
				$extensions[] = strtolower($excluded);
			}
		}

		return array_unique($extensions);
	}

	public static function isWEFile(string $name): bool{
		return array_key_exists($name, self::CONTENT_TYPES) ? self::CONTENT_TYPES[$name]['IsWebEditionFile'] : false;
	}

	public static function isBinaryFile(string $name): bool{
		return array_key_exists($name, self::CONTENT_TYPES) ? self::CONTENT_TYPES[$name]['IsBinaryFile'] : false;
	}

	/**
	 *
	 * @return string[]
	 */
	public static function getWETypes(): array{
		$ret = [];
		foreach(self::CONTENT_TYPES as $name => $type){
			if($type['IsWebEditionFile']){
				$ret[] = $name;
			}
		}
		return $ret;
	}

	public static function getObject(string $name = '', int $id = 0): we_base_mainmodel{
		if(!$name){
			throw new InvalidArgumentException();
		}
		if(array_key_exists($name, self::CONTENT_TYPES) && class_exists(self::CONTENT_TYPES[$name]['Class'], true)){
			$classname = self::CONTENT_TYPES[$name]['Class'];
			$obj = new $classname();
			if($obj instanceof we_base_model){
				$obj->load($id);
			} elseif($obj instanceof we_contents_base){
				$obj->initByID($id);
			}

			return $obj;
		}

		t_e('init with unknown contenttype', $name);
		$classname = 'we_' . $name;
		if(class_exists($classname, true)){
			return new $classname($id);
		}
		t_e('Can NOT initialize document of type -' . $name . '- ' . $classname);
		throw new BadFunctionCallException('Can NOT initialize document of type -' . $name . '- ' . $classname);
	}

	public static function getDefaultCode(string $name): string{
		$code = '';
		switch($name){
			case self::XML:
				$code = '<?xml version="1.0" encoding="' . self::_CHARSET . '" ?>';
				break;
			case self::HTML:
				$code = '<!doctype html>
<html>
	<head>
		<title></title>
		<meta charset="' . self::_CHARSET . '">
	</head>
	<body>
	<!-- webEdition demo template (' . WE_VERSION . ') -->
	</body>
</html>';
				break;
			case self::TEMPLATE:
				$code = '<!DOCTYPE HTML>
<html dir="ltr" lang=\'<we:pageLanguage type="language" />\'>
<head>
	<we:title></we:title>
	<we:description></we:description>
	<we:keywords></we:keywords>
	<we:charset defined="UTF-8">UTF-8</we:charset>
</head>
<body>
	<!-- webEdition demo template (' . WE_VERSION . ') -->
	<article style="width:400px">
		<h1><we:input type="text" name="Headline" style="width:60em"/></h1>
		<p><b><we:input type="date" name="Date" currentdate="true" format="' . g_l('weEditorInfo', '[date_only_format]') . '"/></b></p>
		<we:ifNotEmpty name="Image">
			<p><we:img name="Image" showthumbcontrol="true"/></p>
		</we:ifNotEmpty>
		<we:textarea name="Content" width="400" height="400" wysiwyg="true" removefirstparagraph="false" inlineedit="true"/>
	</article>
</body>
</html>';
				break;
		}
		return $code ? strtr($code, [self::_CHARSET => 'UTF-8']) : '';
	}

	/**
	 *
	 * @param string $name
	 * @return string[]
	 */
	public static function getPermission(string $name): array{
		return array_key_exists($name, self::CONTENT_TYPES) ? self::CONTENT_TYPES[$name]['Permission'] : [];
	}

	public static function getTypeForExtension(string $extension): string{
		if(!$extension){
			return self::APPLICATION;
		}
		$ext = strtolower($extension);
		foreach(self::CONTENT_TYPES as $type => $val){
			if(in_array($ext, $val['Extension'], true)){
				return $type;
			}
		}
		return self::APPLICATION;
	}

	/**
	 *
	 * @param string $name
	 * @return string[]
	 */
	public static function getRealContentTypes(string $name): array{
		return (array_key_exists($name, self::CONTENT_TYPES)) ?
			self::CONTENT_TYPES[$name]['ContentTypes'] :
			[];
	}

	/**
	 * @return string[]
	 */
	public static function getFiles(): array{
		$ret = [];
		foreach(self::CONTENT_TYPES as $type => $val){
			if($val['IsRealFile']){
				$ret[] = $type;
			}
		}
		return $ret;
	}

	/**
	 *
	 * @param string $name
	 * @return bool true if the file is present in filesystem
	 */
	public static function isRealFile(string $name): bool{
		return (array_key_exists($name, self::CONTENT_TYPES)) ?
			self::CONTENT_TYPES[$name]['IsRealFile'] :
			false;
	}

	public static function getFromFilename(string $filename): string{
		if(is_link($filename)){
			return 'link';
		}
		if(is_dir($filename)){
			return self::FOLDER_DOCUMENT;
		}
		$ext = strtolower(preg_replace('#^.*(\..+)$#', '${1}', $filename));
		return self::getTypeForExtension($ext);
	}

}
