<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: aa7c4e7e08c2 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-02-24 13:26 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_base_browserDetect{

	public const UNKNOWN = 'unknown';
	public const OPERA = 'opera';
	public const IE = 'ie';
	public const EDGE = 'edge';
	public const FF = 'firefox';
	public const LYNX = 'lynx';
	public const JAVA = 'java';
	public const KONQUEROR = 'konqueror';
	public const NETSCAPE = 'nn';
	public const MOZILLA = 'mozilla';
	public const APPLE = 'appleWebKit';
	public const SAFARI = 'safari';
	public const CHROME = 'chrome';
	public const SYS_MAC = 'macintosh';
	public const SYS_WIN = 'win';
	public const SYS_UNIX = 'unix';
	public const SYS_ANDROID = 'android';
	public const SYS_IPHONE = 'iphone';
	public const SYS_IPAD = 'ipad';

	/** @var string Browser */
	protected static $br = self::UNKNOWN;

	/** @var string of useragent */
	protected static $ua = '';

	/** @var float Version */
	protected static $v = 0.0;

	/** @var string Operating System */
	protected static $sys = self::UNKNOWN;

	/** @var bool determines, if browser already detected */
	private static $detected = false;

	private function __construct(string $ua = ''){
		//prevent from redetecting the same strings
		if(self::$detected && $ua === ''){
			return;
		}
		if($ua != ''){
			self::$br = self::UNKNOWN;
			self::$v = 0;
			self::$sys = self::UNKNOWN;
		}
		self::$detected = true;
		self::$ua = $ua ?: ($_SERVER['HTTP_USER_AGENT'] ?? '');
		$regs = [];
		if(preg_match('|^([^ ]+) ([^(]*)(\([^)]+\))(.*)$|', self::$ua, $regs)){
			$pre = $regs[1];
			//$mid = $regs[2];
			$bracket = str_replace(['(', ')'], '', $regs[3]);
			$post = $regs[4];

			$tmp = explode('/', $pre);
			$bez = strtolower($tmp[0]);
			$prever = $tmp[1] ?? '';

			switch($bez){
				case 'lynx':
					self::$br = self::LYNX;
					break;
				case 'mozilla':
					self::_getSys($bracket);
					if(stristr($post, 'Java')){
						self::$br = self::JAVA;
						[, $vers] = explode('/', trim($post));
						self::$v = (float) $vers;
						break;
					}
					if(preg_match('|msie (.*)$|i', $bracket, $regs) && (trim($post) === '' || preg_match('|\.net|i', $post)) || preg_match('|Trident.*rv:(\d+\.?\d?)|i', $bracket, $regs)){
						self::$br = self::IE;
						self::$v = (float) ($regs[1] ?? 0);
						break;
					}
					if(preg_match('|konqueror/(.*)$|i', $bracket, $regs)){
						self::$br = self::KONQUEROR;
						self::$v = (float) ($regs[1]);
						break;
					}
					if(preg_match('|galeon/(.*)$|i', $bracket, $regs)){
						self::$br = self::UNKNOWN;
						self::$v = (float) ($regs[1]);
						break;
					}

					if(stristr($post, 'edg')){
						self::$br = self::EDGE;
						self::$v = (preg_match('|edge?/(.+)|i', $post, $regs) ? (float) (trim($regs[1])) : 12);
						break;
					}
					if(preg_match('|AppleWebKit/([0-9.]+)|i', $post, $regs)){
						if(stristr($post, 'chrome')){
							self::$v = (preg_match('|chrome/([0-9]+\.[0-9]+)|i', $post, $regs) ? (float) ($regs[1]) : 1);
							self::$br = self::CHROME;
							break;
						}
						if(stristr($post, 'safari')){
							self::$v = (preg_match('|version/([0-9]+\.[0-9]+)|i', $post, $regs) ? (float) ($regs[1]) : 1);
							self::$br = self::SAFARI;
							break;
						}
						self::$v = (float) ($regs[1]);
						self::$br = self::APPLE;
						break;
					}
					if(preg_match('|firefox/([0-9]+.[0-9]+)|i', $post, $regs)){
						self::$v = (float) ($regs[1]);
						self::$br = self::FF;
						break;
					}
					if(stristr($post, 'gecko')){
						self::$br = self::MOZILLA;
						if(preg_match('|rv:([0-9.]*)|i', $bracket, $regs)){
							self::$v = (float) ($regs[1]);
						}
						break;
					}
					if(preg_match('|opera ([^ ]+)|i', $post, $regs)){
						$reg = [];
						if(stristr($post, 'chrome')){
							self::$v = (preg_match('|chrome/([0-9]+\.[0-9]+)|i', $post, $regs) ? (float) ($regs[1]) : 1);
							self::$br = self::CHROME;
						} else {
							self::$br = self::OPERA;
							self::$v = (preg_match('|version/([^ ]+)|i', $post, $reg) ?
									(float) ($reg[1]) : (float) ($regs[1]));
						}
						break;
					}
					if(stristr($bracket, 'compatible')){
						self::$br = self::UNKNOWN;
						break;
					}
					if(!stristr($bracket, 'msie')){
						self::$br = self::NETSCAPE;
						self::$v = (float) (preg_replace('|[^0-9.]|', '', $prever));
						break;
					}
					break;
				case 'opera':
					if(stristr($post, 'chrome')){
						self::$v = (preg_match('|chrome/([0-9]+\.[0-9]+)|i', $post, $regs) ? (float) ($regs[1]) : 1);
						self::$br = self::CHROME;
					} else {
						self::$br = self::OPERA;
						self::$v = (float) (preg_match('|version/([^ ]+)|i', $post, $reg) ? $reg[1] : $prever);
					}
					self::_getSys($bracket);
					break;
				case 'googlebot':
					self::$br = self::UNKNOWN;
					#self::$v=$prever;
					break;
				case 'nokia-communicator-www-Browser':
					self::$br = self::UNKNOWN;
					break;
			}
			if(self::$sys == self::UNKNOWN){
				if(stristr(self::$ua, 'webtv')){
					self::$sys = 'webtv';
				}
			}
		} elseif(preg_match('|^lynx([^a-z]+)[a-z].*|i', $ua, $regs)){
			self::$br = self::LYNX;
			self::$v = (float) (str_replace('/', '', $regs[1]));
		} else {
			self::$br = self::UNKNOWN;
		}
	}

	private static function _getSys(string $bracket): void{
		switch($bracket){
			case (stristr($bracket, self::SYS_MAC) !== false):
				self::$sys = self::SYS_MAC;
				break;
			case (stristr($bracket, self::SYS_WIN) !== false):
				self::$sys = self::SYS_WIN;
				break;
			case (stristr($bracket, self::SYS_ANDROID) !== false):
				self::$sys = self::SYS_ANDROID;
				break;
			case (stristr($bracket, self::SYS_IPHONE) !== false):
				self::$sys = self::SYS_IPHONE;
				break;
			case (stristr($bracket, self::SYS_IPAD) !== false):
				self::$sys = self::SYS_IPAD;
				break;
			case (stristr($bracket, self::SYS_UNIX) !== false || stristr($bracket, 'x11') !== false || stristr($bracket, 'sun') !== false):
				self::$sys = self::SYS_UNIX;
				break;
		}
	}

	public static function inst(string $ua = ''): self{
		if($ua){
			return new self($ua);
		}
		static $ref = 0;
		if(!is_object($ref)){
			$ref = new self();
		}
		return $ref;
	}

	public function getBrowser(): string{
		return self::$br;
	}

	public static function isIE(): bool{
		return self::inst()->getBrowser() == self::IE;
	}

	public static function isEdge(): bool{
		return self::inst()->getBrowser() == self::EDGE;
	}

	public static function isOpera(): bool{
		return self::inst()->getBrowser() == self::OPERA;
	}

	public static function isSafari(): bool{
		return self::inst()->getBrowser() == self::SAFARI;
	}

	public static function isNN(): bool{
		switch(self::inst()->getBrowser()){
			case self::NETSCAPE:
			case self::MOZILLA:
			case self::FF:
				return true;
		}
		return false;
	}

	public static function isFF(): bool{
		return self::inst()->getBrowser() == self::FF;
	}

	public static function isChrome(): bool{
		return self::inst()->getBrowser() == self::CHROME;
	}

	public static function isMAC(): bool{
		return self::inst()->getSystem() == self::SYS_MAC;
	}

	public static function isUNIX(): bool{
		return self::inst()->getSystem() == self::SYS_UNIX;
	}

	public static function isWin(): bool{
		return self::inst()->getSystem() == self::SYS_WIN;
	}

	public static function getIEVersion(): int{
		return self::isIE() ? intval(self::$v) : -1;
	}

	public function getBrowserVersion(): float{
		return self::$v;
	}

	public function getSystem(): string{
		return self::$sys;
	}

	public function getUserAgent(): string{
		return self::$ua;
	}

	public function getWebKitVersion(): int{
		$regs = [];
		if(preg_match('|AppleWebKit/([^ ]+)|i', self::$ua, $regs)){
			return ((int) $regs[1]);
		}
		return 0;
	}

	public static function isGecko(): bool{
		return str_contains(strtolower(self::inst()->getUserAgent()), 'gecko');
	}

}
