<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e3e190613117 $
 * $Author: Lukas Imhof $
 * $Date: 2021-08-16 15:35 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Definition of webEdition Base Model
 *
 */
class we_base_model extends we_base_mainmodel{
	/** @var we_database_base */
	protected $db;

	/** @var string */
	var $table = '';

	/** @var array<string,string> */
	var $persistent_slots = [];

	/** @var string */
	protected static $persistentTable = '';

	/** @var array<string,string> */
	protected static $persistentSlots = [];

	/** @var string[] */
	protected $keys = ['ID'];

	/** @var bool */
	var $isnew = true;

	/** @var bool */
	public $wasUpdate = false;

	/** @var int[] */
	protected $MediaLinks = [];

	/** @var string[] */
	protected $binFields = [];

	/** @var string[] */
	protected $dateFields = [];

	/** @var int ID from the database record */
	var $ID = 0;

	/** @var int */
	protected $ParentID = 0;

	/** @var string */
	protected $Text = '';

	/** @var string */
	protected $Path = '';

	public const SERIALIZABLE_FIELDS = [
		we_base_request::INTLISTA => SERIALIZE_LIST,
		we_base_request::SERIALIZED => SERIALIZE_JSON,
		we_base_request::SERIALIZED_JSON => SERIALIZE_JSON,
		we_base_request::STRING_LIST => SERIALIZE_LIST,
		we_base_request::EMAILLISTA => SERIALIZE_LIST,
		we_base_request::WEFILELISTA => SERIALIZE_LIST,
	];

	//form name=>type

	/**
	 * Default Constructor
	 */
	public function __construct(string $table, we_database_base $db = null){
		$this->db = ($db ?: new DB_WE());
		$this->table = $table;
	}

	/**
	 * Load entry from database
	 */
	function load(int $id = 0, bool $loadData = true): bool{
		if($id){
			$this->ID = $id;
		}
		if(!$this->isKeyDefined()){
			return false;
		}
		$data = getHash('SELECT `' . implode('`,`', array_keys($this->persistent_slots)) . '` FROM `' . $this->table . '` WHERE ' . $this->getKeyWhere());
		if(!$data){
			return false;
		}
		foreach($data as $fieldName => $value){
			$type = $this->persistent_slots[$fieldName] ?? '';
			$this->{$fieldName} = we_base_request::filterVar($value, $type, isset(self::SERIALIZABLE_FIELDS[$type]) ? [] : ''); //@phan-suppress-current-line PhanDeprecatedFunction
		}
		$this->isnew = false;

		return true;
	}

	/**
	 * save entry in database
	 */
	function save(bool $force_new = false, bool $rebuild = false, array $skipWrite = []): bool{
		$sets = [];
		if($force_new){
			$this->isnew = true;
		}

		foreach($this->persistent_slots as $fieldName => $type){
			if(in_array($fieldName, $skipWrite)){
				continue;
			}
			if(property_exists($this, $fieldName)){
				$eval = $this->{$fieldName};
				$method = SERIALIZE_JSON;
				if(is_array($eval)){
					if(isset(self::SERIALIZABLE_FIELDS[$type])){
						$method = self::SERIALIZABLE_FIELDS[$type];
					} else {
						t_e('FIXME', $fieldName, 'set as ' . $type . ' not set as serialized:', self::SERIALIZABLE_FIELDS);
					}
				}
				if(in_array($fieldName, $this->dateFields, true) && is_numeric($eval)){
					t_e('found old access to datefield', $fieldName, $eval);
				}
				$sets[$fieldName] = is_array($eval) ?
					(empty($eval) ? '' : we_serialize($eval, $method)) :
					(in_array($fieldName, $this->binFields, true) ?
					sql_function('x\'' . bin2hex((string) $eval) . '\'') :
					(in_array($fieldName, $this->dateFields, true) && is_numeric($eval) ?
					($eval == 0 ? null : sql_function('FROM_UNIXTIME(' . $eval . ')')) :
					$eval));
			}
		}

		$where = $this->getKeyWhere();
		$set = we_database_base::arraySetter($sets);

		if($this->isKeyDefined()){
			if($this->isnew){
				$ret = $this->db->query('REPLACE INTO ' . $this->db->escape($this->table) . ' SET ' . $set, false, true);
				# get ID #
				if($ret){
					$this->ID = $this->db->getInsertId();
					$this->isnew = false;
				}
				return $ret;
			}
			$this->wasUpdate = true;
			return $this->db->query('UPDATE ' . $this->db->escape($this->table) . ' SET ' . $set . ' WHERE ' . $where);
		}

		return false;
	}

	protected function convertToHiddenString(string $fieldName, string $type, $data): string{
		$method = SERIALIZE_JSON;
		if(is_array($data)){
			if(isset(self::SERIALIZABLE_FIELDS[$type])){
				$method = self::SERIALIZABLE_FIELDS[$type];
			} else {
				t_e('FIXME', $fieldName, 'set as ' . $type . ' not set as serialized!', self::SERIALIZABLE_FIELDS);
			}
		}
		return (string)
			(is_array($data) ?
			(empty($data) ? '' : we_serialize($data, $method)) :
			(in_array($fieldName, $this->dateFields, true) && !is_numeric($data) ?
			$data->format(we_database_base::DATE_TIME_FORMAT) :
			($type === we_base_request::BOOL ?
			(int) $data :
			$data
			)));
	}

	/**
	 * delete entry from database
	 */
	function delete(): bool{
		if(!$this->isKeyDefined()){
			return false;
		}
		$this->db->query('DELETE FROM ' . $this->db->escape($this->table) . ' WHERE ' . $this->getKeyWhere());
		return true;
	}

	function registerMediaLinks(): bool{ // FIXME: use this for categorys and newsletter too
		$this->MediaLinks = array_filter($this->MediaLinks, function($v){
			return $v && is_numeric($v);
		});

		// filter MediaLinks by media contenttype
		$verifiedIDs = [];
		if(!empty($this->MediaLinks)){
			$whereType = 'AND ContentType IN ("' . we_base_ContentTypes::APPLICATION . '","' . we_base_ContentTypes::IMAGE . '","' . we_base_ContentTypes::VIDEO . '","' . we_base_ContentTypes::PDF . '","' . we_base_ContentTypes::AUDIO . '")';
			$verifiedIDs = $this->db->getAllq('SELECT ID FROM ' . FILE_TABLE . ' WHERE ID IN (' . implode(',', array_unique(array_values($this->MediaLinks))) . ') ' . $whereType, true);
		}
		$this->MediaLinks = array_intersect($this->MediaLinks, $verifiedIDs);

		if(empty($this->MediaLinks)){
			return true;
		}

		foreach($this->MediaLinks as $element => $remObj){
			$this->db->query('REPLACE INTO ' . FILELINK_TABLE . ' SET ' . we_database_base::arraySetter([
					'ID' => $this->ID,
					'DocumentTable' => stripTblPrefix($this->table),
					'type' => 'media',
					'remObj' => $remObj,
					'remTable' => stripTblPrefix(FILE_TABLE),
					'nHash' => is_numeric($element) ? sql_function('x\'00000000000000000000000000000000\'') : sql_function('x\'' . md5($element) . '\''),
					'position' => 0,
					'isTemp' => 0
			]));
		}
		return true;
	}

	function unregisterMediaLinks(): bool{
		return $this->db->query('DELETE FROM ' . FILELINK_TABLE . ' WHERE ID=' . ((int) $this->ID) . ' AND DocumentTable="' . $this->db->escape(stripTblPrefix($this->table)) . '"  AND type="media"');
	}

	protected function getKeyWhere(): string{
		$wheres = [];
		foreach($this->keys as $f){
			if(property_exists($this, $f)){
				$wheres[] = '`' . $f . '`="' . $this->db->escape($this->{$f}) . '"';
			}
		}
		return implode(' AND ', $wheres);
	}

	protected function isKeyDefined(): bool{
		foreach($this->keys as $prim){
			if(!property_exists($this, $prim)){
				return false;
			}
		}
		return true;
	}

	/**
	 *
	 * @param string[] $keys
	 * @return void
	 */
	function setKeys(array $keys): void{
		$this->keys = $keys;
	}

	public function getLogString(string $prefix = ''): string{
		return $prefix . $this->table;
	}

	/**
	 *
	 * @return string[]
	 */
	public function __sleep(): array{
		$tmp = get_object_vars($this);
		unset($tmp['db']);
		return array_keys($tmp);
	}

	public function __wakeup(): void{
		$this->db = new DB_WE();
	}

	public function saveField(string $name): bool{
		$type = $this->persistent_slots[$name];
		$method = (self::SERIALIZABLE_FIELDS[$type] ?? '');

		return $this->db->query('UPDATE ' . $this->db->escape($this->table) . ' SET ' . we_database_base::arraySetter([
					$name => ($method ? we_serialize($this->{$name}, $method) : $this->{$name})
				]) .
				' WHERE ID=' . ((int) $this->ID));
	}

	public function getText(): string{
		return $this->Text;
	}

	public function evalPath(int $id, string $table): string{
		$id = $id ?: (int) $this->ParentID;

		['Text' => $text, 'ParentID' => $pid] = getHash('SELECT Text,ParentID FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $id) ?:
			['Text' => '', 'ParentID' => 0];
		'@phan-var int $pid';
		$path = [$text];

		while($pid > 0){
			$foo = getHash('SELECT Text,ParentID FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $pid);
			if(!$foo){
				$pid = 0;
				continue;
			}
			$path[] = $foo['Text'];
			$pid = (int) $foo['ParentID'];
		}
		return '/' . implode('/', array_reverse($path));
	}

	function setPath(): void{
		$this->Path = fS('SELECT Path FROM ' . $this->db->escape($this->table) . ' WHERE ID=' . ((int) $this->ParentID)) . '/' . $this->Text;
	}

	function pathExists(string $path): bool{
		return fB('SELECT 1 FROM ' . $this->db->escape($this->table) . ' WHERE Path="' . $this->db->escape($path) . '" AND ID!=' . ((int) $this->ID) . ' LIMIT 1');
	}

	function isSelf(): bool{
		return (str_contains(we_base_file::clearPath(dirname($this->Path) . '/'), '/' . $this->Text . '/'));
	}

	public function isSameAsParent(): bool{
		if(!$this->ID){
			return false;
		}
		$pid = (int) $this->ParentID;
		while($pid){
			if($this->ID == $pid){
				return true;
			}
			$pid = fI('SELECT ParentID FROM ' . $this->db->escape($this->table) . '  WHERE ID=' . $pid);
		}

		return false;
	}

	protected function deleteChilds(): bool{
		$childs = $this->db->getAllq('SELECT ID FROM ' . $this->db->escape($this->table) . ' WHERE ParentID=' . ((int) $this->ID), true);
		assert($this instanceof we_category_category ||
			$this instanceof we_customer_customer ||
			$this instanceof we_export_export ||
			$this instanceof we_glossary_glossary ||
			$this instanceof we_navigation_navigation ||
			$this instanceof we_newsletter_base ||
			$this instanceof we_voting_voting);
		foreach($childs as $curchild){
			$child = new static((int) $curchild); //@phan-suppress-current-line PhanTypeMismatchArgumentReal
			$child->delete();
		}
		return true;
	}

}
