<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: af87aa135b22 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-03-25 09:29 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_chooser_multiFile extends we_chooser_base{
	/** @var string[] */
	protected $ids = [];

	/** @var int[] */
	private $disabledDelItems = [];

	/** @var string */
	private $disabledDelReason = '';

	/** @var string */
	private $iconFile;

	/** @var string */
	private $iconFolder;

	/** @var string */
	private $cmd_edit;

	/** @var string */
	private $cmd_del;

	/** @var string */
	private $addbut;

	/** @var int */
	private $nr = 0;

	/** @var string */
	private $css = '';

	/**
	 * @param string[] $ids
	 * @param string $cmd_del
	 * @param string $addbut
	 * @param string $cmd_edit
	 * @param string $iconFolder
	 * @param string $iconFile
	 */
	public function __construct(array $ids, string $cmd_del, string $addbut, string $cmd_edit, string $iconFolder = we_base_ContentTypes::FOLDER_DOCUMENT, string $iconFile = we_base_ContentTypes::APPLICATION, string $extraDelFn = ''){
		$this->ids = $ids;
		$this->cmd_del = $cmd_del;
		$this->addbut = $addbut;
		$this->extraDelFn = $extraDelFn;

		$this->iconFile = $iconFile;
		$this->iconFolder = $iconFolder;
		$this->cmd_edit = $cmd_edit;
	}

	/**
	 *
	 * @param int[] $items
	 * @param string $reason
	 * @return void
	 */
	public function setDisabledDelItems(array $items, string $reason): void{
		$this->disabledDelItems = $items;
		$this->disabledDelReason = $reason;
	}

	function get(we_base_jsCmd $jsCmd): string{
		$table = new we_html_table(['class' => 'default', 'width' => '100%'], count($this->ids), 4);
		$this->nr = 0;
		$c = 0;

		foreach($this->ids as $id){
			$edit = null;

			if($this->isEditable && $this->cmd_edit){
				$edit = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:" . $this->getJsSetHot() . ";we_cmd('" . $this->cmd_edit . "','" . $id . "');");
			}

			if(($this->isEditable && $this->cmd_del) || $this->CanDelete){

				if($this->disabledDelItems){
					$DisArr = $this->disabledDelItems;
					if(in_array($id, $DisArr)){
						$trash = we_html_button::create_button(we_html_button::TRASH, "javascript:" . $this->getJsSetHot() . ($this->extraDelFn ?: '') . ";we_cmd('" . $this->cmd_del . "','" . $id . "');", '', '', true);

						$table->setColAttributes($c, 0, ['title' => $this->disabledDelReason, 'class' => 'chooserFileIcon', 'data-contenttype' => (is_dir($id) ? $this->iconFolder : $this->iconFile)])
							->setCol($c, 1, (string) $id, ['class' => $this->css, 'title' => $this->disabledDelReason]);
					} else {
						$trash = we_html_button::create_button(we_html_button::TRASH, "javascript:" . $this->getJsSetHot() . ($this->extraDelFn ?: '') . ";we_cmd('" . $this->cmd_del . "','" . $id . "');");

						$table->setColAttributes($c, 0, ['class' => 'chooserFileIcon', 'data-contenttype' => (is_dir($id) ? $this->iconFolder : $this->iconFile)])
							->setCol($c, 1, (string) $id, ['class' => $this->css]);
					}
				} else {
					$trash = we_html_button::create_button(we_html_button::TRASH, "javascript:" . $this->getJsSetHot() . ($this->extraDelFn ?: "") . ";we_cmd('" . $this->cmd_del . "','" . $id . "');");

					$table->setColAttributes($c, 0, ['class' => 'chooserFileIcon', 'data-contenttype' => (is_dir($id) ? $this->iconFolder : $this->iconFile)])
						->setCol($c, 1, (string) $id, ['class' => $this->css]);
				}
			} else {
				$trash = '';

				$table->setColAttributes($c, 0, ['class' => 'chooserFileIcon', 'data-contenttype' => (is_dir($id) ? $this->iconFolder : $this->iconFile)])
					->setCol($c, 1, (string) $id, ['class' => $this->css]);
			}

			$table->setCol($c, 2, $edit . $trash, ['style' => 'text-align:right']);

			$c++;
		}
		$jsCmd->addCmd('setIconOfDocClass', 'chooserFileIcon');

		return we_html_element::div(['class' => 'multichooserDiv'],
															we_html_element::div(['class' => 'multichooser', 'id' => 'multi_selector'], $table->getHtml()
				) .
				($this->addbut ? we_html_element::div(['class' => 'button'], $this->addbut) : '')
		);
	}

}
