<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 5fd287473ea9 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-06-16 14:19 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//@phan-file-suppress PhanUndeclaredClassMethod,PhanUndeclaredMethod,PhanUndeclaredExtendedClass
use Sabre\DAV;

class we_davfs_dbDir extends DAV\Collection implements DAV\Sync\ISyncCollection{

	public const DOC_DIR = 'documents';
	public const TEMPLATE_DIR = 'template';
	protected const ALLOWED_TYPES = [
		we_base_ContentTypes::FOLDER_DOCUMENT,
		we_base_ContentTypes::FOLDER_TEMPLATE,
		we_base_ContentTypes::TEMPLATE,
		we_base_ContentTypes::HTML,
		we_base_ContentTypes::JS,
		we_base_ContentTypes::CSS,
		we_base_ContentTypes::TEXT,
		we_base_ContentTypes::XML,
	];

	/** @phan-var string */
	private $path;

	/** @phan-var int */
	private $ID = 0;

	/** @phan-var string */
	private $Table;

	/** @phan-var string */
	private $preFix;

	public function __construct(string $path, string $table, string $preFix, int $id){
		$this->path = $path;
		$this->Table = $table;
		$this->ID = $id ?: path_to_id($path, $this->Table);
		$this->preFix = $preFix;
	}

	function getChildren(){
		$all = DB_WE::$inst->getAllFirstq('SELECT ID,Text FROM ' . $this->Table . ' WHERE ContentType IN("' . implode('","', self::ALLOWED_TYPES) . '") AND ParentID=' . $this->ID, false);
		'@phan-var string[] $all';
		$children = [];
		// Loop through the directory, and create objects for each node
		foreach($all as $id => $node){
			$children[] = $this->getChild($node, (int) $id);
		}

		return $children;
	}

	/**
	 *
	 * @param string $name
	 * @return DAV\Node
	 * @throws DAV\Exception\NotFound
	 */
	function getChild($name, int $id = 0){
		'@phan-var string $name';
		$path = $this->path . '/' . $name;

		// We have to throw a NotFound exception if the file didn't exist

		if(!$this->childExists($name)){
			throw new DAV\Exception\NotFound('The file with name: ' . $name . ' could not be found');
		}

		// Some added security
		/* if($name[0] == '.'){
		  throw new DAV\Exception\NotFound('Access denied');
		  } */

		return fB('SELECT IsFolder FROM ' . $this->Table . ' WHERE ParentID=' . $this->ID . ' AND Text="' . DB_WE::$inst->escape($name) . '"') ?
				new we_davfs_dbDir($path, $this->Table, $this->preFix, $id) :
				($this->Table === FILE_TABLE ? new we_davfs_dbDocument($path, $this->Table, $id) :
				new we_davfs_dbTemplate($path, $this->Table, $id));
	}

	/**
	 * @param string $name
	 * @return bool
	 */
	function childExists($name){
		'@phan-var string $name';
		return fB('SELECT 1 FROM ' . $this->Table . ' WHERE ParentID=' . $this->ID . ' AND Text="' . DB_WE::$inst->escape($name) . '"');
	}

	/**
	 *
	 * @return string
	 */
	function getName(){
		return basename($this->preFix . $this->path);
	}

	/**
	 * @return int
	 */
	public function getLastModified(){
		return fI('SELECT UNIX_TIMESTAMP(Mod_Date) FROM ' . $this->Table . ' WHERE ID=' . $this->ID);
	}

	/**
	 * @return string|null
	 */
	public function getSyncToken(){
		return fS('SELECT MAX(Mod_Date) FROM ' . $this->Table . ' WHERE ParentID=' . $this->ID) ?: (new DateTime())->format('Y-m-d H:i:s');
	}

	/**
	 * The getChanges method returns all the changes that have happened, since
	 * the specified syncToken and the current collection.
	 *
	 * This function should return an array, such as the following:
	 *
	 * [
	 *   'syncToken' => 'The current synctoken',
	 *   'added'   => [
	 *      'new.txt',
	 *   ],
	 *   'modified'   => [
	 *      'modified.txt',
	 *   ],
	 *   'deleted' => array(
	 *      'foo.php.bak',
	 *      'old.txt'
	 *   )
	 * ];
	 *
	 * The syncToken property should reflect the *current* syncToken of the
	 * collection, as reported getSyncToken(). This is needed here too, to
	 * ensure the operation is atomic.
	 *
	 * If the syncToken is specified as null, this is an initial sync, and all
	 * members should be reported.
	 *
	 * The modified property is an array of nodenames that have changed since
	 * the last token.
	 *
	 * The deleted property is an array with nodenames, that have been deleted
	 * from collection.
	 *
	 * The second argument is basically the 'depth' of the report. If it's 1,
	 * you only have to report changes that happened only directly in immediate
	 * descendants. If it's 2, it should also include changes from the nodes
	 * below the child collections. (grandchildren)
	 *
	 * The third (optional) argument allows a client to specify how many
	 * results should be returned at most. If the limit is not specified, it
	 * should be treated as infinite.
	 *
	 * If the limit (infinite or not) is higher than you're willing to return,
	 * you should throw a Sabre\DAV\Exception\TooMuchMatches() exception.
	 *
	 * If the syncToken is expired (due to data cleanup) or unknown, you must
	 * return null.
	 *
	 * The limit is 'suggestive'. You are free to ignore it.
	 *
	 * @param string $syncToken
	 * @param int    $syncLevel @unused-param
	 * @param int    $limit @unused-param
	 *
	 * @return array
	 */
	public function getChanges($syncToken, $syncLevel, $limit = null){
		$sync = $syncToken ? '"' . DB_WE::$inst->escape($syncToken) . '"' : '';
//ignore level and limit
		return [
			'syncToken' => $this->getSyncToken(),
			'added' => DB_WE::$inst->getAllq('SELECT Text FROM ' . $this->Table . ' WHERE ContentType IN("' . implode('","', self::ALLOWED_TYPES) . '") AND ParentID=' . $this->ID . ($sync ? ' AND Creation_Date>' . $sync : ''), true),
			'modified' => DB_WE::$inst->getAllq('SELECT Text FROM ' . $this->Table . ' WHERE ContentType IN("' . implode('","', self::ALLOWED_TYPES) . '") AND ParentID=' . $this->ID . ($sync ? ' AND Mod_Date>' . $sync : ''), true),
			'deleted' => []//we don't support this for now
		];
	}

}
