<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 562fb8bcf3a8 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-06-30 12:28 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//@phan-file-suppress PhanUndeclaredClassMethod,PhanUndeclaredMethod,PhanUndeclaredExtendedClass
use Sabre\DAV;

class we_davfs_mainDir extends DAV\Collection{//FIXME: currently this is the same as _dir, but this should distinguish between different sources
	/** @phan-var string */
	private $path;

	private const DB = 'DB';
	private const GIT = '.git';
	private const HG = '.hg';
	private const SVN = '.svn';
	private const TYPE_DB = 'DB';
	private const TYPE_FILE = 'file';
	private const TYPE_PHP = 'php';
	private const DIRS = [
		we_davfs_dbDir::DOC_DIR => [self::TYPE_DB, ''],
		we_davfs_dbDir::TEMPLATE_DIR => [self::TYPE_DB, ''],
		'weTags' => [self::TYPE_PHP, WE_CUSTOM_PATH . 'tag'],
		'weTw' => [self::TYPE_PHP, WE_CUSTOM_PATH . 'tw'],
		'webEdition' => [self::TYPE_FILE, WEBEDITION_PATH],
		'weHooks' => [self::TYPE_PHP, WE_CUSTOM_PATH . 'hook'],
		'classes' => [self::TYPE_PHP, PHPCLASS_PATH],
		/* self::GIT => WE_CUSTOM_PATH . 'vcs/' . self::GIT,
		  self::SVN => WE_CUSTOM_PATH . 'vcs/' . self::SVN,
		  self::HG => WE_CUSTOM_PATH . 'vcs/' . self::HG */
	];

	//protected static $DIRS = self::DIRS;

	public function __construct(string $path){
		$this->path = $path;
	}

	/**
	 * @return DAV\Node[]
	 */
	function getChildren(){
		$ret = [];
		foreach(array_keys(self::DIRS) as $name){
			if(!$name || !$this->childExists($name)){
				continue;
			}
			try{
				$ret[] = $this->getChild($name);
			} catch (DAV\Exception\NotFound $e){
				//in case the user is not allowed
			}
		}
		return $ret;
	}

	/**
	 *
	 * @param string $name
	 * @return DAV\Node
	 * @throws DAV\Exception\NotFound
	 */
	function getChild($name){
		// We have to throw a NotFound exception if the file didn't exist
		if(!$this->childExists($name)){
			throw new DAV\Exception\NotFound('The file with name: ' . $name . ' could not be found');
		}

		switch($name){
			case we_davfs_dbDir::DOC_DIR:
				if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR, we_base_perm::WORK_TEMPLATES_CAN_SEE])){
					throw new DAV\Exception\NotFound('The file with name: ' . $name . ' could not be found');
				}
				return new we_davfs_dbDir('', FILE_TABLE, $name, 0);
			case we_davfs_dbDir::TEMPLATE_DIR:
				if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR, we_base_perm::WORK_TEMPLATES_CAN_SEE])){
					throw new DAV\Exception\NotFound('The file with name: ' . $name . ' could not be found');
				}
				return new we_davfs_dbDir('', TEMPLATES_TABLE, $name, 0);
			case 'webEdition':
				if(!we_base_WE::setDebugging()){
					throw new DAV\Exception\NotFound('The file with name: ' . $name . ' could not be found');
				}
			//no break;
			default:
				if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
					throw new DAV\Exception\NotFound('The file with name: ' . $name . ' could not be found');
				}
				[$type, $dir] = self::DIRS[$name];
				switch($type){
					case self::TYPE_FILE:
						return new DAV\FSExt\Directory($dir, $name);
					case self::TYPE_PHP:
						return new we_davfs_phpDir($dir, $name);
					default:
						throw new DAV\Exception\NotFound('Not implemented');
				}
		}
	}

	/**
	 *
	 * @param string $name
	 * @return bool
	 */
	function childExists($name){
		if(!array_key_exists($name, self::DIRS)){
			return false;
		}
		[$type, $path] = self::DIRS[$name];
		if($type === self::DB){
			return true;
		}
		return file_exists($path) && is_dir($path);
	}

	function getName(){
		return basename($this->path) ?: '';
	}

	/**
	 * Creates a new subdirectory.
	 *
	 * @param string $name
	 */
	public function createDirectory($name){
		throw new DAV\Exception\Forbidden('Permission denied to ' . $name);
		/* switch($name){
		  default:
		  case self::HG:
		  case self::GIT:
		  case self::SVN:
		  if(!mkdir(self::$DIRS[$name])){
		  throw new DAV\Exception\Forbidden('Permission denied');
		  }

		  clearstatcache(true, self::$DIRS[$name]);
		  } */
	}

}
