<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e6572d6d13ee $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-05-21 11:57 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//@phan-file-suppress PhanUndeclaredClassMethod,PhanUndeclaredMethod
use Sabre\DAV;

abstract class we_davfs_server{

	public static function start(){
//preload all possible functions, we don't have autoloading for
		include_once(WE_LIB_PATH . 'sabre/Uri/functions.php');
		include_once(WE_LIB_PATH . 'sabre/HTTP/functions.php');
		include_once(WE_LIB_PATH . 'sabre/Xml/Serializer/functions.php');
		include_once(WE_LIB_PATH . 'sabre/Xml/Deserializer/functions.php');
		include_once(WE_LIB_PATH . 'sabre/Event/Promise/functions.php');
		include_once(WE_LIB_PATH . 'sabre/Event/Loop/functions.php');

// The server object is responsible for making sense out of the WebDAV protocol
		$server = new DAV\Server(new we_davfs_mainDir('/'));

		$server->setBaseUri(WEBEDITION_DIR . 'davfs.php');
// The lock manager is reponsible for making sure users don't overwrite
// each others changes.
		$server->addPlugin(new DAV\Locks\Plugin(new DAV\Locks\Backend\File(TEMP_PATH . 'locks')));

		$auth = new we_davfs_auth();
		$auth->setRealm('Auth for Login');

		$server->addPlugin($authPl = (new DAV\Auth\Plugin($auth)));
		$server->addPlugin(new DAV\PartialUpdate\Plugin());
		$server->addPlugin(new we_davfs_tmpFile(TEMP_PATH));
		//sync feature
		$server->addPlugin(new DAV\Sync\Plugin());

// This ensures that we get a pretty index in the browser, but it is
// optional.
//		$server->addPlugin(new DAV\Browser\Plugin());
// All we need to do now, is to fire up the server
		$server->exec();
		if(($fail = $authPl->getLoginFailedReasons())){
			t_e('notice', 'DAV-FS failed: ', $fail);
		}
	}

}
