<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 74e5fc329da7 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-04-30 16:17 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_doclist_view extends we_search_view{
	/** @var we_doclist_model */
	var $Model;

	const DOCLIST_DOCUMENTS = 'DocumentsDoclist';
	const DOCLIST_TEMPLATES = 'TemplatesDoclist';
	const DOCLIST_OBJECTFILES = 'ObjectfilesDoclist';
	const DOCLIST_COLLECTION = 'CollectionsDoclist';

	public function __construct(we_doclist_model $model, $whichSearch = ''){
		switch($whichSearch){
			case self::DOCLIST_OBJECTFILES:
				$this->searchclass = new we_doclist_searchObjectfiles($this);
				break;
			default:
				$this->searchclass = new we_doclist_search($this);
		}

		parent::__construct($model);
	}

	/**
	 * create javascript for document list
	 * @return string javascript code
	 */
	public function getSearchJS(string $w = ''): string{
		// set separatae elems for doclist objectfiles!
		$whichSearch = $this->Model->whichSearch;

		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'search/search_view.js', '', ['id' => 'loadVarSearch_view', 'data-searchConf' => setDynamicVar([
					'conf' => [
						'whichSearch' => $whichSearch,
						'we_transaction' => $this->Model->transaction,
						'editorBody' => 'window',
						'rows' => (isset($_REQUEST['searchFields' . $whichSearch]) ? count((array) $_REQUEST['searchFields' . $whichSearch]) - 1 : 0),
						'tab' => 0,
						'modelID' => "placeholder",
						'modelIsFolder' => true,
						'modelHeight' => $this->Model->height,
					],
					'elems' => [
						'btnTrash' => we_html_button::create_button(we_html_button::TRASH, "javascript:weSearch.delRow(__we_new_id__)"),
						'btnSelector' => we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('__we_selector__', document.we_form.elements['search" . $whichSearch . "ParentID[__we_new_id__]'].value, '__we_sel_table__', 'search" . $whichSearch . "ParentID[__we_new_id__]', 'search" . $whichSearch . "[__we_new_id__]');"),
						'fieldSearch' => we_html_tools::textInput('search' . $whichSearch . '[__we_new_id__]', '', 0, ['id' => "search" . $whichSearch . '[__we_new_id__]'], 'text', '170px'),
						'fieldDate' => we_html_tools::textInput('search' . $whichSearch . '[__we_new_id__]', '', 0, ['id' => 'search' . $whichSearch . '[__we_new_id__]', 'required' => 'required'], 'date', '170px'),
						'selStatus' => we_html_tools::select('search' . $whichSearch . '[__we_new_id__]', $this->searchclass->getFieldsStatus(), 1, '', false, [
							'class' => "defaultfont", 'style' => "width:170px;", 'id' => 'search' . $whichSearch . "[__we_new_id__]"]),
						'selSpeicherart' => we_html_tools::select('search' . $whichSearch . '[__we_new_id__]', $this->searchclass->getFieldsSpeicherart(), 1, '', false, [
							'class' => "defaultfont", 'style' => "width:170px;", 'id' => 'search' . $whichSearch . "[__we_new_id__]"]),
						'selLocation' => we_html_tools::select('location' . $whichSearch . '[__we_new_id__]', we_search_search::getLocation(), 1, '', false, [
							'class' => "defaultfont", 'style' => 'width:150px;', 'id' => 'location' . $whichSearch . '[__we_new_id__]']),
						'selLocationDate' => we_html_tools::select('location' . $whichSearch . '[__we_new_id__]', we_search_search::getLocation('date'), 1, '', false, [
							'class' => "defaultfont", 'style' => 'width:150px;', 'id' => 'location' . $whichSearch . '[__we_new_id__]']),
						'selLocationText' => we_html_tools::select('location' . $whichSearch . '[__we_new_id__]', we_search_search::getLocation('text'), 1, '', false, [
							'class' => "defaultfont", 'style' => 'width:150px;', 'id' => 'location' . $whichSearch . '[__we_new_id__]']),
						'selModFields' => we_html_tools::select('search' . $whichSearch . '[__we_new_id__]', we_versions_search::getModFields(), 1, '', false, [
							'class' => "defaultfont", 'style' => "width:170px;", 'id' => 'search' . $whichSearch . '[__we_new_id__]']),
						'selUsers' => we_html_tools::select('search' . $whichSearch . '[__we_new_id__]', $this->searchclass->getUsers(), 1, '', false, ['class' => "defaultfont",
							'style' => "width:170px;", 'id' => 'search' . $whichSearch . "[__we_new_id__]"]),
						'searchFields' => we_html_tools::select('searchFields' . $whichSearch . '[__we_new_id__]', $this->searchclass->getFields($whichSearch, $this->Model->getPropertyInt('TableID')), 1, '', false, [
							'class' => "defaultfont", 'id' => 'searchFields' . $whichSearch . "[__we_new_id__]", 'onchange' => "weSearch.changeit(this.value, __we_new_id__);"]),
					],
					'result' => $this->getResultData($whichSearch),
					]
		)]);
	}

	/**
	 * create search dialog-box
	 * @return string html for search dialog box
	 */
	public function getSearchDialog(string $whichSearch = ''): string{ // TODO: use parent
		$currentSearchFields = $this->Model->getProperty('currentSearchFields');
		$currentSearch = $this->Model->getProperty('currentSearch');
		$currentLocation = $this->Model->getProperty('currentLocation');

		$out = we_contents_base::hiddenTrans() .
			'<table class="default"><tbody id="filterTable' . $whichSearch . '">';

		// we always have at least an empty 'Content' search: it's set in model initialisation
		for($i = 0; $i < $this->Model->height; $i++){
			$button = we_html_button::create_button(we_html_button::TRASH, "javascript:weSearch.delRow(" . $i . ");", '', '', false, false);
			$handle = '';
			$searchInput = we_html_tools::textInput('search' . $whichSearch . '[' . $i . ']', ($currentSearch[$i] ?? ''), 0, [
					'class' => 'wetextinput', 'id' => "search" . $whichSearch . "[" . $i . "]"], 'text', '170px');

			switch($currentSearchFields[$i] ?? ''){
				default:
					$locationDisabled = '';
					break;
				case 'Content':
				case 'Status':
				case 'Speicherart':
				case 'temp_template_id':
				case 'temp_category':
					$locationDisabled = 'disabled';
			}

			if(isset($currentSearchFields[$i])){
				switch($currentSearchFields[$i]){
					case 'Status':
						$searchInput = we_html_tools::select('search' . $whichSearch . '[' . $i . ']', $this->searchclass->getFieldsStatus(), 1, ($currentSearch[$i] ?? ''), false, [
								'class' => 'defaultfont', 'style' => 'width:170px;', 'id' => 'search' . $whichSearch . '[' . $i . ']']);
						break;
					case 'Speicherart':
						$searchInput = we_html_tools::select('search' . $whichSearch . '[' . $i . ']', $this->searchclass->getFieldsSpeicherart(), 1, ($currentSearch[$i] ?? ''), false, [
								'class' => 'defaultfont', 'style' => 'width:170px;', 'id' => 'search' . $whichSearch . '[' . $i . ']']);
						break;
					case 'Publish_Date':
					case 'Creation_Date':
					case 'Mod_Date':
						$handle = 'date';
						$searchInput = we_html_tools::getNewDateInput('search' . $whichSearch . '[' . $i . ']', new DateTime($currentSearch[$i] ?? ''), we_html_tools::FORMAT_DATE);
						break;
					case 'MasterTemplateID':
					case 'temp_template_id':
						$linkPath = ($currentSearch[$i] ?? '');
						$rootDirID = 0;

						$cmd = "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['search" . $whichSearch . "ParentID[" . $i . "]'].value,table:WE().consts.tables.TEMPLATES_TABLE,targetID:'search" . $whichSearch . 'ParentID[' . $i . "]',targetName:'search" . $whichSearch . '[' . $i . "]',rootDirID:'" . $rootDirID . "',filter:WE().consts.contentTypes.TEMPLATE})";
						$button = we_html_button::create_button(we_html_button::SELECT, $cmd);
						$selector = we_html_tools::formElementTable('', we_html_tools::textInput('search' . $whichSearch . '[' . $i . ']', $linkPath, 0, ['readonly' => 'readonly'], 'text', '170px'), we_html_element::hidden('search' . $whichSearch . 'ParentID[' . $i . ']', ''), $button);

						$searchInput = $selector;
						break;
					case 'temp_category':
						$linkPath = ($currentSearch[$i] ?? '');
						$rootDirID = 0;
						$cmd = "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['search" . $whichSearch . "ParentID[" . $i . "]'].value,table:WE().consts.tables.CATEGORY_TABLE,targetID:'search" . $whichSearch . "ParentID[" . $i . "]',targetName:'search" . $whichSearch . "[" . $i . "]',rootDirID:'" . $rootDirID . "'})";
						$button = we_html_button::create_button(we_html_button::SELECT, $cmd);
						$selector = we_html_tools::formElementTable('', we_html_tools::textInput('search' . $whichSearch . '[' . $i . ']', $linkPath, 0, ['readonly' => 'readonly'], 'text', '170px'), we_html_element::hidden('search' . $whichSearch . 'ParentID[' . $i . ']', ''), $button);

						$searchInput = $selector;
						break;
				}
			}
			$locationValue = $currentLocation[$i] ?? '';
			$out .= '
<tr id="filterRow_' . $i . '">
	<td>' . we_html_element::hidden('hidden_searchFields' . $whichSearch . '[' . $i . ']', $currentSearchFields[$i] ?? '') . we_html_tools::select('searchFields' . $whichSearch . '[' . $i . ']', $this->searchclass->getFields($whichSearch, $this->Model->getPropertyInt('TableID')), 1, ($currentSearchFields[$i] ?? ''), false, [
					'class' => "defaultfont", 'id' => 'searchFields' . $whichSearch . '[' . $i . ']', 'onchange' => "weSearch.changeit(this.value, " . $i . ");"]) . ' </td>
	<td id="td_location' . $whichSearch . '[' . $i . ']">' . we_html_tools::select('location' . $whichSearch . '[' . $i . ']', we_search_search::getLocation($handle), 1, ($currentLocation[$i] ?? ''), false, [
					'class' => "defaultfont", 'style' => 'width:150px', $locationDisabled => $locationDisabled, 'id' => 'location' . $whichSearch . '[' . $i . ']']) . ' </td>
	<td id="td_search' . $whichSearch . '[' . $i . ']">' . $searchInput . '</td>
	<td id="td_delButton[' . $i . ']">' . $button . '</td>
	<td id="td_hiddenLocation[' . $i . ']">' . (!$locationDisabled ? '' : we_html_element::hidden('location' . $whichSearch . '[' . $i . ']', $locationValue)) . '</td>
</tr>
		';
		}

		$out .= '</tbody></table>
' . we_html_button::create_button(we_html_button::ADD, "javascript:weSearch.newinput();") .
			we_html_button::create_button(we_html_button::SEARCH, "javascript:weSearch.search(true);");

		$filter = $this->getFilters();

		return (new we_html_multiIconBox((new we_html_multiIconBoxContent())
					->add(new we_html_multiIconBoxContentLine('', $out))
					->add($filter ? new we_html_multiIconBoxContentLine(g_l('searchtool', '[filter]'), $this->getFilters()) : null)
			))->fold(1, false /* (bool) $this->Model->mode */, g_l('searchtool', '[suchen]'))->get();
	}

	protected function getFilters(): string{
		switch(($whichSearch = $this->Model->whichSearch)){
			case self::DOCLIST_OBJECTFILES:
				break;
			default:
				return '';
		}

		$textname = 'WorkspacePath' . $whichSearch;
		$idname = 'WorkspaceID' . $whichSearch;
		$cmd = ''; //'reload_editpage'; => startSearch!
		$pathID = $this->Model->getPropertyInt('currentWorkspace');
		$path = id2path($pathID);
		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements['" . $idname . "'].value,table:'" . FILE_TABLE . "',targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'" . $cmd . "',rootDirID:0})");
		$htmlWorkspace = we_html_tools::formElementTable('', we_html_tools::textInput($textname, $path, 0, ['readonly' => 'readonly'], 'text'), we_html_element::hidden($idname, $pathID), $button);

		return g_l('modules_objectClassfoldersearch', '[Verzeichnis]') . we_html_element::br() .
			$htmlWorkspace . we_html_element::br() .
			we_html_forms::checkboxWithHidden($this->Model->getPropertyB('currentGreenOnly'), 'GreenOnly' . $this->Model->whichSearch, g_l('modules_objectClassfoldersearch', '[sicht]'), false, '', "weSearch.toggleShowVisible(this.checked);")
		;
	}

	public function getHtmlEditor(): string{
		$whichSearch = $this->Model->whichSearch;
		$foundItems = $_SESSION['weS']['search']['foundItems'] ?? 0;
		return (new we_html_multiIconBox((new we_html_multiIconBoxContent())
					->add(new we_html_multiIconBoxContentLine('', $this->getSearchDialog($whichSearch), we_html_multiIconBox::PROP_SEARCH))
					->add(new we_html_multiIconBoxContentLine('',
																							 we_html_element::div(['id' => 'mouseOverDivs_' . $whichSearch]) .
							we_html_element::div(['id' => 'parametersTop_' . $whichSearch], $this->getSearchParameterTop($foundItems, $whichSearch)) .
							$this->makeResultContainer($whichSearch) .
							we_html_element::div(['id' => 'parametersBottom_' . $whichSearch], $this->getSearchParameterBottom($foundItems, $whichSearch)) .
							we_html_element::hiddens(['obj' => '1'])
							, we_html_multiIconBox::PROP_RESULT))
			))->marginLeft()->get();
	}

}
