<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: c0e367dcb733 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-03-08 22:03 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_editor_unusedElements extends we_editor_base{

	private function doRemove(): void{
		$db = DB_WE::$inst;

//$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', we_base_request::_(we_base_request::TRANSACTION, 'we_transaction'), 2);
		$remove = we_base_request::intList('weg');
		if(we_base_request::string('do') == 'delete' && !empty($remove)){
			$delS = $delB = [];
			foreach($remove as $rem => $blockcnt){
				if($blockcnt){
					$delB[] = $rem;
				} else {
					$delS[] = $rem;
				}
			}

			if($delS){
				$db->query('DELETE FROM ' . CONTENT_TABLE . ' WHERE DocumentTable="tblFile" AND Type!="attrib" AND DID IN (SELECT ID FROM ' . FILE_TABLE . ' WHERE TemplateID=' . $this->we_doc->ID . ') AND nHash IN (x\'' . implode('\',x\'', $delS) . '\')'
				);
			}
			if($delB){
				$strs = $db->getAllq('SELECT DISTINCT SUBSTRING_INDEX(c.Name,"__",1) FROM ' . CONTENT_TABLE . ' c WHERE c.DID IN (SELECT ID FROM ' . FILE_TABLE . ' WHERE TemplateID=' . $this->we_doc->ID . ') AND c.DocumentTable="tblFile" AND c.Type!="attrib" AND c.nHash IN (x\'' . implode('\',x\'', $delB) . '\')', true);
				$db->query('DELETE FROM ' . CONTENT_TABLE . ' WHERE DocumentTable="tblFile" AND Type!="attrib" AND DID IN (SELECT ID FROM ' . FILE_TABLE . ' WHERE TemplateID=' . $this->we_doc->ID . ') AND SUBSTRING_INDEX(Name,"__",1) IN ("' . implode('","', $strs) . '")'
				);
			}
		}
	}

	public function show(): string{
		assert($this->we_doc instanceof we_contents_template);
		$this->doRemove();
		$db = DB_WE::$inst;

		$tp = new we_weTag_parser($this->we_doc->getTemplateCode(true));
		$relevantTags = [
			'normal' => [],
			'block' => [],
		];
		'@phan-var array<string,array<string,string>> $relevantTags';
//FIXME: we need to get the names of blocks
		foreach($tp->getTagsWithAttributes(true) as $tag){
			if(empty($tag['attribs']['name'])){
				continue;
			}
			$attrName = $tag['attribs']['name'];
			$isBlock = !empty($tag['attribs']['weblock']);
			$type = !$isBlock ? 'normal' : 'block';
			$name = $attrName . ($isBlock ? implode('', $tag['attribs']['weblock']) : '');
			$nHash = $isBlock ? $name : md5($name);

			if(isset($relevantTags[$type][$nHash])){
				continue;
			}
			$relevantTags[$type][$nHash] = $attrName;
			if($isBlock){
				continue;
			}
			switch($tag['name']){
				case 'img':
					$relevantTags[$type][md5($attrName . we_document_image::ALT_FIELD)] = $attrName;
					$relevantTags[$type][md5($attrName . we_document_image::THUMB_FIELD)] = $attrName;
					$relevantTags[$type][md5($attrName . we_document_image::TITLE_FIELD)] = $attrName;
					break;
				case 'href':
					$relevantTags[$type][md5($attrName . we_base_link::MAGIC_INT_LINK)] = $attrName;
					$relevantTags[$type][md5($attrName . we_base_link::MAGIC_INT_LINK_ID)] = $attrName;
					$relevantTags[$type][md5($attrName . we_base_link::MAGIC_INT_LINK_EXTPATH)] = $attrName;
			}
		}

		if(!empty($relevantTags['normal']) || !empty($relevantTags['block'])){
			$obsolete = $db->getAllq('SELECT
	c.Type,c.Name,HEX(c.nHash) AS nHash,COUNT(1) AS no,IF(c.BDID,c.BDID, SUBSTR(c.Dat,1,150)) AS content
FROM
	' . CONTENT_TABLE . ' c JOIN
	' . FILE_TABLE . ' f ON c.ID=f.DID
WHERE
	f.TemplateID=' . $this->we_doc->ID . ' AND
	c.DocumentTable="tblFile" AND
	c.Type!="attrib" AND
	c.nHash NOT IN (x\'' . md5(we_contents_root::TITLE_FIELD) . '\',x\'' . md5(we_contents_root::DESCRIPTION_FIELD) . '\',x\'' . md5(we_contents_root::KEYWORD_FIELD) . '\') ' .
				(empty($relevantTags['normal']) ? '' : 'AND c.nHash NOT IN (x\'' . implode('\',x\'', array_keys($relevantTags['normal'])) . '\') ') .
				(empty($relevantTags['block']) ? '' : ' AND SUBSTRING_INDEX(c.Name,"__",1) NOT IN ("' . implode('","', array_keys($relevantTags['block'])) . '")') . '
GROUP BY
	c.nHash
ORDER BY
	c.Name');
			'@phan-var array<int,array{Type:string,Name:string,nHash:string,no:int,content:string,real:string,block:string,blockcnt:int}> $obsolete';

			foreach($obsolete as &$ob){
				$bl = explode('blk_', $ob['Name'], 2);
				$cnt = 0;
				$ob['real'] = $bl[0];
				$ob['block'] = isset($bl[1]) ? preg_replace('|(__\d+)+|', '', str_replace('blk_', ' -> ', $bl[1], $cnt)) : '';
				$ob['blockcnt'] = isset($bl[1]) ? $cnt + 1 : $cnt;
				$ob['content'] = htmlspecialchars((string) $ob['content']);
			}
			usort($obsolete, function(array $a, array $b): int{
				return $a['block'] === $b['block'] ?
					strcmp($a['real'], $b['real']) :
					strcmp($a['block'], $b['block']);
			});
		} else {
			$obsolete = [];
		}

		$table = (new we_html_table(['class' => 'dialogBorderBox default middlefont', 'width' => '100%'], count($obsolete) + 1, 6))
			->addHeaderRow([
			'',
			g_l('weEditor', '[unusedElements][block]'),
			g_l('weEditor', '[unusedElements][Name]'),
			g_l('weEditor', '[unusedElements][type]'),
			g_l('weEditor', '[unusedElements][count]'),
			g_l('weEditor', '[unusedElements][content]')
		]);

		foreach($obsolete as $cur){
			$table
				->addRow(new we_html_tr([
						new we_html_td('<input type="checkbox" name="weg[' . $cur['nHash'] . ']" value="' . $cur['blockcnt'] . '"/>'),
						new we_html_td($cur['block']),
						new we_html_td((string) $cur['real']),
						new we_html_td($cur['Type']),
						new we_html_td((string) $cur['no']),
						new we_html_td($cur['content']),
			]));
		}

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine(g_l('weClass', '[unusedElementsTab]'), we_html_tools::alertAttentionBox(g_l('weClass', '[unusedElements][description]', false, true), we_html_tools::TYPE_ALERT, 850, false)))
			->add(new we_html_multiIconBoxContentLine('', $table->getHtml() .
				($obsolete ? we_html_button::create_button(we_html_button::TRASH, "javascript:WE().util.showConfirm(window, '','" . strtr(g_l('weClass', '[unusedElements][delete]'), ["\n" => '\n']) . "',['setDoReload','delete']);") : '')));

		return $this->getPage((new we_html_multiIconBox($parts))->marginLeft()->get() .
				we_html_element::hiddens([
					'do' => ''
				]), we_html_element::jsScript(JS_DIR . 'we_editor_script.js'));
	}

}
