<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 9afa1aa008a1 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-05 00:07 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_exim_ExportWE extends we_exim_Export{

	function __construct(){
		parent::__construct();
		$this->exportType = we_exim_ExIm::TYPE_WE;
	}

	protected function writeExportItem(we_base_mainmodel $doc, array $attribute = []): void{
		assert(is_resource($this->fh));
		if($doc instanceof we_backup_binary){
			$doc->binary2file($this->fh);
		} else {
			fwrite($this->fh, $doc->toXML($attribute));
		}

		fwrite($this->fh, we_backup_util::backupMarker . "\n");
	}

	public function prepareExport(array $ids = []): void{ // check access level of parents
		$this->savePreserves();

		$preparer = new we_export_preparer();
		$preparer->loadPreserves();
		$preparer->prepareExport();
		$preparer->savePreserves();

		$this->loadPreserves();
		parent::prepareExport();
	}

	protected function fileCreate(): bool{
		return (parent::fileCreate() ?
			we_base_file::save($this->exportProperties['file'], we_exim_ExIm::getHeader(we_exim_ExIm::TYPE_WE)) :
			false);
	}

	protected function fileComplete(): void{
		we_base_file::save($this->exportProperties['file'], we_exim_ExIm::getFooter(), 'ab');
	}

	protected function formatOutput(string $content, string $tagname = '', string $format = we_exim_ExIm::TYPE_XML, int $tabs = 2, bool $cdata = false, bool $fix_content = false): string{
		return $content;
	}

}
