<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f2413f100800 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-01-29 10:30 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/**
 * General Definition of WebEdition Export
 *
 */
we_modules_info::isActive(we_modules_info::EXPORT);

class we_export_export extends we_base_model{
	const TYPE_DOCTYPE = 'doctype';
	const TYPE_OBJECT = 'classname';
	const TYPE_DOCUMENT = 'document';

	//properties
	/** @var int */
	var $ID;

	/** @var string */
	var $Text;

	/** @var int */
	var $ParentID = 0;

	/** @var bool */
	var $IsFolder;

	/** @var string */
	var $Path;

	/** @var string */
	var $ExportTo; // local | server

	/** @var string */
	var $ServerPath = '';

	/** @var string */
	var $Filename;

	/** @var string */
	var $Selection = 'auto'; // auto | manual

	/** @var string */
	var $SelectionType = self::TYPE_DOCTYPE; // doctype | classname

	/** @var int */
	var $DocType = 0;

	/** @var int */
	var $Folder = 0;

	/** @var int */
	var $TableID = 0;

	/** @var int[] */
	var $Categorys = [];

	/** @var int[] */
	var $selDocs = [];

	/** @var int[] */
	var $selTempl = [];

	/** @var int[] */
	var $selObjs = [];

	/** @var int[] */
	var $selClasses = [];

	/** @var bool */
	var $HandleDefTemplates = false;

	/** @var bool */
	var $HandleDocIncludes = false;

	/** @var bool */
	var $HandleObjIncludes = false;

	/** @var bool */
	var $HandleDocLinked = false;

	/** @var bool */
	var $HandleDefClasses = false;

	/** @var bool */
	var $HandleObjEmbeds = false;

	/** @var bool */
	var $HandleDoctypes = false;

	/** @var bool */
	var $HandleCategorys = false;

	/** @var bool */
	var $HandleOwners = false;

	/** @var bool */
	var $HandleNavigation = false;

	/** @var bool */
	var $HandleThumbnails = false;

	/** @var int */
	var $ExportDepth = 5;

	/** @var string[] */
	var $Log = [];

	/** @var string */
	var $ExportFilename;

	/** @var string */
	public $ExportType = we_exim_ExIm::TYPE_WE;

	/** @var bool */
	public $XMLCdata = true;

	/** @var string */
	public $XMLTable = 'documents';

	/** @var string */
	public $CSVDelimiter = 'semicolon';

	/** @var string */
	public $CSVLineend = 'windows';

	/** @var string */
	public $CSVEnclose = 'doublequote';

	/** @var bool */
	public $CSVFieldnames = false;

	//var $protected = ['ID', 'ParentID', 'IsFolder', 'Path', 'Text'];

	/**
	 * Default Constructor
	 * Can load or create new Newsletter depends of parameter
	 */
	function __construct(int $exportID = 0){
		parent::__construct(EXPORT_TABLE);
		$this->persistent_slots = [
			'ID' => we_base_request::INT,
			'ParentID' => we_base_request::INT,
			'Text' => we_base_request::STRING,
			'IsFolder' => we_base_request::BOOL,
			'Path' => we_base_request::STRING,
			'ExportType' => we_base_request::STRING,
			'ExportTo' => we_base_request::STRING,
			'ServerPath' => we_base_request::STRING,
			'Filename' => we_base_request::STRING,
			'Selection' => we_base_request::STRING,
			'SelectionType' => we_base_request::STRING,
			'DocType' => we_base_request::INT,
			'Folder' => we_base_request::INT,
			'TableID' => we_base_request::INT,
			'Categorys' => we_base_request::INTLISTA,
			'selDocs' => we_base_request::INTLISTA,
			'selTempl' => we_base_request::INTLISTA,
			'selObjs' => we_base_request::INTLISTA,
			'selClasses' => we_base_request::INTLISTA,
			'HandleDefTemplates' => we_base_request::BOOL,
			'HandleDocIncludes' => we_base_request::BOOL,
			'HandleObjIncludes' => we_base_request::BOOL,
			'HandleDocLinked' => we_base_request::BOOL,
			'HandleDefClasses' => we_base_request::BOOL,
			'HandleObjEmbeds' => we_base_request::BOOL,
			'HandleDoctypes' => we_base_request::BOOL,
			'HandleCategorys' => we_base_request::BOOL,
			'ExportDepth' => we_base_request::BOOL,
			'HandleOwners' => we_base_request::BOOL,
			'HandleNavigation' => we_base_request::BOOL,
			'HandleThumbnails' => we_base_request::BOOL,
			'XMLCdata' => we_base_request::BOOL,
			'CSVDelimiter' => we_base_request::STRING,
			'CSVLineend' => we_base_request::STRING,
			'CSVEnclose' => we_base_request::STRING,
			'CSVFieldnames' => we_base_request::BOOL,
		];
		$this->Filename = $this->Text = 'weExport_' . date('Y_m_d_H_i');

		if($exportID){
			$this->ID = $exportID;
			$this->load($exportID);
		}

		// clear expiered stuff
		$this->selDocs = $this->clearExpired($this->selDocs, FILE_TABLE);
		$this->selTempl = $this->clearExpired($this->selTempl, TEMPLATES_TABLE);
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$this->selObjs = $this->clearExpired($this->selObjs, OBJECT_FILES_TABLE);
			$this->selClasses = $this->clearExpired($this->selClasses, OBJECT_TABLE);
		} else {
			$this->selObjs = [];
			$this->selClasses = [];
		}
	}

	private function clearExpired(array $ids, string $table): array{
		return $ids ? $this->db->getAllq('SELECT ID FROM ' . $table . ' WHERE ID IN(' . implode(',', $ids) . ')', true) : [];
	}

	function delete(): bool{
		//if (!$this->ID) return false;
		if($this->IsFolder){
			$this->deleteChilds();
		}
		parent::delete();
		return true;
	}

	function clearSessionVars(): void{
		if(isset($_SESSION['weS']['module_sessions']['export'])){
			unset($_SESSION['weS']['module_sessions']['export']);
		}
		if(isset($_SESSION['weS']['module_sessions']['exportVars'])){
			unset($_SESSION['weS']['module_sessions']['exportVars']);
		}
	}

	static function filenameNotValid(string $text): bool{
		return preg_match('%[^a-z0-9äöü\._\@\ \-]%i', $text) === 1;
	}

	static function exportToFilenameInvalid(string $filename): bool{//FIXME: what should that do?!
		return (preg_match('%p?html?%i', $filename) || stripos($filename, 'inc') !== false || preg_match('%php3?%i', $filename));
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'delete_group_question' => g_l('export', '[delete_group_question]'),
				'delete_question' => g_l('export', '[delete_question]'),
				'must_save' => (g_l('export', '[must_save]')),
				'name_empty' => (g_l('export', '[name_empty]')),
				'no_perms' => (g_l('export', '[no_perms]')),
				'nothing_to_delete' => (g_l('export', '[nothing_to_delete]')),
				'nothing_to_save' => (g_l('export', '[nothing_to_save]')),
				'save_changed_export' => g_l('export', '[save_changed_export]'),
				'server_finished' => g_l('export', '[server_finished]'),
		]);
	}

}
