<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 8c19f7a2383f $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-12-22 13:26 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_folder_template extends we_folder_base{

	public function __construct($id = 0){
		$this->Table = TEMPLATES_TABLE;
		$this->ContentType = we_base_ContentTypes::FOLDER_TEMPLATE;
		parent::__construct($id);
	}

	protected function hasCopyPerm(): bool{
		return false;
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		if($this->checkMovePathExists()){
			return false;
		}
		return parent::we_save($resave, $skipHook);
	}

	/**
	 * Create the folders on the server in the public and the site directory.
	 * Note: for internal use
	 *
	 * @return boolean true on success, false in case of failure
	 */
	protected function saveToServer(): bool{
		$path = $this->getPath();
		if(!we_base_file::createLocalFolderByPath(TEMPLATES_PATH . $path)){
			return false;
		}
		return true;
	}

	protected function moveAtServer(): bool{
		// renames the folder on the local machine in the root-dir
		$path = $this->getRealPath();
		$oldPath = $this->getRealPath(true);
		if(!file_exists($path) && !file_exists($oldPath)){
			t_e('old path doesn\'t exist', $oldPath);
			return false;
		}
		if(!file_exists($path) && file_exists($oldPath)){
			if(!rename($oldPath, $path)){
				return false;
			}
		}
		return true;
	}

	/**
	 * Beseitigt #Bug 3705: sorgt dafür, das auch leere Dokumentenordner bei einem REbuild angelegt werden
	 */
	public function we_rewrite(): bool{
		return (parent::we_rewrite() && $this->we_save(true));
	}

}
