<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3b8044c13b9c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-12-16 00:05 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_glossary_frames extends we_modules_frame{
	/** @var we_glossary_view */
	protected $View;

	/** @var we_glossary_tree */
	protected $Tree;

	public function __construct(){
		$this->module = we_modules_info::GLOSSARY;
		parent::__construct();

		$this->Tree = new we_glossary_tree($this->jsCmd);
		$this->View = new we_glossary_view();
	}

	public function getFrameset(): string{
		return $this->frameset;
	}

	public function getHTML(string $what = '', int $mode = 0, int $step = 0): string{
		switch($what){
			case 'settings':
				return $this->getSettingsDialog();
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	private function getSettingsDialog(): string{
		// Automatic Replacement
		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', we_html_forms::checkboxWithHidden(we_glossary_replace::useAutomatic(), 'GlossaryAutomaticReplacement', g_l('modules_glossary', '[enable_replacement]'))))
			->noline());

		$content = '<form name="' . we_base_constants::WE_FORM . '" target="cmdFrame" action="' . WEBEDITION_DIR . 'we_cmd.php">' .
			we_html_element::hiddens(['we_cmd[0]' => 'we_showMod',
				'mod' => 'glossary',
				'pnt' => 'settings',
				'cmd' => 'save_glossary_setting']) .
				(new we_html_multiIconBox($parts, 'GlossaryPreferences'))
				->marginLeft()
				->buttons(we_html_button::position_yes_no_cancel(
						we_html_button::create_button(we_html_button::SAVE, "javascript:document.we_form.submit();"), '', we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();")
				))
				->headline(g_l('modules_glossary', '[menu_settings]'))
				->get() .
			'</form>';

		return
			we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'formFunctions.js') . $this->jsCmd->getCmds(), we_html_element::body(['class' => 'weDialogBody']
					, we_html_element::exIFrame('content', $content, 'position:absolute;top:0;bottom:1px;left:0;right:0;overflow: hidden;') .
					we_html_element::iframe('cmdFrame', 'about:blank', 'position:absolute;height:1px;bottom:0;left:0;right:0;overflow: hidden;')
		));
	}

	protected function getHTMLEditorHeader(): string{
		if(we_base_request::bool('home')){
			//FIXME: remove
			return parent::getHTMLEditorHeader();
		}
		$cmdid = we_base_request::string('cmdid');
		if($cmdid && !is_numeric($cmdid)){
			$this->View->Model->Language = substr($cmdid, 0, 5);
		}
		switch(we_base_request::string('cmd')){
			// Folder View
			case 'we_glossary_view_folder':
				return we_glossary_frameEditorFolder::Header($this);
			// Type View
			case 'we_glossary_view_type':
				return we_glossary_frameEditorType::Header($this);
			// Exception View
			case 'we_glossary_view_exception':
			case 'we_glossary_save_exception':
				return we_glossary_frameEditorException::Header($this);
			// Item View
			default:
				return we_glossary_frameEditorItem::Header($this, $this->View);
		}
	}

	protected function getHTMLEditorBody(): string{
		if(we_base_request::bool('home')){
			return $this->View->getHomeScreen();
		}
		$cmdid = we_base_request::string('cmdid');
		if($cmdid && !is_numeric($cmdid)){
			$this->View->Model->Language = substr($cmdid, 0, 5);
		}
		switch(we_base_request::string('cmd')){
			// Folder View
			case 'we_glossary_view_folder':
				return we_glossary_frameEditorFolder::Body($this, $this->View);
			// Type View
			case 'we_glossary_view_type':
				return we_glossary_frameEditorType::Body($this, $this->View, $this->frameset);
			// Exception View
			case 'we_glossary_view_exception':
			case 'we_glossary_save_exception':
				return we_glossary_frameEditorException::Body($this, $this->View);
			// Item View
			default:
				return we_glossary_frameEditorItem::Body($this, $this->View);
		}
	}

	protected function getHTMLEditorFooter(array $btn_cmd = []): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorFooter([]);
		}
		$cmdid = we_base_request::string('cmdid');
		if($cmdid && !is_numeric($cmdid)){
			$this->View->Model->Language = substr($cmdid, 0, 5);
		}
		switch(we_base_request::string('cmd')){
			// Folder View
			case 'we_glossary_view_folder':
				return parent::getHTMLEditorFooter([]);
			// Type View
			case 'we_glossary_view_type':
				return parent::getHTMLEditorFooter([]);
			// Exception View
			case 'we_glossary_view_exception':
			case 'we_glossary_save_exception':
				return we_glossary_frameEditorException::Footer($this, $this->View);
			// Item View
			default:
				return we_glossary_frameEditorItem::Footer($this, $this->View);
		}
	}

	protected function getHTMLCmd(): string{
		if(($pid = we_base_request::int('pid', we_base_request::NOT_VALID_I)) === we_base_request::NOT_VALID_I){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$offset = we_base_request::int('offset', 0);
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_glossary_tree::getItems($pid, $offset, $this->Tree->default_segment), 'sorted' => true]);

		return $this->getHTMLDocument(
				we_html_element::body([], we_html_element::form(['name' => we_base_constants::WE_FORM], we_html_element::hiddens([
							'pnt' => 'cmd',
							'cmd' => 'no_cmd'])
					)
				)
		);
	}

}
