<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e4599ed4a0cf $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-03-15 18:51 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class
 * @desc    class for tag <we:listview>
 *
 */
class we_listview_langlink extends we_listview_base{
	const LANG_FIELD = self::PROPPREFIX . 'lang';
	const DOC_LOCALE_FIELD = self::PROPPREFIX . 'documentlocale';
	const DOC_COUNTRY_FIELD = self::PROPPREFIX . 'documentcountry';
	const DOC_LANG_FIELD = self::PROPPREFIX . 'documnetlanguage';
	const TRG_LOCALE_FIELD = self::PROPPREFIX . 'targetlocale';
	const TRG_COUNTRY_FIELD = self::PROPPREFIX . 'targetcountry';
	const TRG_LANG_FIELD = self::PROPPREFIX . 'targetlanguage';
	const TRG_LANG_NAME_FIELD = self::PROPPREFIX . 'targetlanguage_name';
	const TRG_COUNTRYNAME_FIELD = self::PROPPREFIX . 'targetcountry_name';

	/** @var array<string,array{Locale:string,DID:int,DLocale:string,LDID:int,DocumentTable:string,Path:string,ParentID?:int,Url?:string,TriggerID?:int}> */
	private $foundlinks = [];

	/** @var bool */
	private $objectseourls = false;

	/** @var string */
	private $pagelanguage = ''; //assumed pagelanguage: choosen by using attribut pagelanguage

	/** @var string */
	private $ownlanguage = ''; // effective language of document/object

	/** @var string */
	private $dirsearchtable = FILE_TABLE;

	/** @var bool */
	private $showself = false;

	/** @var bool */
	private $recursive = true; // #7023: search for default corresponding language link-up in parent folders

	/**
	 * ()
	 * constructor of class
	 *
	 * @param   string $name          name of listview
	 * @param   int $rows          number of rows to display per page
	 * @param   int $offset        start offset of first page
	 * @param   string $order         field name(s) to order by
	 * @param   bool $desc           set to true, if order should be descendend
	 * @param   int $cols   		  to display a table this is the number of cols
	 */
	public function __construct(string $name, int $rows, int $offset, string $order, bool $desc, string $dirsearchtable, int $cols, bool $showself, array $id, string $pagelanguage, string $ownlanguage, bool $hidedirindex, bool $objectseourls, bool $recursive){
		parent::__construct($name, $rows, $offset, $order, $desc, [], false, [], $cols, '', '', '', '', [], 'off', $id);

		$this->showself = $showself;
		$this->recursive = $recursive;
		$this->objectseourls = $objectseourls;
		$this->hidedirindex = $hidedirindex;
		$this->id = array_filter($id);
		$this->pagelanguage = $pagelanguage;
		$this->ownlanguage = $ownlanguage;
		$this->dirsearchtable = $dirsearchtable;
		$languages = we_base_country::getWeFrontendLanguagesForBackend();

		// if !$showself: remove the pagelanguage (choosen by using attribute pagelanguage) from the languages-list, so the link to this language won't be found in DB
		if(isset($languages[$this->pagelanguage]) && !$showself){
			unset($languages[$this->pagelanguage]);
		}

		if(str_contains(strtolower($this->order), ' desc')){//was #3849
			$this->order = str_ireplace(' desc', '', $this->order);
			$this->desc = true;
		}

		$this->order = trim($this->order);

		if(!$this->id || !in_array($this->dirsearchtable, [FILE_TABLE, OBJECT_FILES_TABLE], true)){
			$this->anz_all = 0;
			$this->anz = $this->anz_all;
			$this->count = 0;
			return;
		}
		$myID = $id[0];

		$langs = array_keys($languages);

		$this->foundlinks = $this->DB_WE->getAllFirstq('SELECT
	ll.Locale AS frst,
	ll.Locale,
	ll.DID,
	src.Language AS DLocale,
	ll.LDID,
	ll.DocumentTable,
	st.Path' .
			($this->dirsearchtable === FILE_TABLE ? ',st.ParentID ' : ',st.Url,st.TriggerID ') . '
FROM ' .
			LANGLINK_TABLE . ' ll JOIN
			' . $this->DB_WE->escape($this->dirsearchtable) . ' st ON ll.LDID=st.ID JOIN
			' . $this->DB_WE->escape($this->dirsearchtable) . ' src ON ll.DID=src.ID
WHERE
	ll.Locale IN ("' . implode('","', array_map('escape_sql_query', $langs)) . '") AND
	st.IsPublished=1 AND
	ll.DocumentTable="' . stripTblPrefix($this->dirsearchtable) . '" AND
	ll.DID=' . $myID, true, MYSQLI_ASSOC);

		if($this->recursive && ($diff = array_diff($langs, array_keys($this->foundlinks)))){
			foreach($diff as $langkey){
				$this->getParentData($myID, (string) $langkey);
			}
		}

		// Links to documents/objects themselves are not listet in tblLangLink.
		// In the following cases we must create them manually:
		// if($this->showself == true)
		// if($this->showself == false && $this->pagelanguage != $this->ownlanguage)
		if($this->showself || (!$this->showself && $this->pagelanguage != $this->ownlanguage)){
			$dt = [
				'DID' => $myID,
				'DLocale' => $this->ownlanguage,
				'LDID' => $myID,
				'Locale' => $this->ownlanguage,
				'DocumentTable' => stripTblPrefix($this->dirsearchtable),
				'IsObject' => ($this->dirsearchtable === OBJECT_FILES_TABLE ? 0 : 1),
			];
			switch($this->dirsearchtable){
				case FILE_TABLE:
					$dt['Path'] = id2path($myID, FILE_TABLE);
					break;
				case OBJECT_FILES_TABLE:
					$dt['Path'] = id2path($myID, OBJECT_FILES_TABLE);
					$hash = getHash('SELECT Url,TriggerID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $myID);
					$dt['Url'] = $hash['Url'];
					$dt['TriggerID'] = $hash['TriggerID'];
			}
			$this->foundlinks[$this->ownlanguage] = $dt;
		}

		// sort array
		switch($this->order){
			case self::RANDOM_FIELD:
				shuffle($this->foundlinks);
				break;
			case 'Locale':
				if($this->desc){
					krsort($this->foundlinks);
				} else {
					ksort($this->foundlinks);
				}
				break;
			default:
				if(strpos($this->order, '_') > 0){ //csv: ordered list of locales
					$orderArr = explode(',', trim($this->order));
					$orderedLinks = [];
					foreach($orderArr as $orderLocale){
						if(isset($this->foundlinks[$orderLocale])){
							$orderedLinks[$orderLocale] = $this->foundlinks[$orderLocale];
							unset($this->foundlinks[$orderLocale]);
						}
					}
					if($this->desc){
						krsort($this->foundlinks);
					} else {
						ksort($this->foundlinks);
					}
					$this->foundlinks = array_merge($orderedLinks, $this->foundlinks);
				}
		}

		// to go on with $this->foundlinks it must not be associative!
		$this->foundlinks = array_values($this->foundlinks);

		$this->anz_all = count($this->foundlinks);
		$this->anz = $this->anz_all;
		$this->count = 0;
	}

	// Links to documents/objects themselves never use this method: so we do not need to fix the showself-problem in this section
	private function getParentData(int $myid, string $langkey): void{
		$pid = fI('SELECT ParentID FROM ' . $this->dirsearchtable . ' WHERE ID=' . $myid);

		if(!$pid){
			return;
		}

		$data = getHash('SELECT
	ll.DID,
	src.Language AS DLocale,
	ll.LDID,
	ll.Locale,
	ll.DocumentTable,
	st.Path as Path' .
			($this->dirsearchtable === FILE_TABLE ? ',st.ParentID ' : ',st.Url,st.TriggerID ') . '
FROM ' .
			LANGLINK_TABLE . ' ll JOIN
			' . $this->dirsearchtable . ' st ON ll.LDID=st.ID JOIN
			' . $this->DB_WE->escape($this->dirsearchtable) . ' src ON ll.DID=src.ID
WHERE
	ll.Locale="' . $langkey . '" AND
	st.IsPublished=1 AND
	ll.DocumentTable="' . stripTblPrefix($this->dirsearchtable) . '" AND
	src.ID=' . $pid . ' AND
	ll.DID=' . $pid
		);
		if($data){
			$this->foundlinks[$data['Locale']] = $data;
		} else {
			$this->getParentData($pid, $langkey);
		}
	}

	public function next_record(): bool{
		if($this->count >= $this->anz_all){//no more lines
			$this->stop_next_row = $this->shouldPrintEndTR();
			if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
				$this->Record = [];
				$this->count++;
				return true;
			}
			return false;
		}

		$link = $this->foundlinks[$this->count];
		'@phan-var array{Locale:string,DID:int,DLocale:string,LDID:int,DocumentTable:string,Path:string,ParentID?:int,Url?:string,TriggerID?:int} $link';
		$dLocale = explode('_', $link['DLocale']);
		$Locale = explode('_', $link['Locale']);
		$targetLang = $Locale[0] ?? '';
		$targetCountry = $Locale[1] ?? '';

		switch(addTblPrefix($link['DocumentTable'])){
			default:
			case FILE_TABLE:
				$path_parts = pathinfo($link['Path']);
				$WE_PATH = ($this->hidedirindex && seoIndexHide($path_parts['basename']) ?
					($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' :
					$link['Path']);
				break;
			case OBJECT_FILES_TABLE:
				$path_parts = pathinfo((!empty($link['TriggerID']) ?
					id2path($link['TriggerID']) :
					we_http_util::getScriptName())
				);

				if($this->objectseourls && $link['Url'] != '' && show_SeoLinks()){
					$WE_PATH = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' .
						($this->hidedirindex && seoIndexHide($path_parts['basename']) ?
						'' :
						$path_parts['filename'] . '/') .
						$link['Url'];
				} else {
					$WE_PATH = ($this->hidedirindex && seoIndexHide($path_parts['basename']) ?
						($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' :
						we_http_util::getScriptName()) .
						'?' . WEOBJ_PARAM . '=' . $link['LDID'];
				}
				break;
		}

		$this->Record = [
			self::ID_FIELD => $link['LDID'],
			self::PATH_FIELD => $WE_PATH,
			self::LANG_FIELD => $this->pagelanguage,
			self::DOC_LOCALE_FIELD => $link['DLocale'],
			self::DOC_COUNTRY_FIELD => $dLocale[1] ?? '',
			self::DOC_LANG_FIELD => $dLocale[0],
			self::TRG_LOCALE_FIELD => $link['Locale'],
			self::TRG_COUNTRY_FIELD => $targetCountry,
			self::TRG_LANG_FIELD => $targetLang,
			self::TRG_LANG_NAME_FIELD => $targetLang ? we_base_country::getTranslation($targetLang, we_base_country::LANGUAGE, $targetLang) : '',
			self::TRG_COUNTRYNAME_FIELD => $targetLang ? we_base_country::getTranslation($targetCountry, we_base_country::TERRITORY, $targetLang) : ''
		];

		$this->count++;
		return true;
	}

	public function f(string $key, bool $stripWE = false): string{
		return parent::f($key, $stripWE);
	}

	public function useSeoUrls(): bool{
		return $this->objectseourls;
	}

}
