<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: c1bb9c71a67e $
 * $Author: nilsole <nils.reibold@aldente-it.de> $
 * $Date: 2021-04-03 01:21 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class

 *
 */
abstract class we_listview_objectBase extends we_listview_base{
	/** @var array<string,string> */
	protected const FIELD_REPLACEMENTS = [
		'we_filename' => '`of`.Text',
		OBJECT_X_TABLE => 'ob',
		OBJECT_FILES_TABLE => '`of`',
		'we_' => self::PROPPREFIX,
		'wedoc_' => self::PROPPREFIX,
	];

	/** @var int */
	protected $classID = 0; /* ID of a class */

	/** @var int */
	var $triggerID = 0; /* ID of a document which to use for displaying thr detail page */

	/** @var string */
	protected $triggerPath = '';

	/** @var string */
	var $Path = ''; /* internal: Path of document which to use for displaying thr detail page */

	/** @var string[] array of Languages */
	protected $languages = [];

	/** @var bool */
	var $objectseourls = false;
	//used by we_field

	/** @var int by we_field */
	public $docID = 0;

	/** @var array holds all join and field information */
	protected $matrix = [];

	/** @var string */
	protected $group = '';

	/** @var string[] */
	protected $joins = [];

	/** @var array<int,array<string,mixed>> */
	protected $customerArray = [];

	public function __construct(string $name = '0', int $rows = 999999999, int $offset = 0, string $order = '', bool $desc = false, array $cats = [], bool $catOr = false, array $workspaceID = [
		], int $cols = 0, string $calendar = '', string $datefield = '', string $date = '', string $weekstart = '', array $categoryids = [], string $customerFilterType = 'all', array $id = [
		], int $triggerID = 0, bool $searchable = false, int $docID = 0, array $languages = [
		], bool $objectseourls = false, bool $hidedirindex = false){
		parent::__construct($name, $rows, $offset, $order, $desc, $cats, $catOr, $workspaceID, $cols, $calendar, $datefield, $date, $weekstart, $categoryids, $customerFilterType, $id);
		$this->triggerID = $triggerID;
		$this->searchable = $searchable;
		$this->docID = $docID; //Bug #3720
		$this->languages = $languages;
		$this->objectseourls = $objectseourls;
		$this->hidedirindex = $hidedirindex;

		$this->triggerPath = ($this->triggerID ? id2path($this->triggerID, FILE_TABLE) : '');
		$this->triggerID = $this->triggerPath ? $this->triggerID : 0;

		$this->Path = ($this->docID ?
			id2path($this->docID, FILE_TABLE) :
			($this->triggerID && show_SeoLinks() ?
			id2path($this->triggerID, FILE_TABLE) :
			we_base_WE::getWeDoc()->Path
			)
			) ?: we_base_WE::getWeDoc()->Path;
	}

	public function next_record(): bool{
		if($this->calendar_struct){
			if($this->count >= $this->anz){
				return false;
			}
			parent::next_record();
			$this->Record = [];
			if(!$this->calendar_struct['forceFetch']){
				return !empty($this->calendar_struct['calendar']);
			}
		}

		if($this->DB_WE->next_record()){
			$paramName = $this->docID ? 'we_oid' : WEOBJ_PARAM;
			$this->Record = array_merge(array_change_key_case($this->DB_WE->getRecord()), [
				self::CLASS_FIELD => $this->classID,
				self::PATH_FIELD => $this->Path . '?' . $paramName . '=' . $this->DB_WE->f(self::ID_FIELD),
				self::CUSTOMER_PREFIX . 'ID' => $this->DB_WE->f(self::CUSTOMER_FIELD),
				self::SHOPVARIANTS_FIELD => 0]); //check this for global variants

			$path_parts = pathinfo($this->Path);
			if($this->objectseourls && $this->Record[self::URL_FIELD] && show_SeoLinks()){
				if(!$this->triggerID && $this->Record[self::TRIGGER_FIELD]){
					$path_parts = pathinfo(id2path((int) $this->Record[self::TRIGGER_FIELD]));
				}
				$this->Record[self::PATH_FIELD] = (!empty($path_parts['dirname']) && $path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' .
					($this->hidedirindex && seoIndexHide($path_parts['basename']) ?
					'' :
					$path_parts['filename'] . '/'
					) . $this->Record[self::URL_FIELD];
			} else {
				$this->Record[self::PATH_FIELD] = ($this->hidedirindex && seoIndexHide($path_parts['basename']) ?
					($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' :
					$this->Path
					) . '?' . $paramName . '=' . $this->Record[self::ID_FIELD];
			}
			if(($dat = $this->f(we_base_variants::ELEMENT_NAME))){
				$variants = we_unserialize($dat);
				if(is_array($variants) && !empty($variants)){
					$this->Record[self::SHOPVARIANTS_FIELD] = count($variants);
				}
			}
			// for seeMode #5317
			$this->Record[self::LASTPATH_FIELD] = $this->LastDocPath . '?' . $paramName . '=' . $this->Record[self::ID_FIELD];

			if($this->customers && $this->Record[self::CUSTOMER_FIELD] && isset($this->customerArray[$this->Record[self::CUSTOMER_FIELD]])){
				$this->Record = array_merge($this->Record, self::preFixArrayKeys($this->customerArray[$this->Record[self::CUSTOMER_FIELD]]));
			}

			$this->count++;
			return true;
		}

		$this->stop_next_row = $this->shouldPrintEndTR();
		if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
			$this->Record = [];
			$this->count++;
			return true;
		}

		return !empty($this->calendar_struct['calendar']);
	}

	function f(string $key, bool $stripWE = false): string{
		return parent::f($key, $stripWE);
	}

	protected function fillMatrix(int $classID): void{
		$regs = [];
		$table = OBJECT_X_TABLE . $classID;
		$tableInfo = we_contents_objectFile::getSortedTableInfo($classID, true, $this->DB_WE/* , $withVariant */);
		foreach($tableInfo as $fieldInfo){
			if(!preg_match('/(.+?)_(.*)/', $fieldInfo['name'], $regs)){
				continue;
			}
			[, $type, $name] = $regs;
			$nextClassID = (int) $name;
			if($type === we_contents_objectBase::TYPE_OBJECT && $name != $this->classID){
				if(!empty($this->matrix[self::OBJECT_FIELD_PREFIX . $nextClassID]['type'])){
					continue;
				}
				$this->matrix[self::OBJECT_FIELD_PREFIX . $nextClassID] = [
					'type' => $type,
					'table' => $table,
					'classID' => $classID,
					'alias' => 'ob' . $classID,
					'aliasf' => 'of' . $nextClassID,
					'joinClassID' => $nextClassID,
					'join' => OBJECT_X_TABLE . $nextClassID,
					'on' => OBJECT_FILES_TABLE . ' `of' . $nextClassID . '` ON (`of' . $nextClassID . '`.ID=`ob' . $classID . '`.' . we_contents_object::QUERY_PREFIX . $nextClassID . ' AND `of' . $nextClassID . '`.IsPublished=1) LEFT JOIN
' . OBJECT_X_TABLE . $nextClassID . ' AS `ob' . $nextClassID . '` ON (`ob' . $nextClassID . '`.OF_ID=`of' . $nextClassID . '`.ID AND `ob' . $nextClassID . '`.OF_DocumentTable="tblObjectFiles")'
				];
				$this->fillMatrix($nextClassID);
			} else {
				if(!isset($this->matrix[$name])){
					$this->matrix[$name] = [
						'type' => $type,
						'table' => $table,
						'alias' => 'ob' . $classID,
						'aliasf' => 'of' . $classID,
						'classID' => $classID,
					];
				}
			}
		}
	}

	/**
	 *
	 * @return array
	 */
	protected function makeSQLParts(): array{
		//FIXME: this fails if the condition is formated as (`bla`=12)
		if(!$this->classID){
			t_e('no classid given!');
			return [];
		}
		$from = $fromTable = $orderArr = $descArr = $ordertmp = [];
		$publ_cond = [
			'`of`.IsPublished=1'
		];
		$cond = ' ' . preg_replace_callback("/'([^']*)'/", function (array $match): string{
				return "'##WE##" . base64_encode($match[1]) . "##WE##'";
			}, strtr($this->condition, ['&lt;&gt;' => '!=', '&gt;' => '>', '&lt;' => '<'])) . ' ';

		if($this->order){
			foreach(array_map('trim', explode(',', $this->order)) as $f){
				$g = explode(' ', $f);
				$orderArr[] = $g[0];
				$descArr[] = isset($g[1]) && strtolower(trim($g[1])) === 'desc';
			}
		}

		//get Metadata for class (default title, etc.)
		//BugFix #4629
		$fieldnames = getHash('SELECT DefaultDesc,DefaultTitle,DefaultKeywords FROM ' . OBJECT_TABLE . ' WHERE ID=' . $this->classID);
		$fields = ['`of`.ID AS OF_ID'];
		foreach($fieldnames as $key => $val){
			if(!$val || $val === '_'){
				continue;
			}
			switch($key){
				case 'DefaultDesc':
					$fields[] = 'ob' . $this->classID . '.`' . $val . '` AS ' . self::DESC_FIELD;
					break;
				case 'DefaultTitle':
					$fields[] = 'ob' . $this->classID . '.`' . $val . '` AS ' . self::TITLE_FIELD;
					break;
				case 'DefaultKeywords':
					$fields[] = 'ob' . $this->classID . '.`' . $val . '` AS ' . self::KEYWORDS_FIELD;
					break;
			}
		}
		$tableFields = array_keys(getHash('SELECT * FROM ' . OBJECT_FILES_TABLE . ' LIMIT 1'));
		foreach($tableFields as $cur){
			$fields[] = '`of`.`' . $cur . '` AS `' . self::PROPPREFIX . $cur . '`';
		}

		$charclass = '[\!\=%&\(\)\*\+\.\/<>\|~, -]';
		//joins need to keep their original order
		foreach($this->matrix as $p){
			if(!isset($fromTable[$p['table']]) && $this->classID === intval($p['classID'])){
				$fromTable[$p['table']] = $p['table'] . ' AS `' . $p['alias'] . '`';
			}
			if(!empty($p['join'])){
				$from[$p['join']] = $p['on'];
			}
		}
		//sort keys in reverse order so "Bild" is not matched before "Bild-XX"
		krsort($this->matrix);
		foreach($this->matrix as $n => $p){
			$n2 = $n;
			if(!empty($p['joinClassID'])){
				$n = $p['joinClassID'];
			}

			$fields[] = $p['alias'] . '.`' . $p['type'] . '_' . $n . '` AS `' . $n2 . '`';

			if(($pos = array_search($n, $orderArr)) !== false){
				$ordertmp[$pos] = $p['alias'] . '.`' . $p['type'] . '_' . $n . '`' . ($descArr[$pos] ? ' DESC' : '');
			}
			//some replacements if old conditions may occur
			$cond = strtr($cond, self::FIELD_REPLACEMENTS);

			// we have different data types for date/time and DateTime, convert DateTime (UNIX) to DateTime (string representation)
			if($p['type'] === we_contents_objectBase::TYPE_DATETIME){
				$cond = preg_replace('/(' . $charclass . ')' . $n . '(' . $charclass . ')/', '${1}' . 'FROM_UNIXTIME(' . $p['alias'] . '.`' . $p['type'] . '_' . $n . '`,"%Y-%m-%d %H:%i:%s")$2', $cond);
			} else {
				$cond = preg_replace('/(' . $charclass . ')' . $n . '(' . $charclass . ')/', '${1}' . $p['alias'] . '.`' . $p['type'] . '_' . $n . '`$2', $cond);
			}
		}
		if(count($fromTable) !== 1){
			t_e('we_listview_objectBase::makeSQLParts(): Wrong number of entries in var $fromTable', $fromTable, $from, $this->classID);
		}
		$from = array_slice($fromTable, 0, 1) + $from;

		$cond = preg_replace_callback("/'##WE##([^']*)##WE##'/", function (array $match): string{
			return '\'' . (base64_decode($match[1]) ?: '') . '\'';
		}, $cond);

		$order = '';
		foreach($orderArr as $pos => $curOrd){
			switch(strtolower($curOrd)){
				case self::RANDOM_FIELD:
					$ordertmp = [];
					$order = 'RANDOM ';
					break 2;
				case self::FILENAME_FIELD:
					$ordertmp[$pos] = '`of`.Text' . ($descArr[$pos] ? ' DESC' : '');
					break;
				case self::DATEPUBLISHED_FIELD:
					$ordertmp[$pos] = '`of`.Publish_Date' . ($descArr[$pos] ? ' DESC' : '');
					break;
				case self::DATECREATION_FIELD:
					$ordertmp[$pos] = '`of`.Creation_Date' . ($descArr[$pos] ? ' DESC' : '');
					break;
				case 'vfile':
					$ordertmp[$pos] = 'fl.position';
					break;
				default:
					$ordertmp[$pos] = strtr($curOrd, self::FIELD_REPLACEMENTS) . ($descArr[$pos] ? ' DESC' : '');
			}
		}
		if($ordertmp){
			ksort($ordertmp);
			$order = trim(implode(',', $ordertmp));
		}
		if($order === 'RANDOM '){
			$fields[] = 'RAND() AS RANDOM';
		}

		$group = $this->group ? $this->group . ',`of`.ID' : ((count($from) > 1) ? '`of`.ID ' : '');

		return [
			'fields' => implode(',', $fields),
			'order' => $order ? ' ORDER BY ' . $order : '',
			'tables' => implode(' LEFT JOIN ', $from),
			//FIXME: afaik grouping is not needed
			'groupBy' => $group ? ' GROUP BY ' . $group : '',
			'publ_cond' => ' ( ' . implode(' AND ', $publ_cond) . ' ) ',
			'cond' => trim($cond)
		];
	}

	public function getCustomerRestrictionQuery(int $classID): array{
		return //at least check only documents of the specified class
			[
				' JOIN ' . OBJECT_FILES_TABLE . ' o ON (cf.modelId=o.ID AND o.TableID='.$classID.' AND IsFolder=0)',
				'cf.modelTable="' . stripTblPrefix(OBJECT_FILES_TABLE) . '" AND '
		];
	}

	public function getFoundDocument(): we_document_base{
		static $doc = null;
		static $id = 0;
		if($id === ($docID = (int) $this->f(self::ID_FIELD))){
			assert($doc instanceof we_contents_objectFile);
			return $doc;
		}
		$model = new we_contents_objectFile($docID);
		$id = $docID;
		return ($doc = $model);
	}

	/**
	 *
	 * @return array{id:int,type:string,req:array}
	 */
	public function getShopArticleID(): array{
		return ['id' => (int) $this->f(we_listview_base::ID_FIELD), 'type' => we_shop_shop::OBJECT, 'req' => []];
	}

	public function getTableID(): int{
		return (int) $this->f(we_listview_base::CLASS_FIELD);
	}

	public function getHrefArray(string $name): array{
		$ret = we_unserialize($this->f($name));
		assert(is_array($ret));
		return $ret;
	}

	public function useSeoUrls(): bool{
		return $this->objectseourls;
	}

}
