<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 44f79311fc0c $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-03-23 19:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * General Definition of WebEdition Navigation
 *
 */
class we_navigation_navigation extends we_base_model implements we_backup_restoreIF, we_exim_xmlIF{

	public const BACKUP_XML_TAG = 'we:navigation';
	public const PERSISTENTS = [
		'Attributes' => we_base_request::SERIALIZED_JSON,
		'CatAnd' => we_base_request::BOOL,
		'CatParameter' => we_base_request::RAW,
		'Categories' => we_base_request::INTLISTA,
		'ClassID' => we_base_request::INT,
		'CreatorID' => we_base_request::INT,
		'CurrentOnAnker' => we_base_request::BOOL,
		'CurrentOnUrlPar' => we_base_request::BOOL,
		'Depended' => we_base_request::BOOL,
		'Display' => we_base_request::RAW_CHECKED, //note: it is desired to have test<sup>a</sup>
		'DocTypeID' => we_base_request::INT,
		'DynamicSelection' => we_base_request::STRING,
		'FolderID' => we_base_request::INT,
		'ID' => we_base_request::INT,
		'IconID' => we_base_request::INT,
		'IsFolder' => we_base_request::BOOL,
		'LinkID' => we_base_request::INT,
		'LinkSelection' => we_base_request::RAW,
		'ModifierID' => we_base_request::INT,
		'Ordn' => we_base_request::INT,
		'Owners' => we_base_request::INTLISTA,
		'OwnersReadOnly' => we_base_request::INTLISTA,
		'Parameter' => we_base_request::RAW,
		'ParentID' => we_base_request::INT,
		'Path' => we_base_request::STRING,
		'Publish_Date' => we_base_request::DATE_TIME,
		'Selection' => we_base_request::STRING,
		'SelectionType' => we_base_request::STRING,
		'ShowCount' => we_base_request::INT,
		'Sort' => we_base_request::SERIALIZED_JSON,
		'Text' => we_base_request::STRING,
		'TitleField' => we_base_request::STRING,
		'Url' => we_base_request::URL,
		'UrlID' => we_base_request::INT,
		'UseDocumentFilter' => we_base_request::BOOL,
		'WorkspaceID' => we_base_request::INT,
	];
	const SELECTION_STATIC = 'static';
	const SELECTION_DYNAMIC = 'dynamic';
	const STYPE_URLLINK = 'urlLink';
	const STYPE_DOCLINK = 'docLink';
	const STYPE_OBJLINK = 'objLink';
	const STYPE_CATLINK = 'catLink';
	const DYN_CLASS = 'classname';
	const DYN_DOCTYPE = 'doctype';
	const DYN_CATEGORY = 'category';
	const LSELECTION_INTERN = 'intern';
	const LSELECTION_EXTERN = 'extern';
	const IS_NO_WORKSPACE = 'isNoWorkspace';
	const IS_WORKSPACE = 'isWorkspace';
	const IS_WORKSPACE_R = 'isWorkspaceR';
	const defaultPreviewCode = '<we:navigation navigationname="default" parentid="@###PARENTID###@" />

<we:navigationEntry type="folder" navigationname="default">
  <li><we:navigationField name="text" />
    <we:ifHasEntries><ul><we:navigationEntries /></ul></we:ifHasEntries>
  </li>
</we:navigationEntry>

<we:navigationEntry type="item" navigationname="default">
  <li><we:navigationField complete="link"/></li>
</we:navigationEntry>

<ul>
<we:navigationWrite navigationname="default" />
</ul>';

//properties

	/** @var int */
	var $ID = 0;

	/** @var string */
	var $Text = '';

	/** @var string */
	var $Display = '';

	/** @var string */
	var $Name = '';

	/** @var int */
	var $ParentID = 0;

	/** @var string */
	var $TitleField = '';

	/** @var int */
	var $IconID = 0;

	/** @var bool */
	var $IsFolder = false;

	/** @var string */
	var $Path = '/';

	/** @var DateTime */
	var $Publish_Date = null;

	/** @var string */
	var $Selection = self::SELECTION_STATIC;

	/** @var string */
	var $SelectionType = self::STYPE_DOCLINK;

	/** @var string */
	var $DynamicSelection = self::DYN_DOCTYPE;

	/** @var string */
	public $ContentType = we_base_ContentTypes::NAVIGATION; //For backup

	/** @var int */
	var $FolderID = 0;

	/** @var int */
	var $DocTypeID = 0;

	/** @var int */
	var $ClassID = 0;

	/** @var int[] */
	var $Categories = [];

	/** @var bool */
	var $CatAnd = true;

	/** @var string[] */
	var $CategoryPaths = [];

	/** @var array<int,array{field:string,order:string}> */
	var $Sort = [];

	/** @var int */
	var $ShowCount = 0;

	/** @var int */
	var $LinkID = 0;

	/** @var bool */
	var $CurrentOnUrlPar = false;

	/** @var bool */
	var $CurrentOnAnker = false;

	/** @var int */
	var $Ordn = 0;

	/** @var bool */
	var $Depended = false;

	/** @var int */
	var $WorkspaceID = -1;

	/** @var string */
	var $CatParameter = 'catid';

	/** @var string */
	var $Parameter = '';

	/** @var string */
	var $LinkSelection = self::LSELECTION_INTERN;

	/** @var string */
	var $Url = we_base_link::EMPTY_EXT;

	/** @var int */
	var $UrlID = 0;

	/** @var string */
	var $previewCode = '';

	/** @var array<string,string> */
	var $Attributes = [];

	/** @var string */
	var $Table = NAVIGATION_TABLE;

	/** @var we_navigation_customerFilter */
	public $customerFilter = null;

	/** @var int[] */
	public $Owners = [];

	/** @var array<int,bool> */
	public $OwnersReadOnly = [];

	/** @var int */
	protected $CreatorID = 0;

	/** @var int */
	protected $ModifierID = 0;

	/** @var string */
	public $errMsg = '';

	/** @var bool */
	var $UseDocumentFilter = true;

	/**
	 * Default Constructor
	 * Can load or create new navigation depends of parameter
	 */
	public function __construct(int $navigationID = 0, we_database_base $db = null){
		parent::__construct(NAVIGATION_TABLE, $db);
		$this->persistent_slots = self::PERSISTENTS;

		if(($ws = we_users_util::get_ws(NAVIGATION_TABLE))){
			$this->ParentID = reset($ws);
		}

		if($navigationID){
			$this->load($navigationID);
			$this->Publish_Date = new DateTime();
			$this->ModifierID = we_users_user::getID();
			$this->previewCode = str_replace('@###PARENTID###@', (string) ($this->ID), self::defaultPreviewCode);
		} elseif(isWE()){
			$this->Publish_Date = new DateTime();
			$this->CreatorID = we_users_user::getID();
		}
		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$this->customerFilter = $this->customerFilter ?: new we_navigation_customerFilter();
		}
	}

	function load(int $id = 0, bool $loadData = true): bool{
		if(parent::load($id)){
			$this->CategoryPaths = id2pathA(array_filter($this->Categories) ?: [], CATEGORY_TABLE, false, true);
			if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
				$this->customerFilter = we_navigation_customerFilter::getFilterById($id);
				$this->customerFilter->setUseDocumentFilter($this->UseDocumentFilter);
			}
			return true;
		}
		return false;
	}

	public function getSelectionType(bool $repaired = false): string{
		return $this->IsFolder && $repaired ? $this->DynamicSelection : $this->SelectionType;
	}

	function save(bool $order = true, bool $rebuild = false, array $skipWrite = []): bool{
		$hasCustomer = we_modules_info::isActive(we_modules_info::CUSTOMER);

		$this->Text = self::encodeSpecChars($this->Text);
		$paths = $this->CategoryPaths;
		$this->Categories = path_to_idA($this->CategoryPaths, CATEGORY_TABLE);

		$this->setPath();
		$this->Publish_Date = $rebuild ? $this->Publish_Date : new DateTime();
		$oldPid = -1;

		if($order){
			$ord_count = fI('SELECT COUNT(1) FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . intval($this->ParentID));
			if($this->ID == 0){
				$this->Ordn = $ord_count;
			} else {
				if($this->Ordn > ($ord_count - 1)){
					$this->Ordn = $ord_count;
				}
				$oldPid = fI('SELECT ParentID FROM ' . NAVIGATION_TABLE . ' WHERE ID=' . intval($this->ID));
			}
		}

		if($this->Selection === self::SELECTION_STATIC){
			$this->ShowCount = 0;
		}
		$this->Attributes = is_array($this->Attributes) ? array_filter($this->Attributes) : $this->Attributes;
		$this->UseDocumentFilter = $hasCustomer && $this->customerFilter->getUseDocumentFilter();
		if(parent::save(false, true)){
			$this->registerMediaLinks();
		}

		if($hasCustomer){
			/* $this->customerFilter->setModelId($this->ID);
			  $this->customerFilter->setMode(we_customer_abstractFilter::OFF); */
			$this->customerFilter->save();
		} else {
			we_navigation_customerFilter::delByModelID($this->ID, stripTblPrefix(NAVIGATION_TABLE));
		}

		if($order && $oldPid != $this->ParentID){
// the entry has been moved
			$this->reorder($oldPid);
			$this->reorder($this->ParentID);
			$this->previewCode = str_replace('@###PARENTID###@', (string) ($this->ID), self::defaultPreviewCode);
		}
		$this->CategoryPaths = $paths;

		$this->Name = $this->Text;
		if(!$rebuild){
			we_navigation_cache::delNavigationTree($this->ID);
			if($oldPid != $this->ParentID){
				we_navigation_cache::delNavigationTree($this->ParentID);
				we_navigation_cache::delNavigationTree($oldPid);
			}
		}
		return true;
	}

	function registerMediaLinks(): bool{
		$this->unregisterMediaLinks();
		if($this->IconID){
			$this->MediaLinks[] = $this->IconID;
		}
		if($this->SelectionType === self::STYPE_DOCLINK && $this->LinkID){
			$this->MediaLinks[] = $this->LinkID;
		}

		return parent::registerMediaLinks();
	}

	function delete(): bool{
		if(!we_base_permission::hasPerm([we_base_perm::NAVIGATION_DELETE])){
			$this->errMsg = g_l('navigation', '[no_perms]');
			return false;
		}

		if(!$this->ID){
			$this->errMsg = g_l('navigation', '[nothing_to_delete]');
			return false;
		}

		$users = [];
		if(in_array(($ret = $this->isWorkspace($users)), [self::IS_WORKSPACE, self::IS_WORKSPACE_R], true)){
			$this->errMsg = sprintf(g_l('alert', ($ret === self::IS_WORKSPACE ? '[delete][workspace][user]' : '[delete][workspace][user_r]')), $this->Path, implode("\n", $users));
			return false;
		}

		if($this->IsFolder){
			$this->deleteChilds();
		}

		parent::delete();
		we_customer_abstractFilter::deleteModel([$this->ID], NAVIGATION_TABLE);
		$this->unregisterMediaLinks();
		we_navigation_cache::delNavigationTree($this->ParentID);

		return true;
	}

	private function isWorkspace(array &$users = []): string{
		if(!$this->IsFolder){
			return self::IS_NO_WORKSPACE;
		}

		$users = we_users_util::getUsersForDocWorkspace([$this->ID], 'workSpaceNav');
		if($users){
			return self::IS_WORKSPACE;
		}

		$childs = [];
		we_base_util::pushChilds($childs, $this->ID, NAVIGATION_TABLE, true);

		$users = we_users_util::getUsersForDocWorkspace($childs, 'workSpaceNav');
		$users = array_unique($users);
		if($users){
			return self::IS_WORKSPACE_R;
		}

		return self::IS_NO_WORKSPACE;
	}

	function clearSessionVars(): void{
		if(isset($_SESSION['weS']['module_sessions']['navigation'])){
			unset($_SESSION['weS']['module_sessions']['navigation']);
		}
	}

	public static function filenameNotValid(string $text): bool{
		return str_contains($text, '/');
	}

	function isSelf(): bool{
		if(!$this->ID){
			return false;
		}
		$count = 0;
		$parentid = $this->ParentID;
		while($parentid != 0){
			if($parentid == $this->ID){
				return true;
			}
			$parentid = fI('SELECT ParentID FROM ' . NAVIGATION_TABLE . ' WHERE ID=' . intval($parentid));
			$count++;
			if($count == 9999){
				return false;
			}
		}
		return false;
	}

	function isAllowedForUser(): bool{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return true;
		}
//checkWS
		return fB('SELECT 1 FROM ' . NAVIGATION_TABLE . ' WHERE ID=' . intval($this->ParentID) . ' ' . self::getWSQuery());
	}

	/**
	 *
	 * @return array<int,array{id:int,text:string,field:string}>
	 */
	function getDynamicEntries(): array{
		if($this->Selection !== self::SELECTION_DYNAMIC){
			return [];
		}
		switch($this->DynamicSelection){
			case self::DYN_DOCTYPE:
				return we_navigation_dynList::getDocuments($this->DocTypeID, $this->FolderID, $this->Categories, $this->CatAnd ? ' AND ' : ' OR ', $this->Sort, (int) $this->ShowCount, $this->TitleField);
			case self::DYN_CATEGORY:
				return we_navigation_dynList::getCatgories($this->FolderID, (int) $this->ShowCount);
			default:
				return $this->ClassID > 0 ?
						we_navigation_dynList::getObjects($this->ClassID, $this->FolderID, $this->Categories, $this->CatAnd ? ' AND ' : ' OR ', $this->Sort, (int) $this->ShowCount, $this->TitleField) :
						[];
		}
	}

	/**
	 *
	 * @return array<int,array{ID:int,Text:string}>
	 */
	function getChilds(): array{
		$ret = $this->db->getAllq('SELECT ID,Text FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . intval($this->ID) . ' ORDER BY Ordn', false);
		'@phan-var array<int,array{ID:int,Text:string}> $ret';
		return $ret;
	}

	/**
	 *
	 * @return array<int,array{ID:int,Ordn:int}>
	 */
	private function getDynamicChilds(): array{
		$ret = $this->db->getAllq('SELECT ID,Ordn FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . intval($this->ID) . ' AND IsFolder=0 AND Depended=1 ORDER BY Ordn');
		'@phan-var array<int,array{ID:int,Ordn:int}> $ret';
		return $ret;
	}

	/**
	 *
	 * @param array<int,array{ID:int,Ordn:int}> $items
	 * @return array<int,array{id:int,text:string}>
	 */
	function populateGroup(array $items = []): array{
		$info = $this->getDynamicEntries();
		$new_items = [];

		foreach($info as $k => $item){
			$navigation = new we_navigation_navigation();
			$navigation->ParentID = $this->ID;
			$navigation->Selection = self::SELECTION_STATIC;
			$navigation->SelectionType = ($this->DynamicSelection === self::DYN_DOCTYPE ? self::STYPE_DOCLINK : ($this->DynamicSelection === self::DYN_CATEGORY ? self::STYPE_CATLINK : self::STYPE_OBJLINK));
			$navigation->LinkID = $item['id'];
			$navigation->Ordn = isset($items[$k]) ? $items[$k]['Ordn'] : $k;
			$navigation->Depended = true;
			$navigation->Text = $item['field'] ?: $item['text'];
			$navigation->IconID = $this->IconID;
			$navigation->Url = $this->Url;
			$navigation->UrlID = $this->UrlID;
			$navigation->CatParameter = $this->CatParameter;
			$navigation->LinkSelection = $this->LinkSelection;
			$navigation->Parameter = $this->Parameter;
			$navigation->WorkspaceID = $this->WorkspaceID;

			$navigation->save(false);

			$new_items[] = [
				'id' => $navigation->ID,
				'text' => $navigation->Text
			];
		}

// delete old items??


		return $new_items;
	}

	/**
	 *
	 * @return array<int,array{ID:int,Ordn:int}>
	 */
	function depopulateGroup(): array{
		$items = $this->getDynamicChilds();
		foreach($items as $id){
			$navigation = new we_navigation_navigation($id['ID']);
			$navigation->delete();
		}
		return $items;
	}

	function hasDynChilds(): bool{
		return ($this->ID && fB('SELECT 1 FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . intval($this->ID) . ' AND Depended=1 LIMIT 1'));
	}

	function hasAnyChilds(): bool{
		return ($this->ID && fB('SELECT 1 FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . intval($this->ID) . ' LIMIT 1'));
	}

	function hasIndependentChilds(): bool{
		return ($this->ID && fB('SELECT 1 FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . intval($this->ID) . ' AND Depended=0 LIMIT 1'));
	}

	public function reorderAbs(int $newPos): bool{
		if(!$this->ID || $this->Ordn == $newPos){
			return false;
		}
		if($newPos == -1){//last entry
			$this->Ordn = fI('SELECT MAX(Ordn) FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . intval($this->ParentID)) + 1;
			$this->saveField('Ordn');
			$this->reorder($this->ParentID);
		} else {
//check position
			if($newPos < 0 || $newPos > fI('SELECT MAX(Ordn) FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . intval($this->ParentID))){
				return false;
			}

			if($newPos < $this->Ordn){
				$this->db->query('UPDATE ' . NAVIGATION_TABLE . ' SET Ordn=Ordn+1 WHERE (Ordn BETWEEN ' . $newPos . ' AND ' . $this->Ordn . ') AND ParentID=' . $this->ParentID . ' AND ID!=' . $this->ID);
			} else {//$newPos>Ordn
				$this->db->query('UPDATE ' . NAVIGATION_TABLE . ' SET Ordn=Ordn-1 WHERE (Ordn BETWEEN ' . ($this->Ordn + ($this->Ordn > 0 ? 0 : 1)) . ' AND ' . $newPos . ') AND ParentID=' . $this->ParentID . ' AND ID!=' . $this->ID);
			}

			$this->Ordn = $newPos;
			$this->saveField('Ordn');
			$this->reorder($this->ParentID);
		}
		$this->Ordn = fI('SELECT Ordn FROM ' . NAVIGATION_TABLE . ' WHERE ID=' . intval($this->ID));
		return true;
	}

	private function reorder(int $pid): void{
		$this->db->query('SET @count:=-1');
		$this->db->query('UPDATE ' . NAVIGATION_TABLE . ' SET Ordn=(@count:=@count+1) WHERE ParentID=' . $pid . ' ORDER BY Ordn');
	}

	function reorderUp(): bool{
		if(!($this->ID && $this->Ordn > 0)){
			return false;
		}
		$this->db->query('UPDATE ' . NAVIGATION_TABLE . ' SET Ordn=' . intval($this->Ordn) . ' WHERE ParentID=' . intval($this->ParentID) . ' AND Ordn=' . intval(--$this->Ordn));
		$this->saveField('Ordn');
		$this->reorder($this->ParentID);
		return true;
	}

	function reorderDown(): bool{
		if(!$this->ID){
			return false;
		}
		$num = fI('SELECT COUNT(1) FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . intval($this->ParentID));
		if($this->Ordn < ($num - 1)){
			$this->db->query('UPDATE ' . NAVIGATION_TABLE . ' SET Ordn=' . intval($this->Ordn) . ' WHERE ParentID=' . intval($this->ParentID) . ' AND Ordn=' . intval(++$this->Ordn));
			$this->saveField('Ordn');
			$this->reorder($this->ParentID);
			return true;
		}

		return false;
	}

	function getTableIdForItem(): array{
		switch($this->SelectionType){
			case self::STYPE_URLLINK:
				return ['', 0];
			case self::STYPE_CATLINK:
				if($this->LinkSelection === self::LSELECTION_EXTERN){
					return ['', 0];
				}
				return [FILE_TABLE, $this->UrlID];
			case self::STYPE_OBJLINK:
				return [OBJECT_FILES_TABLE, $this->LinkID];
			default:
			case self::STYPE_DOCLINK:
				return [FILE_TABLE, $this->LinkID];
		}
	}

	/**
	 *
	 * @return array{path:string,param:array<string,mixed>}
	 */
	private function getHrefPath(int $id): array{
		$param = [];
		parse_str($this->Parameter, $param);
		$object = ['TriggerID' => 0, 'Url' => ''];
		'@phan-var array{TriggerID:int,Url:string} $object';

		switch($this->SelectionType){
			case self::STYPE_URLLINK: //folder and entry
				return [
					'path' => $this->Url,
					'param' => $param
				];
			case self::STYPE_CATLINK:
				if(!empty($this->CatParameter)){
					$param[$this->CatParameter] = $id;
				}
				return [
					'path' => $this->LinkSelection === self::LSELECTION_EXTERN ? $this->Url : we_navigation_items::id2path($this->UrlID),
					'param' => $param
				];
			case self::STYPE_OBJLINK: //folder and entry
				if(NAVIGATION_OBJECTSEOURLS){
					$object = getHash('SELECT Url,TriggerID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $id . ' LIMIT 1') ?: ['TriggerID' => 0, 'Url' => ''];
					if(empty($object['Url'])){//@phan-suppress-current-line PhanRedundantCondition
						$param[WEOBJ_PARAM] = $id;
					}
					$p = we_navigation_items::id2path($this->WorkspaceID ?:
									($object['TriggerID'] ?: we_navigation_dynList::getFirstDynDocument($this->WorkspaceID) ));
				} else {
					$param[WEOBJ_PARAM] = $id;
					$p = we_navigation_items::id2path($this->WorkspaceID ?: we_navigation_dynList::getFirstDynDocument($this->WorkspaceID));
				}

				if(!NAVIGATION_OBJECTSEOURLS || empty($object['Url'])){
					return [
						'path' => ($p === '/' ? '' : $p),
						'param' => $param
					];
				}
				$path_parts = pathinfo($p);
				return [
//FIXME: can we use seoIndexHide($path_parts['basename'])??
					'path' => ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' . (
					(NAVIGATION_DIRECTORYINDEX_HIDE && NAVIGATION_DIRECTORYINDEX_NAMES && in_array($path_parts['basename'], array_map('trim', explode(',', NAVIGATION_DIRECTORYINDEX_NAMES)))) ?
					'' : $path_parts['filename'] . '/'
					) . $object['Url'],
					'param' => $param
				];
			case self::STYPE_DOCLINK:
			default: //folder and entry
				$p = we_navigation_items::id2path($id);
				return [
					'path' => ($p === '/' ? '' : $p),
					'param' => $param
				];
		}
	}

	/**
	 * @param int $id
	 * @return string
	 */
	function getHref(int $id = 0): string{
		['path' => $hrefPath, 'param' => $param] = $this->getHrefPath($this->IsFolder ? $this->LinkID : ($id ?: $this->LinkID));
		if($this->CurrentOnAnker && !empty($this->Attributes['anchor'])){
			$param['we_anchor'] = $this->Attributes['anchor'];
		}
		$path = str_replace(['&amp;', '&'], ['&', '&amp;'],
				(strtr(trim($hrefPath), [' ' => '%20']) .
				($param ? '?' . http_build_query($param) : '') .
				((!empty($this->Attributes['anchor']) ) ? ('#' . $this->Attributes['anchor']) : '')
		));


//FIXME: can we use seoIndexHide($path_parts['basename'])??
		if(NAVIGATION_DIRECTORYINDEX_HIDE && NAVIGATION_DIRECTORYINDEX_NAMES && $this->LinkSelection !== self::LSELECTION_EXTERN && $this->SelectionType !== self::STYPE_URLLINK){ //Fix #8353
			$dirindexnames = array_map('trim', explode(',', '/' . str_replace(',', ',/', NAVIGATION_DIRECTORYINDEX_NAMES)));
			return str_replace($dirindexnames, '/', $path);
		}

		return $path;
	}

	function we_load(int $id): bool{
		return parent::load($id);
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		return $this->save();
	}

	function setAttribute(string $name, string $value): void{
		$this->Attributes[$name] = $value;
	}

	function getAttribute(string $name): string{
		return $this->Attributes[$name] ?? '';
	}

	function initByRawData(array $data): void{
		foreach($data as $key => $value){
			$this->{$key} = $value;
		}
	}

	public static function encodeSpecChars(string $string): string{
		$open = '!@###';
		$close = '!###@';
		$amp = '!!##@';

		$string = preg_replace([
			'|<br(\/)?>|',
			'|<(\/)?b>|',
			'|<(\/)?i>|',
			'|&([^;]+);|',
				], [
			$open . 'br${1}' . $close,
			$open . '${1}b' . $close,
			$open . '${1}i' . $close,
			$amp . '${1};',
				], $string);

		return strtr(htmlspecialchars($string), [
			$open => '<',
			$close => '>',
			$amp => '&',
		]);
	}

	public static function getWSQuery(): string{
		return (we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ? '' : ' AND (' . we_users_util::get_ws_paths(NAVIGATION_TABLE) . ')');
	}

	public static function naviEditor(): void{
		$path = we_base_request::_(we_base_request::FILE, 'we_cmd', '', 1);
		$id = (!empty($path)) ? path_to_id($path, NAVIGATION_TABLE) : 0;
		$navi = new we_navigation_navigation($id);
		$db = new DB_WE();

		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			$dirs = ['0' => '/'];
			$def = 0;
		} else {
			$dirs = [];
			$def = null;
		}

		if($id){
			$def = $navi->ParentID;
		}

		$all = $db->getAllFirstq('SELECT ID,Path FROM ' . NAVIGATION_TABLE . ' WHERE IsFolder=1 ' . we_navigation_navigation::getWSQuery() . ' ORDER BY Path', false);
		foreach($all as $qid => $qpath){
			if($def === null){
				$def = $qid;
			}
			$dirs[$qid] = $qpath;
		}

		$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[name]'), we_html_tools::textInput('Text', $navi->Text, 0, ['style' => "width:440px;", 'oninput' => "setSaveState();"])))
						->space(we_html_multiIconBox::SPACE_MED)
						->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[group]'), we_html_tools::select('ParentID', $dirs, 1, $navi->ParentID, false, ['style' => 'width:440px;',
									'onchange' => "queryEntries(this.value);"])))
						->space(we_html_multiIconBox::SPACE_MED)
						->noline()
				)
				->add((new we_html_multiIconBoxContentLine('', '<div id="details" class="blockWrapper" style="width: 440px;height: 100px;"></div>'))
						->space(we_html_multiIconBox::SPACE_MED)
						->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[order]'), we_html_element::hidden('Ordn', $navi->Ordn) .
						we_html_tools::textInput('OrdnTxt', (string) ($navi->Ordn + 1), 0, ['onchange' => "document.we_form.Ordn.value=(document.we_form.OrdnTxt.value-1);"], 'text', '117px') .
						we_html_tools::select('OrdnSelect', ['begin' => g_l('navigation', '[begin]'), 'end' => g_l('navigation', '[end]')], 1, '', false, ['onchange' => "changeOrder(this);"], true, 317)))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline());

		$buttonsBottom = '<div style="float:right">' .
				we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:WE().util.dialogTop(window).save();", '', '', ($id ? false : true), false), '', we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();")) . '</div>';

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => "loaded=1;queryEntries(" . $def . ");"], we_html_element::form(
								['name' => we_base_constants::WE_FORM, 'onsubmit' => "return false"], (new we_html_multiIconBox($parts))->marginLeft()->buttons($buttonsBottom)->headline(g_l('navigation', '[add_navigation]'))
										->get()
		));

		echo we_html_tools::getHtmlTop('', we_html_element::jsScript(WE_JS_MODULES_DIR . 'navigation/weNaviEditor.js', '', ['id' => 'loadVarNavi', 'data-navi' => setDynamicVar([
						'naviID' => $id
			])])
				, $body);
	}

	public static function clearCache(): void{
		$jsCmd = new we_base_jsCmd();
		if(we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT])){
			we_navigation_cache::clean(true);
			$jsCmd->addMsg(g_l('cache', '[cache_deleted]'), we_base_util::WE_MESSAGE_NOTICE);
		} else {
			$jsCmd->addMsg(g_l('alert', '[no_perms_action]'), we_base_util::WE_MESSAGE_ERROR);
		}
		echo we_html_tools::getHtmlTop('', $jsCmd->getCmds(), we_html_element::body());
	}

	public static function reset_customer_filter(): void{
		$jsCmd = new we_base_jsCmd();
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			DB_WE::$inst->query('UPDATE ' . NAVIGATION_TABLE . ' SET UseDocumentFilter=1');
			if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
				DB_WE::$inst->query('DELETE FROM ' . CUSTOMER_FILTER_TABLE . ' WHERE modelTable="' . stripTblPrefix(NAVIGATION_TABLE) . '"');
			}
			//$jsCmd->addCmd('we_navigation_progress_reset_customer_filter');
			$jsCmd->addMsg(g_l('navigation', '[reset_customerfilter_done_message]'), we_base_util::WE_MESSAGE_NOTICE);
		} else {
			$jsCmd->addMsg(g_l('alert', '[no_perms_action]'), we_base_util::WE_MESSAGE_ERROR);
		}
		echo we_html_tools::getHtmlTop('', $jsCmd->getCmds(), we_html_element::body());
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
					'view' => [
						'catLink' => g_l('navigation', '[catLink]'),
						'categories' => g_l('navigation', '[categories]'),
						'delete_alert' => g_l('navigation', '[delete_alert]'),
						'depopulate_question' => g_l('navigation', '[depopulate_question]'),
						'docLink' => g_l('navigation', '[docLink]'),
						'documents' => g_l('navigation', '[documents]'),
						'no_perms' => (g_l('navigation', '[no_perms]')),
						'no_workspace' => (g_l('navigation', '[no_workspace]')),
						'nothing_selected' => (g_l('navigation', '[nothing_selected]')),
						'nothing_to_delete' => (g_l('navigation', '[nothing_to_delete]')),
						'nothing_to_save' => (g_l('navigation', '[nothing_to_save]')),
						'objLink' => g_l('navigation', '[objLink]'),
						'objects' => g_l('navigation', '[objects]'),
						'populate_question' => g_l('navigation', '[populate_question]'),
						'reset_customerfilter_question' => g_l('navigation', '[reset_customerfilter_question]'),
						'save_populate_question' => g_l('navigation', '[save_populate_question]'),
						//'reset_customerfilter_done_message' => g_l('navigation', '[reset_customerfilter_done_message]'),
						'urlLink' => g_l('navigation', '[urlLink]'),
					],
					'rule' => [
						'save_error_fields_value_not_valid' => (g_l('alert', '[save_error_fields_value_not_valid]')),
					]
		]);
	}

	public static function getJSConsts(): string{
		return we_base_util::setLangString([
					'DYN_CLASS' => we_navigation_navigation::DYN_CLASS,
					'DYN_DOCTYPE' => we_navigation_navigation::DYN_DOCTYPE,
					'DYN_CATEGORY' => we_navigation_navigation::DYN_CATEGORY,
					'STYPE_DOCLINK' => we_navigation_navigation::STYPE_DOCLINK,
					'STYPE_URLLINK' => we_navigation_navigation::STYPE_URLLINK,
					'STYPE_OBJLINK' => we_navigation_navigation::STYPE_OBJLINK,
					'STYPE_CATLINK' => we_navigation_navigation::STYPE_CATLINK,
					'SELECTION_DYNAMIC' => we_navigation_navigation::SELECTION_DYNAMIC,
					'SELECTION_STATIC' => we_navigation_navigation::SELECTION_STATIC,
					'LSELECTION_INTERN' => we_navigation_navigation::LSELECTION_INTERN,
					'LSELECTION_EXTERN' => we_navigation_navigation::LSELECTION_EXTERN,
		]);
	}

	public function toXML(array $attribs = [], array $extraSlots = [], string $content = ''): string{
		return parent::toXML($attribs, ['ContentType' => we_base_request::STRING]);
	}

	public function getPersistent(): array{
		return $this->persistent_slots;
	}

}
