<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 883c3facb40d $
 * $Author: Lukas Imhof $
 * $Date: 2021-09-06 23:27 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Definition of WebEdition Newsletter Base
 *
 */
class we_newsletter_base extends we_base_model{

	const STATUS_ERROR = -1;
	const STATUS_SUCCESS = 4;
	const STATUS_EMAIL_EXISTS = 1;
	const STATUS_EMAIL_INVALID = 2;
	const STATUS_CONFIRM_FAILED = 3;
	const FEMALE_SALUTATION_FIELD = 'female_salutation';
	const MALE_SALUTATION_FIELD = 'male_salutation';
	const EMAIL_REPLACE_TEXT = '###EMAIL###';
	const EMAILTOKEN_REPLACE_TEXT = '###EMAILTOKEN###';
	const FIRSTNAME_REPLACE_TEXT = '###FIRSTNAME###';
	const LASTNAME_REPLACE_TEXT = '###LASTNAME###';
	const CUSTOMERID_REPLACE_TEXT = '###CUSTOMERID###';
	const TITLE_REPLACE_TEXT = '###TITLE###';

	const ADRESS_KEYS_5_COLS = ['email' => 0, 'salutation' => 1, 'title' => 2, 'forename' => 3, 'surname' => 4];
	const ADRESS_KEYS_6_COLS = ['email' => 0, 'salutation' => 2, 'title' => 3, 'forename' => 4, 'surname' => 5];

	/**
	 * @param string $emails
	 * @return string[]
	 */
	public static function getEmailsFromList(string $emails): array{
		$arr = array_filter(explode("\n", $emails));
		if(!$arr){
			return [];
		}
		$ret = [];
		'@phan-var string[] $ret';

		foreach($arr as $row){
			$arr2 = explode(',', $row);
			if(!$arr2){
				continue;
			}
			$ret[] = $arr2[0];
		}

		return $ret;
	}

	/**
	 * @param string $emails
	 * @param int $group
	 * @param int[] $blocks
	 * @return array<int,array{string,string,string,string,string,string,string,int[]}>
	 */
	public static function getEmailsFromListCSV(string $emails, int $group = 0, array $blocks = []): array{
		$arr = array_filter(explode("\n", $emails));
		if(!$arr){
			return [];
		}
		$ret = [];
		'@phan-var array<int,array{string,string,string,string,string,string,string,int[]}> $ret';

		foreach($arr as $row){
			$arr2 = explode(',', $row);
			if(!$arr2){
				continue;
			}

			$keys = count($dat) === 6 ? self::ADRESS_KEYS_6_COLS : self::ADRESS_KEYS_5_COLS;
			$ret[] = [
				$arr2[$keys['email']],
				'1', //use html
				isset($arr2[$keys['salutation']]) ? trim($arr2[$keys['salutation']]) : '',
				isset($arr2[$keys['title']]) ? trim($arr2[$keys['title']]) : '',
				isset($arr2[$keys['forename']]) ? trim($arr2[$keys['forename']]) : '',
				isset($arr2[$keys['surname']]) ? trim($arr2[$keys['surname']]) : '',
				(string) $group,
				$blocks
			];
		}

		return $ret;
	}

	/**
	 * @param string[] $files
	 * @param int $emails_only
	 * @param int $group
	 * @param int[] $blocks
	 * @return array<int,array{string,string,string,string,string,string,6?:string,7?:int[]}>
	 */
	public static function getEmailsFromExternCSV(array $files, int $emails_only = 0, int $group = 0, array $blocks = []): array{
		$arr = array_filter($files);
		if(!$arr){
			return [];
		}
		$ret = [];
		foreach($arr as $file){
			$filename = (string) $file;
			if(!str_contains($filename, '..')){
				$dataArr = explode("\n", str_replace("\r\n", "\n", we_base_file::load(WE_DOCUMENT_ROOT . $filename)));
				foreach($dataArr as $value){
					$dat = explode(',', $value);
					if(str_replace(' ', '', implode('', $dat)) === ''){
						continue;
					}

					$keys = count($dat) === 6 ? self::ADRESS_KEYS_6_COLS : self::ADRESS_KEYS_5_COLS;
					switch($emails_only){
						case 2:
							$ret[] = [
								trim($dat[$keys['email']]),
								'1', //use html
								isset($dat[$keys['salutation']]) ? trim($dat[$keys['salutation']]) : '',
								isset($dat[$keys['title']]) ? trim($dat[$keys['title']]) : '',
								isset($dat[$keys['forename']]) ? trim($dat[$keys['forename']]) : '',
								isset($dat[$keys['surname']]) ? trim($dat[$keys['surname']]) : '',
							];
							break;
						default:
							$ret[] = [
								trim($dat[$keys['email']]),
								'1', //use html
								isset($dat[$keys['salutation']]) ? trim($dat[$keys['salutation']]) : '',
								isset($dat[$keys['title']]) ? trim($dat[$keys['title']]) : '',
								isset($dat[$keys['forename']]) ? trim($dat[$keys['forename']]) : '',
								isset($dat[$keys['surname']]) ? trim($dat[$keys['surname']]) : '',
								(string) $group,
								$blocks
							];
					}
				}
			}
		}

		return $ret;
	}

	/**
	 * @param string[] $files
	 * @return string[]
	 */
	public static function getEmailsFromExtern(array $files): array{
		$arr = array_filter($files);
		if(!$arr){
			return [];
		}
		$ret = [];
		foreach($arr as $file){
			$filename = (string) $file;
			if(!str_contains($filename, '..')){
				try{
					$dataArr = explode("\n", str_replace("\r\n", "\n", we_base_file::load(WE_DOCUMENT_ROOT . $filename)));
					foreach($dataArr as $value){
						$dat = explode(',', $value);
						if(str_replace(' ', '', implode('', $dat)) === ''){
							continue;
						}
						$ret[] = $dat[0];
					}
				}catch(we_exception_file $e){
					t_e($e->getMessage());
				}
			}
		}

		return $ret;
	}

	/**
	 * Enter description here...
	 *
	 * @param string[] $files
	 * @param int $status (0=all; 1=invalid; 2=valid )
	 * @param int[] $emailkey
	 * @return array<int,array{string,string,string,string,string,string}>
	 */
	static function getEmailsFromExtern2(array $files, int $status, array &$emailkey): array{
		$files = array_filter($files);
		if(!$files){
			return [];
		}
		$ret = [];
		$countEMails = 0;
		$rawData = '';
		$saveTrimmed = ($status == we_newsletter_newsletter::CSV_STATUS_ALL);
		foreach($files as $file){
			assert(is_string($file));
			if(str_contains($file, '..')){
				continue;
			}
			$dataArr = explode("\n", strtr(($rawData = we_base_file::load(WE_DOCUMENT_ROOT . $file)), ["\r\n" => "\n"]));
			foreach($dataArr as $value){
				$dat = explode(',', $value);
				if(str_replace(' ', '', implode('', $dat)) === ''){
					continue;
				}
				$countEMails++;
				if(($status == we_newsletter_newsletter::CSV_STATUS_INVALID && we_check_email($dat[0])) || ($status == we_newsletter_newsletter::CSV_STATUS_VALID && !we_check_email($dat[0]))){
					continue;
				}
				$emailkey[] = $countEMails - 1;

				$keys = count($dat) === 6 ? self::ADRESS_KEYS_6_COLS : self::ADRESS_KEYS_5_COLS;
				$ret[] = [
					trim($dat[$keys['email']]),
					isset($dat[$keys['salutation']]) ? trim($dat[$keys['salutation']]) : '',
					isset($dat[$keys['title']]) ? trim($dat[$keys['title']]) : '',
					isset($dat[$keys['forename']]) ? trim($dat[$keys['forename']]) : '',
					isset($dat[$keys['surname']]) ? trim($dat[$keys['surname']]) : ''
				];
			}

			if($saveTrimmed){
				$content = '';
				foreach($ret as $entry){
					$content .= implode(',', array_slice($entry, 0, 6)) . "\n";
				}
				if($content !== $rawData){
					while(($lock = we_base_file::lock(basename($file))) == false){
						usleep(100000);
					}
					we_base_file::save(WE_DOCUMENT_ROOT . $file, $content);
					$lock !== false && we_base_file::unlock($lock);
				}
			}
		}

		return $ret;
	}

}
