<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 7d142f7c469d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 11:26 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_shop_revenueTop{
	private static $selectedYear;
	private static $selectedMonth;
	private static $orderBy;
	private static $orderDesc;
	private static $actPage;

	private static function getTitleLink(string $text, string $orderKey): string{
		$href = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_revenueTop' .
			'&ViewYear=' . self::$selectedYear .
			'&ViewMonth=' . self::$selectedMonth .
			'&actPage=' . self::$actPage .
			'&orderBy=' . $orderKey .
			( (self::$orderBy == $orderKey && !self::$orderDesc) ? '&orderDesc=true' : '' );

		return '<span onclick="document.location=\'' . $href . '\';">' . $text . '</span>' . (self::$orderBy == $orderKey ? ' <i class="fa fa-sort-' . (self::$orderDesc ? 'desc' : 'asc') . ' fa-lg"></i>' : '<i class="fa fa-sort fa-lg"></i>');
	}

	private static function getPagerLink(): string{
		return WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_revenueTop' .
			'&ViewYear=' . self::$selectedYear .
			'&ViewMonth=' . self::$selectedMonth .
			'&orderBy=' . self::$orderBy .
			(self::$orderDesc ? '&orderDesc=true' : '' );
	}

	private static function yearSelect(string $select_name): string{
		$yearStart = 2001;
		$yearNow = date('Y');
		$opts = [];

		while($yearNow > $yearStart){
			$opts[$yearNow] = $yearNow;
			$yearNow--;
		}
		return we_html_tools::select($select_name, $opts, 1, we_base_request::int($select_name, -1), false, ['id' => $select_name]);
	}

	private static function monthSelect(string $select_name, int $selectedMonth): string{
		$opts = we_base_country::g_l_a('modules_shop', '[month]');
		$opts[0] = '-';
		ksort($opts, SORT_NUMERIC);
		return we_html_tools::select($select_name, $opts, 1, (string) $selectedMonth, false, ['id' => $select_name]);
	}

	public static function getDialog(): string{
		self::$selectedYear = we_base_request::int('ViewYear', (int) date('Y'));
		self::$selectedMonth = we_base_request::int('ViewMonth', 1);
		self::$orderBy = we_base_request::string('orderBy', 'ID');
		self::$orderDesc = we_base_request::bool('orderDesc');
		self::$actPage = we_base_request::int('actPage', 0);
		$DB_WE = DB_WE::$inst;

// get some preferences!
		$feldnamen = explode('|', fS('SELECT pref_value from ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));
		$waehr = "&nbsp;" . htmlspecialchars($feldnamen[0]);
		/* 		$numberformat = $feldnamen[2];
		  $classid = (isset($feldnamen[3]) ? $feldnamen[3] : ''); */
		$defaultVat = (float) ($feldnamen[1] ?: 0);
		$nrOfPage = (!empty($feldnamen[4]) && $feldnamen[4] !== 'default') ? (int) $feldnamen[4] : 20;

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('',
				//'headline' => '<label for="ViewYear">' . g_l('modules_shop', '[selectYear]') . '</label>',
																							self::yearSelect('ViewYear') . self::monthSelect('ViewMonth', self::$selectedMonth) . we_html_button::create_button(we_html_button::VIEW, "javascript:we_submitDateform();")))
			->space(we_html_multiIconBox::SPACE_SMALL)
			->noline()
		);

		$to = (self::$selectedMonth && self::$selectedMonth < 12 ?
			self::$selectedYear . '-' . (self::$selectedMonth + 1) :
			(self::$selectedYear + 1) . '-01');

		$queryShopDateCondtion = '(o.DateOrder BETWEEN "' . self::$selectedYear . '-' . (self::$selectedMonth ?: '01') . '-01" AND "' . $to . '-01")';

		if(($maxRows = fI('SELECT COUNT(1) FROM ' . SHOP_ORDER_TABLE . ' o JOIN ' . SHOP_ORDER_ITEM_TABLE . ' oi ON o.ID=oi.orderID WHERE ' . $queryShopDateCondtion))){


			$amountOrders = fI('SELECT COUNT(1) FROM ' . SHOP_ORDER_TABLE . ' o WHERE ' . $queryShopDateCondtion);
			$editedOrders = fI('SELECT COUNT(1) FROM ' . SHOP_ORDER_TABLE . ' o WHERE ' . $queryShopDateCondtion . ' AND (!ISNULL(o.DateShipping) OR !ISNULL(o.DateFinished) )');

			$query = 'SELECT
SUM((oi.Price*oi.quantity*IF(o.pricesNet && o.calcVat && COALESCE(oi.Vat,' . $defaultVat . '),(1+COALESCE(oi.Vat,' . $defaultVat . ')/100),1))) AS brutto,
SUM((oi.Price*oi.quantity*IF(o.pricesNet,1,(1/(1+COALESCE(oi.Vat,' . $defaultVat . ')/100))))) AS netto
FROM ' . SHOP_ORDER_TABLE . ' o JOIN ' . SHOP_ORDER_ITEM_TABLE . ' oi ON o.ID=oi.orderID WHERE ' . $queryShopDateCondtion;


			$payed = getHash($query . ' AND o.DatePayment IS NOT NULL AND o.DateCancellation IS NULL');
			//$canceled = f($query . ' AND o.DatePayment IS NULL AND o.DateCancellation IS NOT NULL');
			$unpayed = getHash($query . ' AND o.DatePayment IS NULL AND o.DateCancellation IS NULL');

			$articleVatArray = $DB_WE->getAllFirstq('SELECT
oi.Vat,
SUM((IF(o.pricesNet,oi.Price*oi.quantity*(COALESCE(oi.Vat,' . $defaultVat . ')/100), (oi.Price*oi.quantity)-( (oi.Price*oi.quantity)/(1+ (COALESCE(oi.Vat,' . $defaultVat . ')/100) )) )))
FROM
	' . SHOP_ORDER_TABLE . ' o JOIN
	' . SHOP_ORDER_ITEM_TABLE . ' oi ON o.ID=oi.orderID
WHERE
	' . $queryShopDateCondtion . ' AND
	o.DateCancellation IS NULL AND
	o.calcVat AND
	COALESCE(oi.Vat,' . $defaultVat . ')>0
GROUP BY
	oi.Vat', false);

			'@phan-var array<float,float> $articleVatArray';

			// generate vat table
			if(!empty($articleVatArray)){
				$vatTable = '
			<tr>
				<td colspan="7" class="shopContentfontR" style="padding-top:10px;">' . g_l('modules_shop', '[plusVat]') . ':</td>
			</tr>';
				foreach($articleVatArray as $vat => $amount){
					$vatTable .= '
				<tr>
					<td colspan="5"></td>
					<td class="shopContentfontR">' . $vat . '&nbsp;%</td>
					<td class="shopContentfontR">' . we_base_util::formatNumber((float) $amount) . $waehr . '</td>
				</tr>';
				}
			} else {
				$vatTable = '';
			}

			$parts->add(new we_html_multiIconBoxContentLine('', '
			<table class="defaultfont" style="width:680px;">
			<tr>
				<th>' . g_l('modules_shop', '[anual]') . '</th>
				<th>' . (self::$selectedMonth ? g_l('modules_shop', '[monat]') : '' ) . '</th>
				<th>' . g_l('modules_shop', '[anzahl]') . '</th>
				<th>' . g_l('modules_shop', '[unbearb]') . '</th>
				<th>' . g_l('modules_shop', '[schonbezahlt]') . '</th>
				<th>' . g_l('modules_shop', '[unbezahlt]') . '</th>
				<th class="shopContentfontR">' . g_l('modules_shop', '[umsatzgesamt]') . '</th>
			</tr>' . '
			<tr class="shopContentfont">
				<td>' . self::$selectedYear . '</td>
				<td>' . (self::$selectedMonth > 0 ? self::$selectedMonth : '' ) . '</td>
				<td>' . $amountOrders . '</td>
				<td class="defaultfont shopNotPayed">' . ($amountOrders - $editedOrders) . '</td>
				<td>' . we_base_util::formatNumber((float) $payed['netto']) . $waehr . '</td>
				<td class="defaultfont shopNotPayed">' . we_base_util::formatNumber((float) $unpayed['netto']) . $waehr . '</td>
				<td class="shopContentfontR">' . we_base_util::formatNumber((float) ($payed['netto'] + $unpayed['netto'])) . $waehr . '</td>
			</tr>' .
					$vatTable .
					'<tr><td colspan="7"><hr style="color:black"/></td></tr>' .
					'<tr style="border-top:2px solid black" class="shopContentfont">
				<td colspan="4"></td>
				<td>' . we_base_util::formatNumber((float) $payed['brutto']) . $waehr . '</td>
				<td class="defaultfont shopNotPayed">' . we_base_util::formatNumber((float) $unpayed['brutto']) . $waehr . '</td>
				<td class="shopContentfontR">' . we_base_util::formatNumber((float) $payed['brutto'] + $unpayed['brutto']) . $waehr . '</td>
			</tr>' .
					'</table>'
			));

			$headline = [
				self::getTitleLink(g_l('modules_shop', '[bestellung]'), 'o.ID'),
				g_l('modules_shop', '[ArtName]'), // 'shoptitle'
				g_l('modules_shop', '[anzahl]'),
				self::getTitleLink(g_l('modules_shop', '[artPrice]'), 'Price'),
				g_l('modules_shop', '[Gesamt]'),
				self::getTitleLink(g_l('modules_shop', '[artOrdD]'), 'DateOrder'),
				self::getTitleLink(g_l('modules_shop', '[ArtID]'), 'orderDocID'),
				self::getTitleLink(g_l('modules_shop', '[artPay]'), 'DatePayment'),
			];
			$content = [];
			'@phan-var string[][] $content';
			$all = $DB_WE->getAllq('SELECT o.ID,
	@price:=(oi.Price*IF(o.pricesNet && o.calcVat && COALESCE(oi.Vat,' . $defaultVat . '),(1+COALESCE(oi.Vat,' . $defaultVat . ')/100),1)) AS priceToShow,
	(oi.quantity*@price) AS articleSum,
	oi.quantity,
	oi.customFields,
	od.DocID,
	od.title,
	od.variant,
	DATE_FORMAT(DateOrder,"' . g_l('date', '[format][mysql]') . '") AS formatDateOrder,
	DATE_FORMAT(DatePayment,"' . g_l('date', '[format][mysql]') . '") AS formatDatePayment,
	DateCancellation IS NOT NULL AS isCancelled
FROM ' . SHOP_ORDER_TABLE . ' o JOIN ' . SHOP_ORDER_ITEM_TABLE . ' oi ON o.ID=oi.orderID JOIN ' . SHOP_ORDER_DOCUMENT_TABLE . ' od ON oi.orderDocID=od.ID
WHERE ' .
				$queryShopDateCondtion . '
ORDER BY ' . we_base_request::string('orderBy', 'o.ID') . (self::$orderDesc ? ' DESC' : '') . ' LIMIT ' . (self::$actPage * $nrOfPage) . ',' . $nrOfPage);

			foreach($all as $hash){
				$variantStr = ($hash['variant'] ? '<br/><strong>' . g_l('modules_shop', '[variant]') . ': ' . $hash['variant'] . '</strong>' : '');
				if($hash['customFields']){
					$cf = we_unserialize($hash['customFields']);
					$customFields = we_html_element::br();
					foreach($cf as $key => $val){
						$customFields .= $key . '=' . $val . we_html_element::br();
					}
				} else {
					$customFields = '';
				}

				$content[] = [
					'<a href="javascript:we_cmd(\'openOrder\',' . $hash['ID'] . ',\'shop\',\'' . SHOP_ORDER_TABLE . '\');">' . $hash['ID'] . '</a>',
					$hash['title'] . '<span class="small">' . $variantStr . ' ' . $customFields . '</span>',
					(string) $hash['quantity'],
					we_base_util::formatNumber((float) $hash['priceToShow']) . $waehr,
					we_base_util::formatNumber((float) $hash['articleSum']) . $waehr,
					(string) $hash['formatDateOrder'],
					(string) $hash['DocID'],
					($hash['formatDatePayment'] ?: ( $hash['isCancelled'] ?
					'<span class="defaultfont shopNotPayed">' . g_l('modules_shop', '[artCanceled]') . '</span>' :
					'<span class="defaultfont shopNotPayed">' . g_l('modules_shop', '[artNPay]') . '</span>')),
				];
			}


			$parts
				->add((new we_html_multiIconBoxContentLine('', we_html_tools::dialogBorder(670, $content, $headline)))
					->noline()
				)
				->add(new we_html_multiIconBoxContentLine('', we_shop_pager::getStandardPagerHTML(self::getPagerLink(), self::$actPage, $nrOfPage, $maxRows)));
		} else {
			$parts->add(new we_html_multiIconBoxContentLine('', g_l('modules_shop', '[NoRevenue]') . ' (' . (self::$selectedMonth ? g_l('modules_shop', '[month][' . self::$selectedMonth . ']') . ' ' : '') . self::$selectedYear . ')'
			));
		}

		return we_html_tools::getHtmlTop('', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/edit_shop_revenueTop.js'), we_html_element::body(['class' => "weEditorBody",
					'onload' => "self.focus(); setHeaderTitle('" . (self::$selectedMonth ? g_l('modules_shop', '[month][' . self::$selectedMonth . ']') . ' ' : '') . self::$selectedYear . "');"], '<form action="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_revenueTop" method="post">' .
						(new we_html_multiIconBox($parts, 'revenues'))
						->marginLeft()
						->headline(sprintf(g_l('tabs', '[module][revenueTotal]'), self::$selectedYear))
						->get() . '</form>'));
	}

}
