<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 28cd35b6fd7b $
 * $Author: Lukas Imhof $
 * $Date: 2021-09-13 17:52 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_addDelNewsletterEmail extends we_weTag_tag{
	/** @var bool */
	public static $isDoubleOptIn = false;

	/** @var bool */
	public static $MAILING_LIST_EMPTY = true;

	protected static function doUnsubscribe(bool $isCustomer, string $emailField, array $abos, array $paths, string $unsubscribe_mail): array{
		if(!we_newsletter_util::unsubscribeNewsletter($isCustomer, $emailField, $abos, $paths, $unsubscribe_mail)){
			return [];
		}
		try{
			return we_newsletter_util::getNewsletterFields('', '', we_base_request::_(we_base_request::EMAIL, 'we_unsubscribe_email__'));
		} catch (we_exception_newsletter $e){
			we_newsletter_util::$REMOVENEWSLETTER_STATUS = $e->getCode();
			return [];
		}
	}

	/*
	 *
	 * type           Anmeldeverfahren, moegliche Werte sind: customer, csv, emailonly
	 * fieldGroup     Erwartet eine Feldgruppe (Bereich) aus der webEdition KV; Default: "Newsletter"; Nur bei $type == customer
	 * mailingList    Erwartet den Namen der Mailing-Liste OHNE Feldgruppe (Bereich) aus der webEdition KV; Default: "Ok"; Nur bei $type == customer
	 */

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['type' => false], __CLASS__);

		$useListsArray = we_base_request::bool('we_use_lists__');
		$confirmID = we_base_request::string('confirmID');
		$subscribeMail = we_base_request::_(we_base_request::EMAIL, 'we_subscribe_email__');
		$isSubscribe = $subscribeMail || $confirmID;
		$unsubscribeMail = we_base_request::_(we_base_request::EMAIL, 'we_unsubscribe_email__');
		$isUnsubscribe = $unsubscribeMail != '';
		$doubleoptin = self::getAttribute('doubleoptin', $attribs, false, we_base_request::BOOL);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$adminmailid = intval(self::getAttribute('adminmailid', $attribs, 0, we_base_request::INT));
		$adminsubject = self::getAttribute('adminsubject', $attribs, '', we_base_request::STRING);
		$adminemail = self::getAttribute('adminemail', $attribs, '', we_base_request::STRING);
		$fieldGroup = self::getAttribute('fieldGroup', $attribs, 'Newsletter', we_base_request::STRING);
		$abos = $paths = [];
		$db = DB_WE::$inst;

		$customerFieldPrefs = we_newsletter_view::getSettings();
		$subList = we_base_request::stringList('we_subscribe_list__');

		if(!$useListsArray){
			switch($type){
				case 'customer':
					$tmpAbos = self::getAttribute('mailingList', $attribs, [], we_base_request::STRING_LIST);
					if(!$tmpAbos || $tmpAbos[0] == ''){
						$abos[0] = $fieldGroup . '_Ok';
					} else {// #6100
						foreach($tmpAbos as $abo){
							$abos[] = $fieldGroup . '_' . $abo;
						}
					}
					break;
				case 'csv':
					$paths = self::getAttribute('path', $attribs, ['newsletter.txt'], we_base_request::FILELISTA) ?: ['newsletter.txt'];
					break;
			}
		} elseif($subList){
			switch($type){
				case 'customer':
					$tmpAbos = self::getAttribute('mailingList', $attribs, [], we_base_request::STRING_LIST);
					foreach($subList as $nr){
						$abos[] = $fieldGroup . '_' . $tmpAbos[intval($nr) - 1];
					}
					break;
				default:
					$tmpPaths = self::getAttribute('path', $attribs, [], we_base_request::FILELISTA);
					foreach($subList as $nr){
						$paths[] = $tmpPaths[intval($nr) - 1];
					}
					break;
			}

			if(empty($abos) && empty($paths)){
				self::$MAILING_LIST_EMPTY = true;
				if($isSubscribe){
					we_newsletter_util::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
				} else {
					we_newsletter_util::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
				}
				return '';
			}
		} else {
			self::$MAILING_LIST_EMPTY = true;
			if($isSubscribe){
				we_newsletter_util::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
			} else {
				we_newsletter_util::$REMOVENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
			}
			return '';
		}

		$db->query('DELETE FROM ' . NEWSLETTER_CONFIRM_TABLE . ' WHERE expire<NOW()');

		//NEWSLETTER SUBSCTIPTION
		$f = [];
		$notifyAdmin = false;
		if($isSubscribe){
			we_newsletter_util::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_SUCCESS;
			try{
				$f = $subFields = we_newsletter_util::getNewsletterFields($confirmID, we_base_request::_(we_base_request::EMAIL, 'mail', ''), we_base_request::_(we_base_request::EMAIL, 'we_subscribe_email__'), $abos); //FIXME: use data from above
				'@phan-var array{subscribe_mail:string,subscribe_salutation:string,subscribe_title:string,subscribe_firstname:string,subscribe_lastname:string,lists:string[]} $subFields';
			} catch (we_exception_newsletter $e){
				we_newsletter_util::$WRITENEWSLETTER_STATUS = $e->getCode();
				return '';
			}

			// Setting Globals FOR WE-Tags
			we_newsletter_util::$MAIL = $subFields['subscribe_mail'] ?? '';
			we_newsletter_util::$SALUTATION = $subFields['subscribe_salutation'] ?? '';
			we_newsletter_util::$TITLE = $subFields['subscribe_title'] ?? '';
			we_newsletter_util::$FIRSTNAME = $subFields['subscribe_firstname'] ?? '';
			we_newsletter_util::$LASTNAME = $subFields['subscribe_lastname'] ?? '';

			if(!empty($subFields['lists'])){ // allways false when no dbloptin!
				if(!we_modules_info::isActive(we_modules_info::CUSTOMER) || str_contains(implode('', $subFields['lists']), '.')){
					$paths = $subFields['lists'];
				} else {
					$abos = $subFields['lists'];
					$type = 'customer';
				}
			}

			if(empty($subFields)){
				we_newsletter_util::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_ERROR;
				return '';
			}

			if($doubleoptin && !$confirmID){ // Direkte Anmeldung mit doubleoptin => zuerst confirmmail verschicken.
				$customerEmailField = '';
				try{
					$customerEmailField = we_newsletter_view::getCustomerEmailField();
				} catch (Exception $ex) {
					if($type === 'customer'){
						t_e('warning', $ex->getMessage());
					}
				}

				$includeimages = self::getAttribute('includeimages', $attribs, false, we_base_request::BOOL);
				$expires = self::getAttribute('expiredoubleoptin', $attribs, 1440, we_base_request::INT);
				$id = self::getAttribute('id', $attribs, 0, we_base_request::INT);
				$subject = self::getAttribute('subject', $attribs, 'newsletter', we_base_request::STRING);
				$from = self::getAttribute('from', $attribs, 'newsletter@' . $_SERVER['SERVER_NAME'], we_base_request::EMAIL);
				$toCC = self::getAttribute('recipientCC', $attribs, [], we_base_request::EMAILLISTA);
				$toBCC = self::getAttribute('recipientBCC', $attribs, [], we_base_request::EMAILLISTA);
				$mailid = self::getAttribute('mailid', $attribs, '', we_base_request::INT);
				we_newsletter_util::addDoubleOptIn($db, $type, $customerEmailField, $subFields, $abos, $paths, $mailid, $includeimages, $expires, $id, $subject, $from, $toCC, $toBCC);
				return '';
			}

			switch($type){
				case 'customer':
					we_newsletter_util::addToDB($customerFieldPrefs, $subFields, $abos);
					break;
				case 'emailonly':
					//nicht in eine Liste eintragen sondern adminmail versenden
					we_newsletter_util::$WRITENEWSLETTER_STATUS = we_newsletter_base::STATUS_SUCCESS;
					$notifyAdmin = true;
					break;
				case 'csv':
					//in die Liste eintragen
					we_newsletter_util::addToFile($paths, $subFields);
					if(we_newsletter_util::$WRITENEWSLETTER_STATUS === we_newsletter_base::STATUS_SUCCESS){
						$notifyAdmin = true;
					}
			}
			$db->query('DELETE FROM ' . NEWSLETTER_CONFIRM_TABLE . ' WHERE subscribe_mail="' . $db->escape(strtolower(we_newsletter_util::$MAIL)) . '"');

		} elseif($isUnsubscribe){//NEWSLETTER UNSUBSCTIPTION
				$customerEmailField = '';
				try{
					$customerEmailField = we_newsletter_view::getCustomerEmailField();
				} catch (Exception $ex) {
					if($type === 'customer'){
						t_e('warning', $ex->getMessage());
					}
				}
			$f = self::doUnsubscribe($type === 'customer', $customerEmailField, $abos, $paths, we_base_request::_(we_base_request::EMAIL, 'we_unsubscribe_email__'));
			$notifyAdmin = !empty($f);
		}

		try{
			if($notifyAdmin){
				we_newsletter_util::mailNewSuccessfullNewsletterActiviation($adminmailid, $adminemail, $adminsubject, $f, self::getAttribute('includeimages', $attribs, false, we_base_request::BOOL));
			}
		} catch (we_exception_mail $e){
			t_e('addDelNewsletterEmail, failed sending email to admin: ' . $e->getMessage(), $f);
		}

		unset($_REQUEST['we_unsubscribe_email__'], $_REQUEST['we_subscribe_email__'], $_REQUEST['we_subscribe_title__'], $_REQUEST['we_subscribe_salutation__'], $_REQUEST['we_subscribe_firstname__'], $_REQUEST['we_subscribe_lastname__'], $_REQUEST['we_subscribe_list__']);
		return '';
	}

}
