<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 4d342d19796e $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-08-06 12:04 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_condition extends we_weTag_tag{
	/** @var string */
	protected static $conditionName = '';

	/** @var int */
	private static $count = 0;

	/** @var array<string,string> */
	protected static $conditions = ['we_lv_condition' => ''];

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @return string parsed tag for php
	 */
	public static function parse(array $attribs, string $content): string{
		$attribs['_type'] = 'start';
		return '<?php ' . we_weTag_parser::printTag('condition', $attribs) . ';?>' . $content . '<?php ' . we_weTag_parser::printTag('condition', ['_type' => 'stop']) . ';?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		$name = self::getAttribute('_name_orig', $attribs, 'we_lv_condition', we_base_request::STRING);
		//internal Attribute
		switch(self::getAttribute('_type', $attribs, '', we_base_request::STRING)){
			case 'start':

				self::$count++;

				if(self::$count == 1){
					self::$conditionName = $name;
					self::$conditions[self::$conditionName] = '(';
				} else {
					self::$conditions[self::$conditionName] .= '(';
				}
				break;
			case 'stop':
				self::$conditions[self::$conditionName] .= ')';
				self::$conditions[self::$conditionName] = str_replace('()', '', self::$conditions[self::$conditionName]);
				self::$count--;
				if(self::$count == 0){
					$GLOBALS[self::$conditionName] = self::$conditions[self::$conditionName];
					self::$conditionName = '';
				}
				break;
		}
		return '';
	}

	public static function append(string $val){
		if(self::$conditionName){
			self::$conditions[self::$conditionName] .= $val;
		}
	}

	public static function appendNeutral(){
		if(self::$conditionName){
			self::$conditions[self::$conditionName] .= (preg_match('/^(.*)AND ?$/', self::$conditions[self::$conditionName]) ? '1 ' : '0 ');
		}
	}

	public static function getCond(string $name): string{
		return isset(self::$conditions[$name]) ? self::$conditions[$name] : $name; //in case this is plain sql
	}

	public static function isActive(): bool{
		return !empty(self::$conditionName);
	}

}
