<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: dcbe32926508 $
 * $Author: Lukas Imhof $
 * $Date: 2021-08-26 12:56 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_listdir extends we_weTag_tag{
	private const defaultPos = [
		'position' => 0,
		'size' => 0,
		'field' => '',
		'id' => 0,
		'path' => ''
	];

	/** @var array<int,array{ID:int,Path:string,Text:string,sort:string}>	 */
	private static $files = [];

	/** @var array<string,mixed> */
	public static $position = self::defaultPos;

	public static function parse(array $attribs, string $content): string{
		return '<?php
		' . we_weTag_parser::printTag('listdir', $attribs) . ';
while(' . we_weTag_parser::printTag('listdir', ['_internal' => true]) . '){
	?>' . $content . '<?php
}
we_tag_listview::post();?>';
	}

	protected static function internal(): bool{
		$pos = self::$position['position'];
		if(!isset(self::$files[$pos])){
			return false;
		}
		$we_locfile = self::$files[$pos];
		$current = we_tag_listview::$current;
		'@phan-var we_listview_pseudo $current';
		$current->field = $we_locfile['Text'];
		$current->ID = $we_locfile['ID'];
		$current->Path = $we_locfile['Path'];
		$current->IsFolder = (bool) $we_locfile['IsFolder'];
		self::$position = [
			'position' => $pos + 1,
			'size' => count(self::$files),
			'field' => $we_locfile['Text'],
			'id' => $we_locfile['ID'],
			'path' => $we_locfile['Path']];
		return true;
	}

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		if(self::getAttribute('_internal', $attribs, false, we_base_request::BOOL)){
			return self::internal();
		}
		self::$files = [];
		self::$doc = we_base_WE::getWeDoc();

		$dirID = self::getAttribute('id', $attribs, self::$doc->ParentID, we_base_request::INT);
		$index = self::getAttribute('index', $attribs, ['index.html', 'index.htm', 'index.php', 'default.htm', 'default.html', 'default.php'], we_base_request::STRING_LIST);
		$field = (string) self::getAttribute('field', $attribs, '', we_base_request::STRING);
		$dirfield = (string) self::getAttribute('dirfield', $attribs, $field, we_base_request::STRING);
		$sort = self::getAttribute('order', $attribs, '', we_base_request::STRING);
		$desc = self::getAttribute('desc', $attribs, false, we_base_request::BOOL);
		$searchable = self::getAttribute('searchable', $attribs, false, we_base_request::BOOL);
		$all = DB_WE::$inst->getAllq('SELECT
	f.ID,
	f.Text,
	f.IsFolder,
	f.Path,
	IF(f.IsFolder,pI.ID,0) AS FolderIndex,' .
			($field ? 'c.Dat AS name,' : '') .
			($sort ? 'sc.Dat' : 'f.Text') . ' AS sort
FROM ' .
			FILE_TABLE . ' f LEFT JOIN ' .
			FILE_TABLE . ' pI ON (pI.ParentID=f.ID AND pI.IsFolder=0 AND pI.Text IN ("' . implode('","', $index) . '") AND (pI.IsPublished=1))' .
			($field ? ' LEFT JOIN ' . CONTENT_TABLE . ' c ON (c.DID=IF(f.IsFolder,pI.ID,f.ID) AND c.nHash=IF(f.IsFolder,x\'' . md5($dirfield) . '\',x\'' . md5($field) . '\'))' : '') .
			($sort ? ' LEFT JOIN ' . CONTENT_TABLE . ' sc ON (sc.DID=IF(f.IsFolder,pI.ID,f.ID) AND sc.nHash=x\'' . md5($sort) . '\')' : '') . '
WHERE
	((f.IsPublished=1 ' . ($searchable ? 'AND f.IsSearchable=1' : '') . ') OR (f.IsFolder=1)) AND
	f.ParentID=' . intval($dirID) . '
GROUP BY
	f.ID
ORDER BY
	f.IsFolder DESC,
	' . ($sort ? 'sort' . ($desc ? ' DESC' : '') . ',' : '') .
	'f.Text' . ($desc ? ' DESC' : ''));

		foreach($all as $rec){
			$id = (int) ($rec['IsFolder'] ?
				$rec['FolderIndex'] :
				$rec['ID']);

			if($id){
				self::$files[] = [
					'ID' => (int) $rec['ID'],
					'Path' => $rec['Path'],
					'Text' => (($rec['name']??'') ?: $rec['Text']),
					'IsFolder' => $rec['IsFolder'],
				];
			}
		}
		self::$position = self::defaultPos;
		//Fake listview
		we_tag_listview::pre(new we_listview_pseudo());
		return true;
	}

	public static function resetPosition(): void{
		self::$position = self::defaultPos;
	}

}
