<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 8bf6068e56e5 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-08-06 10:34 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_listview extends we_weTag_tag{
	/** @var we_listview_base[] */
	protected static $lvArray = [];

	/** @var we_listview_base|null */
	public static $current = null;

	/** @var int */
	public static $collectionID = 0;

	public static function parse(array $attribs, string $content): string{
		switch(self::getParserAttribute('type', $attribs, '', we_base_request::STRING)){
			case 'shopVariant':
				return self::parseError('listview type shopVariant unsupported, use variant instead');
			default:
				self::attributeMissing($attribs, ['type' => false], __CLASS__);
				break;
			case 'document':
			case 'search':
			case 'languagelink':
			case 'customer':
			case 'onlinemonitor':
			case 'order':
			case 'category':
				break;
			case 'sql':
				$predefinedSQL = self::getParserAttribute('predefinedSQL', $attribs, '', we_base_request::RAW);
				if(!$predefinedSQL){
					self::attributeMissing($attribs, ['select' => false, 'from' => false, 'where' => false], __CLASS__);
				}
				break;
			case 'object':
				if(we_modules_info::isActive(we_modules_info::OBJECT)){
					self::attributeMissing($attribs, ['classid' => false], __CLASS__);
				}
				break;
			case 'orderitem':
				if(we_modules_info::isActive(we_modules_info::SHOP)){
					self::attributeMissing($attribs, ['orderid' => false], __CLASS__);
				}
				break;
			case 'multiobject':
				if(we_modules_info::isActive(we_modules_info::OBJECT)){
					self::attributeMissing($attribs, ['name' => false], __CLASS__);
				}
				break;
		}
		return '<?php ' . we_weTag_parser::printTag('listview', $attribs) . ';?>' . $content . '<?php we_tag_listview::post();?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['type' => false], __CLASS__);
		self::$doc = we_base_WE::getWeDoc();

		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$doctype = array_map('we_base_file::addPreSlash', self::getAttribute('doctype', $attribs, [], we_base_request::STRING_LIST));
		$doctypeids = self::getAttribute('doctypeIDs', $attribs, [], we_base_request::INTLISTA) ?:
			($doctype ? path_to_idA($doctype, DOC_TYPES_TABLE) : []);
		$classid = (int) self::getAttribute('classid', $attribs, 0, we_base_request::INT);
		$we_lv_cats = (array) we_base_request::_(we_base_request::WEFILELISTA, 'we_lv_cats_' . $name, self::getAttribute('categories', $attribs, [], we_base_request::WEFILELISTA));
		$categoryids = (array) self::getAttribute('categoryids', $attribs, [], we_base_request::INTLISTA);
		$we_lv_categoryids = we_base_request::intList('we_lv_categoryids_' . $name, $categoryids);
		$we_lv_catOr = we_base_request::bool('we_lv_catOr_' . $name, self::getAttribute('catOr', $attribs, false, we_base_request::BOOL));

		$we_rows = intval(self::getAttribute('rows', $attribs, 100000000, we_base_request::INT));
		$order = self::getAttribute('order', $attribs, '', we_base_request::STRING);
		$we_lv_order = we_base_request::string('we_lv_order_' . $name, $order);

		$we_lv_numorder = we_base_request::bool('we_lv_numorder_' . $name, self::getAttribute('numorder', $attribs, false, we_base_request::BOOL));
		$id = self::getAttribute('id', $attribs, [], we_base_request::INTLISTA);
		$cond = we_tag_condition::getCond((string) (self::getAttribute('condition', $attribs, 'we_lv_condition', we_base_request::RAW)));
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$desc = self::getAttribute('desc', $attribs, false, we_base_request::BOOL);
		$we_lv_desc = we_base_request::bool('we_lv_desc_' . $name, $desc);

		$we_offset = intval(self::getAttribute('offset', $attribs, 0, we_base_request::INT));
		$workspaceID = self::getAttribute('workspaceID', $attribs, self::getAttribute('workspaceid', $attribs, [], we_base_request::INTLISTA, true), we_base_request::INTLISTA);
		$we_lv_ws = we_base_request::intList('we_lv_ws_' . $name, $workspaceID);

		$we_lv_languages = we_base_request::stringList('we_lv_languages_' . $name, self::getAttribute('languages', $attribs, [], we_base_request::STRING_LIST));

		$triggerid = self::getAttribute('triggerid', $attribs, 0, we_base_request::INT);
		$docid = self::getAttribute('docid', $attribs, 0, we_base_request::INT);
		$customers = self::getAttribute('customers', $attribs, [], we_base_request::INTLISTA);
		$casesensitive = self::getAttribute('casesensitive', $attribs, false, we_base_request::BOOL);
		//$customer = self::getAttribute('customer', $attribs, false, we_base_request::BOOL);
		$we_lv_ct = we_base_request::string('we_lv_ct_' . $name, self::getAttribute('contenttypes', $attribs, '', we_base_request::STRING));

		$cols = self::getAttribute('cols', $attribs, 0, we_base_request::INT);
		$we_lv_se = we_base_request::bool('we_lv_se_' . $name, self::getAttribute('searchable', $attribs, true, we_base_request::BOOL));

		$calendar = self::getAttribute('calendar', $attribs, '', we_base_request::STRING);
		$datefield = self::getAttribute('datefield', $attribs, '', we_base_request::STRING);
		$date = self::getAttribute('date', $attribs, '', we_base_request::STRING);
		$weekstart = self::getAttribute('weekstart', $attribs, 'monday', we_base_request::STRING);

		// deprecated, because subfolders acts the other way arround as it should
		$subfolders = (isset($attribs['subfolders'])) ?
			!self::getAttribute('subfolders', $attribs, false, we_base_request::BOOL) :
			self::getAttribute('recursive', $attribs, true, we_base_request::BOOL);

		$we_lv_subfolders = isset($_REQUEST['we_lv_subfolders_' . $name]) ? (bool) $_REQUEST['we_lv_subfolders_' . $name] : $subfolders;

		$cfilter = self::getAttribute('cfilter', $attribs, '', we_base_request::STRING);
		$hidedirindex = self::getAttribute('hidedirindex', $attribs, TAGLINKS_DIRECTORYINDEX_HIDE, we_base_request::BOOL);
		$objectseourls = self::getAttribute('objectseourls', $attribs, TAGLINKS_OBJECTSEOURLS, we_base_request::BOOL);

		if($we_lv_languages){
			switch($we_lv_languages[0]){
				case 'self':
				case 'top':
					$we_lv_langguagesdoc = self::getDocForTag($we_lv_languages[0]);
					$we_lv_languages = [$we_lv_langguagesdoc->Language];
			}
		}

		$we_lv_calendar = we_base_request::string('we_lv_calendar_' . $name, $calendar);
		$we_lv_datefield = we_base_request::string('we_lv_datefield_' . $name, $datefield);
		$we_lv_date = we_base_request::string('we_lv_date_' . $name, ($date ?: date('Y-m-d')));
		$we_lv_weekstart = we_base_request::string('we_lv_weekstart_' . $name, $weekstart);

		if(in_array('we_doc', $we_lv_cats, true)){
			$we_lv_cats = [];
			//FIXME: make this more simple
			$we_lv_categoryids = array_filter(self::$doc->Category);
		}
		$lv = null;

		switch($type){
			case 'document':
				if($id === [0]){
					break;
				}
				$lv = new we_listview_document($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $doctypeids, $we_lv_cats, $we_lv_catOr, $casesensitive, $we_lv_ws, $we_lv_ct, $cols, $we_lv_se, $cond, $we_lv_calendar, $we_lv_datefield, $we_lv_date, $we_lv_weekstart, $we_lv_categoryids, $cfilter, $we_lv_subfolders, $customers, $id, $we_lv_languages, $we_lv_numorder, $hidedirindex, $triggerid);
				break;
			case 'search':
				$lv = new we_listview_search($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $doctypeids, $classid, $we_lv_cats, $we_lv_catOr, $casesensitive, $we_lv_ws, $triggerid, $cols, $cfilter, $we_lv_languages, $hidedirindex, $objectseourls);
				break;
			case 'object':
				if(!we_modules_info::isActive(we_modules_info::OBJECT)){
					echo self::modulMissingError('Object/DB', __CLASS__ . ' type="object"');
					return '';
				}
				if($id === [0]){
					break;
				}

				if(fB('SELECT 1 FROM ' . OBJECT_TABLE . ' WHERE ID=' . $classid)){
					$lv = new we_listview_object($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $classid, $we_lv_cats, $we_lv_catOr, $cond, $triggerid, $cols, $we_lv_se, $we_lv_calendar, $we_lv_datefield, $we_lv_date, $we_lv_weekstart, $we_lv_categoryids, $we_lv_ws, $cfilter, $docid, $customers, $id, $we_lv_languages, $hidedirindex, $objectseourls);
				} else {
					t_e('warning', 'Class with id=' . $classid . ' does not exist');
					return '';
				}
				break;
			case 'languagelink':
				$lv = self::langlink($attribs, $name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $cols, $hidedirindex, $objectseourls, $we_lv_subfolders);
				break;
			case 'customer':
				if(!we_modules_info::isActive(we_modules_info::CUSTOMER)){
					self::pre(new we_listview_pseudo());
					return self::modulMissingError('Customer', __CLASS__ . ' type="customer"');
				}
				$lv = new we_listview_customer($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $cond, $cols, $docid, $hidedirindex);
				break;
			case 'onlinemonitor':
				if(!we_modules_info::isActive(we_modules_info::CUSTOMER)){
					self::pre(new we_listview_pseudo());
					return self::modulMissingError('Customer', __CLASS__ . ' type="onlinemonitor"');
				}
				$lv = self::onlinemonitor($attribs, $name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $cols, $hidedirindex, $cond, $docid);
				break;
			case 'order':
				if(!we_modules_info::isActive(we_modules_info::SHOP)){
					self::pre(new we_listview_pseudo());
					return self::modulMissingError('Shop', __CLASS__ . ' type="order"');
				}
				$lv = new we_listview_shopOrder($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $cond, $cols, $docid, $hidedirindex);
				break;
			case 'orderitem':
				if(!we_modules_info::isActive(we_modules_info::SHOP)){
					self::pre(new we_listview_pseudo());
					return self::modulMissingError('Shop', __CLASS__ . ' type="orderitem"');
				}
				$orderid = self::getAttribute('orderid', $attribs, 0, we_base_request::INT);
				$lv = new we_listview_shopOrderitem($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $cond, $cols, $docid, $orderid, $hidedirindex);
				break;
			case 'multiobject':
				if(!we_modules_info::isActive(we_modules_info::OBJECT)){
					echo self::modulMissingError('Object/DB', __CLASS__ . ' type="multiobject"');
					break;
				}
				$name = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
				$lv = new we_listview_multiobject($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $we_lv_cats, $we_lv_catOr, $cond, $triggerid, $cols, $we_lv_se, $we_lv_calendar, $we_lv_datefield, $we_lv_date, $we_lv_weekstart, $we_lv_categoryids, $cfilter, $docid, $we_lv_languages, $hidedirindex, $objectseourls);
				break;
			case 'variant':
				$defaultname = self::getAttribute('defaultname', $attribs, '', we_base_request::STRING);
				$docId = self::getAttribute('documentid', $attribs, 0, we_base_request::INT);
				$objectId = self::getAttribute('objectid', $attribs, 0, we_base_request::INT) ?: (we_tag_listview::$current ? intval(we_tag_listview::$current->f(we_listview_base::ID_FIELD)) : 0);

				$lv = new we_listview_variants($name, $we_rows, $defaultname, $docId, $objectId, $we_offset, $hidedirindex, $objectseourls);
				break;
			case 'category':
				$parentid = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
				$parentidname = self::getAttribute('parentidname', $attribs, '', we_base_request::STRING);
//$categoryids="' . $categoryids . '";
//$parentid="' . $parentid . '";
				$lv = new we_listview_category($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $parentid, $categoryids, $cols, ($parentidname ?: ''), $hidedirindex);
				break;
			case 'collectionitems':
				/*
				 * priorities for $id:
				 * 1) self::$collectionID: coming from we_gallery plugin in tinymce
				 * 2) self::$doc->getElement($name): selection from tag <we:collectionSelect>
				 * 3) $id: attribute on listview: can thus be used as default from template
				 */
				$myid = (int) (self::$collectionID ?:
					((we_tag_listview::$current && (($collectionID = (int) (we_tag_listview::$current->f($name))) > 0)) ? $collectionID :
					((isset(self::$doc) && ($collectionID = self::$doc->getElementI($name))) ? $collectionID : $id[0])));

				if(!($collection = DB_WE::$inst->getHash('SELECT remTable,remClass FROM ' . VFILE_TABLE . ' WHERE ID=' . $myid))){
					t_e('warning', 'Collection with id=' . $myid . ' does not exist');
					self::pre(new we_listview_pseudo());
					return '';
				}

				switch($collection['remTable']){
					case 'tblFile':
						$lv = new we_listview_collectionItems($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $doctypeids, $we_lv_cats, $we_lv_catOr, $casesensitive, $we_lv_ws, $we_lv_ct, $cols, $we_lv_se, $cond, $we_lv_calendar, $we_lv_datefield, $we_lv_date, $we_lv_weekstart, $we_lv_categoryids, $cfilter, $we_lv_subfolders, $customers, [
							$myid], $we_lv_languages, $we_lv_numorder, $hidedirindex, $triggerid);
						break;
					case 'tblObjectFiles':
						if(!fB('SELECT 1 FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($collection['remClass']))){
							t_e('warning', 'Class with id=' . $collection['remClass'] . ' does not exist');
							return '';
						}
						$lv = new we_listview_collectionObjects($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, (int) $collection['remClass'], $we_lv_cats, $we_lv_catOr, $we_lv_ws, $cols, $we_lv_se, $cond, $we_lv_calendar, $we_lv_datefield, $we_lv_date, $we_lv_weekstart, $we_lv_categoryids, $cfilter, $docid, $customers, [
							$myid], $we_lv_languages, $hidedirindex, $triggerid, $objectseourls);
				}
				break;
			case 'sql':
				$lv = self::sql($attribs, $name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc);
				break;
			default:
				self::pre(new we_listview_pseudo());
				return self::parseError('Unknown type ' . $type, __CLASS__);
		}

		self::pre($lv ?? new we_listview_pseudo());
		return '';
	}

	protected static function langlink(array $attribs, string $name, int $we_rows, int $we_offset, string $we_lv_order, bool $we_lv_desc, int $cols, bool $hidedirindex, bool $objectseourls, bool $we_lv_subfolders): we_listview_langlink{
		$we_lv_pagelanguage = we_base_request::string('we_lv_pagelanguage_' . $name, self::getAttribute('pagelanguage', $attribs, '', we_base_request::STRING));
		$we_lv_langguagesdoc = self::getDocForTag($we_lv_pagelanguage);
		$we_lv_ownlanguage = $we_lv_langguagesdoc->Language;

		switch(we_tag_listview::$current ? get_class(we_tag_listview::$current) : ''){//@phan-suppress-current-line PhanPossiblyNullTypeArgumentInternal
			case 'we_listview_object':
				$we_lv_pageID = (int) we_tag_listview::$current->f(we_listview_base::ID_FIELD);
				$we_lv_linktype = OBJECT_FILES_TABLE;
				$we_lv_pagelanguage = $we_lv_pagelanguage === 'self' ?
					we_tag_listview::$current->f(we_listview_base::LANG_FIELD) :
					($we_lv_pagelanguage === 'top' ? $we_lv_ownlanguage : $we_lv_pagelanguage);
				$we_lv_ownlanguage = we_tag_listview::$current->f(we_listview_base::LANG_FIELD);
				break;
			default:
				$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
				$we_lv_pagelanguage = (in_array($we_lv_pagelanguage, ['self', 'top'], true) ? $we_lv_ownlanguage : self::getDocForTag($docAttr)->Language);

				/**
				 * Fix #9694
				 * attention: we can not check $we_lv_langguagesdoc instanceof we_contents_objectFile
				 * $we_lv_langguagesdoc is always instance of webEditionDocument because
				 * we need an webEdition Document to show webEdition object detail pages
				 */
				try{
					$we_lv_pageID = we_base_WE::getWeObj()->ID;
					$we_lv_linktype = OBJECT_FILES_TABLE;
				} catch (we_exception_base $ex){
					$we_lv_pageID = $we_lv_langguagesdoc->ID;
					$we_lv_linktype = FILE_TABLE;
				}
		}
		$showself = self::getAttribute('showself', $attribs, false, we_base_request::BOOL);

		return new we_listview_langlink($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $we_lv_linktype, $cols, $showself, [$we_lv_pageID], $we_lv_pagelanguage, $we_lv_ownlanguage, $hidedirindex, $objectseourls, $we_lv_subfolders);
	}

	public static function pre(we_listview_base $newLV): void{
		//add current lv to the stack
		if(self::$current){
			self::$lvArray[] = self::$current;
		}
		//set new lv
		self::$current = $newLV;
	}

	protected static function onlinemonitor(array $attribs, string $name, int $we_rows, int $we_offset, string $we_lv_order, bool $we_lv_desc, int $cols, bool $hidedirindex, string $cond, int $docid): we_listview_customerOnlineMonitor{
		//$id = we_base_request::int('we_omid', 0);
		$lastaccesslimit = self::getAttribute('lastaccesslimit', $attribs, 300, we_base_request::INT);
		$lastloginlimit = self::getAttribute('lastloginlimit', $attribs, 0, we_base_request::INT);
		return new we_listview_customerOnlineMonitor($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $cond, $cols, $docid, $lastaccesslimit, $lastloginlimit, $hidedirindex);
	}

	protected static function sql(array $attribs, string $name, int $we_rows, int $we_offset, string $we_lv_order, bool $we_lv_desc): we_listview_sql{
		$select = self::getAttribute('select', $attribs, '', we_base_request::STRING);
		$from = self::getAttribute('from', $attribs, '', we_base_request::STRING);
		$where = self::getAttribute('where', $attribs, '', we_base_request::STRING);
		$predefinedSQL = self::getAttribute('predefinedSQL', $attribs, '', we_base_request::RAW);

		return new we_listview_sql($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $select, $from, $where, $predefinedSQL);
	}

//this function is used by all tags adding elements to lvArray
	public static function post(): void{
		if(empty(self::$lvArray)){
			self::$current = null;
			return;
		}
		//get the last lv
		self::$current = array_pop(self::$lvArray);
	}

}
