<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: d08bde146d37 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-05-06 19:54 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_setVar extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false)/* : mixed */{
		self::attributeMissing($attribs, ['to' => false], __CLASS__);
		self::$doc = we_base_WE::getWeDoc();
		assert(self::$doc instanceof we_document_webEdition);

		$nameFrom = self::getAttribute('namefrom', $attribs, '', we_base_request::STRING);
		$nameTo = self::getAttribute('nameto', $attribs, $nameFrom, we_base_request::STRING);
		$to = self::getAttribute('to', $attribs, '', we_base_request::STRING);
		$propertyTo = self::getAttribute('propertyto', $attribs, false, we_base_request::BOOL);
		$striptags = self::getAttribute('striptags', $attribs, false, we_base_request::BOOL);
		$formnameTo = self::getAttribute('formnameto', $attribs, we_base_constants::DEFAULT_FORM, we_base_request::STRING);
		$prepareSQL = self::getAttribute('prepareSQL', $attribs, false, we_base_request::BOOL);

		$valueFrom = self::getValue($attribs);
		$valueFrom = ($striptags ? strip_tags($valueFrom) : $valueFrom);
		$valueFrom = ($prepareSQL ? DB_WE::$inst->escape($valueFrom) : $valueFrom);

		if(!$nameTo){
			self::attributeMissing($attribs, ['nameto' => false], __CLASS__);
		}

		switch($to){
			case 'object':
				if(!empty(we_tag_write::$objElements[$formnameTo])){
					if($propertyTo){
						we_tag_write::$objElements[$formnameTo]->$nameTo = $valueFrom;
					} else {
						we_tag_write::$objElements[$formnameTo]->setElementS($nameTo, $valueFrom);
					}
				}
				break;
			case 'document':
				if(!empty(we_tag_write::$docElements[$formnameTo])){
					if($propertyTo){
						we_tag_write::$docElements[$formnameTo]->$nameTo = $valueFrom;
					} else {
						we_tag_write::$docElements[$formnameTo]->setElementS($nameTo, $valueFrom);
					}
				}
				break;
			case 'top' :
			case 'self' :
				if($propertyTo){
					if($to === 'top'){
						we_base_WE::$mainDoc['doc']->$nameTo = $valueFrom;
					} else {
						self::$doc->$nameTo = $valueFrom;
					}
					break;
				}
			default:
				self::redirectTagOutput($valueFrom, $nameTo, $to);
		}
		return '';
	}

	/**
	 *
	 * @param array $attribs
	 * @return string
	 */
	protected static function getValue(array $attribs): string{
		if(isset($attribs['value'])){
			return (string) self::getAttribute('value', $attribs, '', we_base_request::RAW);
		}
		$from = self::getAttribute('from', $attribs, '', we_base_request::STRING);
		$propertyFrom = self::getAttribute('propertyfrom', $attribs, false, we_base_request::BOOL);
		$nameFrom = self::getAttribute('namefrom', $attribs, '', we_base_request::STRING);
		$typeFrom = self::getAttribute('typefrom', $attribs, 'text', we_base_request::STRING);
		$varType = $typeFrom === 'href' ? we_base_request::URL : self::getAttribute('varType', $attribs, we_base_request::STRING, we_base_request::STRING);
		$formnameFrom = self::getAttribute('formnamefrom', $attribs, we_base_constants::DEFAULT_FORM, we_base_request::STRING);

		switch($from){
			case 'request':
			case 'post':
			case 'get':
			case 'global':
			case 'session':
				return we_weTag_util::tag('var', ['type' => $from, '_name_orig' => $nameFrom, 'name' => $nameFrom, 'varType' => $varType]);
			case 'top':
				return (string) ($propertyFrom ?
						(we_base_WE::$mainDoc['doc']->$nameFrom ?? '') :
						we_base_WE::$mainDoc['doc']->getField(['name' => $nameFrom], $typeFrom, true)
						);
			case 'self':
				return (string) ($propertyFrom ?
						(self::$doc->$nameFrom ?? '') :
						self::$doc->getField(['name' => $nameFrom], $typeFrom, true)
						);
			case 'object':
			case 'document':
				return (string) ($propertyFrom ?
						($GLOBALS['we_' . $from][$formnameFrom]->$nameFrom ?? '') :
						(is_object($GLOBALS['we_' . $from][$formnameFrom]) && $GLOBALS['we_' . $from][$formnameFrom]->issetElement($nameFrom) ? $GLOBALS['we_' . $from][$formnameFrom]->getElement($nameFrom) : ''));
			case 'sessionfield':
				return (string) ($_SESSION['webuser'][$nameFrom] ?? '');
			case 'calendar':
				['code' => $langcode] = self::getOutLang($attribs);
				return we_tag_listview::$current->getCalendarFieldValue($nameFrom, $langcode);
			case 'listview':
				if(!we_tag_listview::$current){
					return self::parseError(g_l('parser', '[setVar_lv_not_in_lv]'), __CLASS__);
				}
				return we_weTag_util::tag('field', ['name' => $nameFrom, 'type' => $typeFrom]);
			case 'block':
				$nameFrom = self::getPostName($nameFrom);
				return (string) self::$doc->getField(['name' => $nameFrom], $typeFrom, true);
			case 'listdir':
				return (string) (we_tag_listdir::$position[$nameFrom] ?? '');
		}
		return '';
	}

}
