<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: b26067fc7281 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-02-04 13:57 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_writeVoting extends we_weTag_tag{
	/** @var ?int */
	public static $status = null;

	/** @var int */
	public static $successorID = 0;

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): void{
		$id = self::getAttribute('id', $attribs, 0, we_base_request::INT);
		$additionalFields = self::getAttribute('additionalfields', $attribs, [], we_base_request::INTLISTA);
		$allowredirect = self::getAttribute("allowredirect", $attribs, false, we_base_request::BOOL);
		$deletesessiondata = self::getAttribute("deletesessiondata", $attribs, false, we_base_request::BOOL);
		$writeto = self::getAttribute("writeto", $attribs, "voting", we_base_request::STRING);

		$vars = implode(',', array_keys($_REQUEST));
		$voting = $matches = [];
		if(preg_match_all('/_we_voting_answer_(' . ($id ?: '[0-9]+') . ')_?([0-9]+)?/', $vars, $matches)){
			foreach($matches[0] as $key => $value){
				$id = $matches[1][$key];
				if(!isset($voting[$id]) || !is_array($voting[$id])){
					$voting[$id] = [];
				}
				if(($dat = we_base_request::string($value)) !== ''){// Bug #6118: !empty geht hier nicht, da es die 0 nicht durch lässt
					$voting[$id][] = $dat;
				}
			}
		}
		$addFields = [];
		foreach($additionalFields as $field){
			if(($dat = we_base_request::string($field)) !== ''){
				$addFields[$field] = $dat;
			}
		}


		if($deletesessiondata){
			unset($_SESSION['_we_voting_sessionData']);
		}


		foreach($voting as $id => $value){
			$voting = new we_voting_voting((int) $id);
			if($voting->IsRequired && $value == []){
				self::$status = we_voting_voting::ERROR;
				if($voting->Log){
					$votingsession = (empty($_SESSION['_we_voting_sessionID']) ? 0 : $_SESSION['_we_voting_sessionID']);
					$voting->logVoting(we_voting_voting::ERROR, $votingsession, '', '', 0);
				}
				break;
			}
			if($writeto === 'voting'){
				self::$status = $voting->vote($value, $addFields);
				if(self::$status !== we_voting_voting::SUCCESS){
					break;
				}
			} else {
				self::$status = $voting->setSuccessor($value);
				if(self::$status !== we_voting_voting::SUCCESS){
					break;
				}
				$_SESSION['_we_voting_sessionData'][$id] = ['value' => $value, 'addFields' => $addFields];
			}
		}
		if($allowredirect && !we_base_WE::$mainDoc['doc']->InWebEdition && self::$successorID){
			$mypath = id2path(self::$successorID);
			if($mypath && $mypath != we_http_util::getScriptName()){
				header('Location: ' . $mypath); /* Redirect browser */

				/* Make sure that code below does not get executed when we redirect. */
				exit;
			}
		}
	}

}
