<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4f156532507d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-06 23:12 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_versions_fragment extends we_fragment_base{

	protected function doTask(): void{
		we_versions_version::todo($this->data);
	}

	protected function updateProgressBar(): void{
		$this->jsCmd->addCmd('setProgress', [
			'progress' => ((int) ((100 / count($this->alldata)) * (1 + $this->currentTask))),
			'name' => 'pb1',
			'text' => we_base_util::shortenPath($this->data["path"] . " - " . g_l('versions', '[version]') . " " . $this->data["version"], 60),
			'win' => 'wizbusy'
		]);
	}

	protected function finish(): void{
		if(!empty($_SESSION['weS']['versions']['logResetIds'])){
			$versionslog = new we_versions_log();
			$versionslog->saveVersionsLog($_SESSION['weS']['versions']['logResetIds'], we_versions_log::VERSIONS_RESET);
		}
		unset($_SESSION['weS']['versions']['logResetIds']);
		switch(we_base_request::string('type')){
			case 'delete_versions':
				$responseText = g_l('versions', '[deleteDateVersionsOK]');
				break;
			case 'reset_versions':
				$responseText = g_l('versions', '[resetAllVersionsOK]');
				break;
			default:
				$responseText = we_base_request::string('responseText', "");
		}
		$this->jsCmd->addMsg($responseText, we_base_util::WE_MESSAGE_NOTICE);
		$this->jsCmd->addCmd('reloadEditors');
		$this->jsCmd->addCmd('close');
	}

	/**
	 * Create and returns data Array with IDs and other information for the fragmment class for rebuilding documents
	 *
	 * @return array
	 */
	public static function getDocuments(string $type): array{
		switch($type){
			case 'delete_versions' :
				return self::getDocumentsDelete();
			case 'reset_versions' :
				return self::getDocumentsReset();
		}
		return [];
	}

	/**
	 * Create and returns data Array with IDs and other information for the fragmment class for rebuilding all documents and templates (Called from getDocuments())
	 *
	 * @return array
	 */
	private static function getDocumentsDelete(): array{
		$data = [];
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){

			foreach(DB_WE::$inst->getAllq($_SESSION['weS']['versions']['query']) as $rec){
				$data[] = [
					'ID' => $rec['ID'],
					'documentID' => $rec['documentID'],
					'type' => 'version_delete',
					'version' => $rec['version'],
					'logDate' => $rec['logDate'],
					'path' => $rec['Path'],
					'table' => $rec['documentTable'],
					'contenttype' => $rec['ContentType'],
					'text' => $rec['Text']
				];
			}
			unset($_SESSION['weS']['versions']['query']);
		}
		return $data;
	}

	private static function getDocumentsReset(): array{
		$data = [];
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){

			foreach(DB_WE::$inst->getAllq($_SESSION['weS']['versions']['query']) as $rec){
				$data[] = [
					'ID' => $rec['ID'],
					'documentID' => $rec['documentID'],
					'type' => 'version_reset',
					'version' => $rec['version'],
					'logDate' => $rec['logDate'],
					'path' => $rec['Path'],
					'table' => $rec['documentTable'],
					'contenttype' => $rec['ContentType'],
					'text' => $rec['Text']
				];
			}
			unset($_SESSION['weS']['versions']['query']);
		}
		return $data;
	}

}
