<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4836b978d48d $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-06-22 16:35 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_widget_pad extends we_widget_base{//Notizen

	public function __construct(string $curID = '', array $aProps = []){

	}

	/**
	 * @return array{string,array{string,string}}
	 */
	public function getInsertDiv(int $iCurrId, array $aProps, we_base_jsCmd $jsCmd): array{
		[$pad_header_enc, $pad_csv] = explode(',', $aProps[3]);

		$iFrmPadAtts = [
			'src' => WEBEDITION_DIR . 'we_cmd.php?' . http_build_query([
				'mod' => 'pad',
				'we_cmd' => [
					0 => 'widget_cmd',
					1 => 'reload',
					2 => $pad_csv . ' ',
					3 => ' ',
					4 => 'home',
					5 => $aProps[1] . ' ',
					6 => $pad_header_enc . ' ',
					7 => 'm_' . $iCurrId,
		]]),
			'id' => 'm_' . $iCurrId . '_inline',
			'style' => 'width:100%;height:287px'
		];

		$oTblDiv = str_replace('>', ' allowtransparency="true">', we_weTag_tag::htmlTag('iframe', $iFrmPadAtts, '', true));

		$aLang = [
			g_l('cockpit', '[notes]') . ' - ' . (base64_decode($pad_header_enc) ?: ''),
			''
		];
		return [$oTblDiv, $aLang];
	}

	/**
	 * @return array<string,mixed>
	 */
	public static function getDefaultConfig(): array{
		return [
			'width' => self::WIDTH_LARGE,
			'expanded' => 1,
			'height' => 307,
			'res' => 1,
			'cls' => 'blue',
			'csv' => base64_encode(g_l('cockpit', '[notepad_defaultTitle_DO_NOT_TOUCH]')) . ',30020',
			'dlgHeight' => 560,
			'isResizable' => false
		];
	}

	public static function showDialog(): void{
		[$jsFile, /* $oSelCls */, $oSctClsHTML] = self::getDialogPrefs();

		$sort = implode(we_html_element::br(),[
			we_html_forms::radiobutton('0', false, 'rdo_sort', g_l('cockpit', '[by_pubdate]')),
			we_html_forms::radiobutton('1', false, 'rdo_sort', g_l('cockpit', '[by_valid_from]')),
			we_html_forms::radiobutton('2', false, 'rdo_sort', g_l('cockpit', '[by_valid_until]')),
			we_html_forms::radiobutton('3', false, 'rdo_sort', g_l('cockpit', '[by_priority]')),
			we_html_forms::radiobutton('4', true, 'rdo_sort', g_l('cockpit', '[alphabetic]'))
		]);

		$display = implode(we_html_element::br(),[
			we_html_forms::radiobutton('0', true, 'rdo_display', g_l('cockpit', '[all_notes]')),
			we_html_forms::radiobutton('1', false, 'rdo_display', g_l('cockpit', '[only_valid]')),
		]);

		$date = implode(we_html_element::br(),[
			we_html_forms::radiobutton('0', true, 'rdo_date', g_l('cockpit', '[by_pubdate]')),
			we_html_forms::radiobutton('1', false, 'rdo_date', g_l('cockpit', '[by_valid_from]')),
			we_html_forms::radiobutton('2', false, 'rdo_date', g_l('cockpit', '[by_valid_until]'))
		]);


		$prio = new we_html_table(['class' => 'default'], 2, 2, [
			new we_html_tr([new we_html_td(we_html_forms::radiobutton('0', false, 'rdo_prio', g_l('cockpit', '[high]')), ['width' => '70', 'style' => 'padding-right:10px;']), new we_html_td('<i class="fa fa-dot-circle-o" style="color:red"></i>', ['width' => '20'])]),
			new we_html_tr([new we_html_td(we_html_forms::radiobutton('1', false, 'rdo_prio', g_l('cockpit', '[medium]'))), new we_html_td('<i class="fa fa-dot-circle-o" style="color:#F2F200"></i>')]),
			new we_html_tr([new we_html_td(we_html_forms::radiobutton('2', true, 'rdo_prio', g_l('cockpit', '[low]'))), new we_html_td('<i class="fa fa-dot-circle-o" style="color:green"></i>')]),
		]);

		$oSctValid = we_html_tools::select('sct_valid', [g_l('cockpit', '[always]'), g_l('cockpit', '[from_date]'), g_l('cockpit', '[period]')
						], 1, g_l('cockpit', '[always]'), false, ['style' => "width:120px;"], true, 120);


		[$pad_header_enc,] = explode(',', we_base_request::string('we_cmd', '', 1));
		$pad_header = base64_decode($pad_header_enc) ?: '';
		$all = DB_WE::$inst->getAllq('SELECT distinct(WidgetName) FROM ' . NOTEPAD_TABLE . ' WHERE UserID=' . we_users_user::getID(), true);
		'@phan-var string[] $all';
		$options = array_merge(
				[$pad_header => $pad_header, g_l('cockpit', '[change]') => g_l('cockpit', '[change]')], array_combine($all, $all) ?: []
		);

		$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[sorting]'), $sort))
						->space(we_html_multiIconBox::SPACE_MED)
				)
				->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[display]'), $display))
						->space(we_html_multiIconBox::SPACE_MED)
				)
				->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[display_date]'), $date))
						->space(we_html_multiIconBox::SPACE_MED)
				)
				->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[default_priority]', false, true), $prio->getHTML()))
						->space(we_html_multiIconBox::SPACE_MED)
				)
				->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[default_validity]', false, true), $oSctValid))
						->space(we_html_multiIconBox::SPACE_MED)
				)
				->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[title]'), we_html_tools::select('sct_title', array_unique($options), 1, "", false, ['id' => 'title'])))
						->space(we_html_multiIconBox::SPACE_MED)
				)
				->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[bg_color]', false, true), $oSctClsHTML))
				->space(we_html_multiIconBox::SPACE_MED)
		);

		echo we_html_tools::getHtmlTop(g_l('cockpit', '[notepad]'), $jsFile .
				we_html_element::jsScript(JS_DIR . 'widgets/pad.js', '', ['id' => 'loadVarWidget', 'data-widget' => setDynamicVar([
						'sObjId' => we_base_request::string('we_cmd', 'm_0', 0),
						'sTb' => '', //($isHome ? $title : g_l('cockpit', '[notes]') . " - " . $title ),
						'sType' => 'pad',
						'sInitProps' => '' //($isHome ? $sInitProps : '')
			])]), we_html_element::body(
						['class' => 'weDialogBody', 'onload' => "initDlg();"
						], we_html_element::form(['onsubmit' => "return false;"
								], (new we_html_multiIconBox($parts, 'padProps'))
										->marginLeft()
										->buttons(we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:save();"), we_html_button::create_button(we_html_button::PREVIEW, "javascript:preview();"), we_html_button::create_button(we_html_button::CLOSE, "javascript:exit_close();")))
										->headline(g_l('cockpit', '[notepad]'))
										->get()
		)));
	}

	private static function convertDate(string $strDate): string{
		$date = new DateTime($strDate);
		return $date->format(g_l('weEditorInfo', '[date_only_format]'));
	}

	private static function getDateSelector(string $label, string $name): string{
		return we_html_element::span(['class' => 'default', 'id' => $name . '_cell'], $label .
						we_html_tools::getNewDateInput($name, new DateTime(), we_html_tools::FORMAT_DATE)
		);
	}

	/**
	 * Creates the HTML code with the note list
	 *
	 * @param string $title
	 * @param string $q_sort
	 * @param int $bDate
	 * @param int $bDisplay
	 * @return string
	 */
	private static function getNoteList(string $title, string $q_sort, int $bDate, int $bDisplay): string{
		$DB_WE = we_database_base::$inst;
		$all = $DB_WE->getAllq('SELECT ID,WidgetName,UserID,Creation_Date,Title,Text,Priority,Valid,ValidFrom,ValidUntil FROM ' .
				NOTEPAD_TABLE .
				' WHERE
		WidgetName="' . $DB_WE->escape($title) . '" AND
		UserID=' . we_users_user::getID() .
				($bDisplay ? ' AND (
	Valid = "always" OR (
		Valid = "date" AND ValidFrom <= DATE_FORMAT(NOW(), "%Y-%m-%d")
	) OR (
		Valid = "period" AND ValidFrom <= DATE_FORMAT(NOW(), "%Y-%m-%d") AND ValidUntil >= DATE_FORMAT(NOW(),"%Y-%m-%d")
	)
)' : ''
				) .
				' ORDER BY ' . $q_sort);
		$notes = new we_html_table(['style' => 'width:100%;padding:0 5px;', 'class' => 'default'], 0, 3);
		$hidden = '';
		$rcd = 0;
		foreach($all as $data){
			$data['ValidUntil'] = ($data['ValidUntil'] === '3000-01-01' ? '' : $data['ValidUntil']);
			$hidden .= we_html_element::hidden('notepadData[' . $rcd . ']', setDynamicVar($data), 'notepadData[' . $rcd . ']');
// fixme: handle data vars
			$validity = $data['Valid'];
			switch($bDate){
				case 1:
					$showDate = ($validity === 'always' ? '-' : self::convertDate($data['ValidFrom']));
					break;
				case 2:
					$showDate = ($validity === 'always' || $validity === 'date' ? '-' : self::convertDate($data['ValidUntil']));
					break;
				default:
					$showDate = self::convertDate($data['Creation_Date']);
			}

			$today = date('Ymd');
			$vFrom = str_replace('-', '', $data['ValidFrom']);
			$vTill = str_replace('-', '', $data['ValidUntil']);
			if($bDisplay == 1 && $data['Valid'] != 'always'){
				if($data['Valid'] === 'date'){
					if($today < $vFrom){
						continue;
					}
				} else {
					if($today < $vFrom || $today > $vTill){
						continue;
					}
				}
			}
			$showTitle = strtr($data['Title'], ['<' => '&lt;', '>' => '&gt;', '\'' => '&#039;', '"' => '&quot;']);
			switch($data['Priority']){
				case 'high':
					$color = 'red';
					break;
				default:
				case 'medium':
					$color = 'yellow';
					break;
				case 'low':
					$color = 'green';
					break;
			}
			$notes->addRow(new we_html_tr([
						new we_html_td('<i class="fa fa-dot-circle-o" style="color:' . $color . '"></i>', ['style' => "width:15px;height:20px;vertical-align:middle"]),
						new we_html_td($showDate, ['style' => "width:60px;padding-left:5px;vertical-align:middle;text-align:center", 'class' => "middlefont"]),
						new we_html_td($showTitle, ['style' => "padding-left:5px;vertical-align:middle", 'class' => "middlefont"]),
							],
							['style' => 'cursor:pointer;', 'id' => $rcd . '_tr', 'onmouseover' => "fo=document.forms[0];if(fo.elements.mark.value==''){setColor(this," . $rcd . ",'#EDEDED');}",
						'onmouseout' => "if(document.forms[0].elements.mark.value===''){setColor(this," . $rcd . ",'#FFFFFF');}", 'onmousedown' => "selectNote(" . $rcd . ");"]));
			$rcd++;
		}
		return $notes->getHtml() . $hidden;
	}

	public function showPreview(): void{
		$sInitProps = substr(we_base_request::string('we_cmd', '', 0), -5); //binary data
		$bSort = $sInitProps[0];
		$bDisplay = (int) $sInitProps[1];
		$bDate = (int) $sInitProps[2];
		//$bPrio = intval($sInitProps{3});
		//$bValid = $sInitProps{4};
		$title = base64_decode(we_base_request::_(we_base_request::RAW, 'we_cmd', '', 4)) ?: '';
		$command = we_base_request::string('we_cmd', '', 2);

		switch($command){
			case 'delete' :
				DB_WE::$inst->query('DELETE FROM ' . NOTEPAD_TABLE . ' WHERE ID=' . we_base_request::int('we_cmd', 0, 1));
				break;
			case 'update' :
				[$q_ID, $q_Title, $q_Text, $q_Priority, $q_Valid, $q_ValidFrom, $q_ValidUntil] = explode(';', we_base_request::string('we_cmd', '', 1));
				DB_WE::$inst->query('UPDATE ' . NOTEPAD_TABLE . ' SET ' . we_database_base::arraySetter([
							'Title' => base64_decode($q_Title) ?: '',
							'Text' => base64_decode($q_Text) ?: '',
							'Priority' => $q_Priority,
							'Valid' => $q_Valid,
							'ValidFrom' => ($q_Valid === 'always' ? sql_function('CURRENT_DATE()') : $q_ValidFrom),
							'ValidUntil' => ($q_Valid === 'always' || $q_Valid === 'date' ? '3000-01-01' : $q_ValidUntil)
						]) . ' WHERE ID = ' . intval($q_ID));
				break;
			case 'insert' :
				[$q_Title, $q_Text, $q_Priority, $q_Valid, $q_ValidFrom, $q_ValidUntil] = explode(';', we_base_request::string('we_cmd', '', 1));
				DB_WE::$inst->query('INSERT INTO ' . NOTEPAD_TABLE . ' SET ' . we_database_base::arraySetter([
							'WidgetName' => $title,
							'UserID' => we_users_user::getID(),
							'Title' => base64_decode($q_Title) ?: '',
							'Text' => base64_decode($q_Text) ?: '',
							'Priority' => $q_Priority,
							'Valid' => $q_Valid,
							'ValidFrom' => ($q_Valid === 'always' ? sql_function('CURRENT_DATE()') : $q_ValidFrom),
							'ValidUntil' => ($q_Valid === 'always' || $q_Valid === 'date' ? '3000-01-01' : $q_ValidUntil)
				]));
				break;
		}

		switch($bSort){
			case 1:
				$q_sort = 'Priority DESC, Title';
				break;
			case 2:
				$q_sort = 'ValidFrom, Title';
				break;
			case 3:
				$q_sort = 'Title';
				break;
			case 4:
				$q_sort = 'ValidUntil, Title';
				break;
			default:
				$q_sort = 'Creation_Date,Title';
		}

// validity settings
		$sctValid = we_html_tools::select('sct_valid', [g_l('cockpit', '[always]'), g_l('cockpit', '[from_date]'), g_l('cockpit', '[period]')
						], 1, [g_l('cockpit', '[always]')], false, ['style' => "width:100px;", 'onchange' => "toggleTblValidity()"], true, 100, 'middlefont');
		$oTblValidity = self::getDateSelector(g_l('cockpit', '[from]'), 'f_ValidFrom') . ' ' . self::getDateSelector(g_l('cockpit', '[until]'), 'f_ValidUntil');
		$oTblPeriod = new we_html_table(['width' => '100%', 'class' => 'default'], 1, 2);
		$oTblPeriod->setCol(0, 0, $sctValid, ['class' => 'middlefont']);
		$oTblPeriod->setCol(0, 1, $oTblValidity, ['style' => "text-align:right"]);

// Edit note prio settings
		$rdoPrio = [we_html_forms::radiobutton('0', false, 'rdo_prio', '<i class="fa fa-dot-circle-o" style="color:red;margin-left:5px;" title="' . g_l('cockpit', '[high]') . '" ></i>', true, 'middlefont'),
			we_html_forms::radiobutton('1', false, 'rdo_prio', '<i class="fa fa-dot-circle-o" style="color:#F2F200;margin-left:5px;" title="' . g_l('cockpit', '[medium]') . '"></i>', true, 'middlefont'),
			we_html_forms::radiobutton('2', true, 'rdo_prio', '<i class="fa fa-dot-circle-o" style="color:green;margin-left:5px;" title="' . g_l('cockpit', '[low]') . '"></i>', true, 'middlefont')
		];
		$oTblPrio = new we_html_table(['class' => 'default'], 1, 3);
		$oTblPrio->setCol(0, 0, $rdoPrio[0]);
		$oTblPrio->setCol(0, 1, $rdoPrio[1]);
		$oTblPrio->setCol(0, 2, $rdoPrio[2]);

// Edit note buttons
		$delete_button = we_html_button::create_button(we_html_button::DELETE, "javascript:deleteNote();", '', "", true, false);
		$cancel_button = we_html_button::create_button(we_html_button::CANCEL, "javascript:cancelNote();");
		$save_button = we_html_button::create_button(we_html_button::SAVE, "javascript:saveNote();");
		$buttons = we_html_button::position_yes_no_cancel($delete_button, $cancel_button, $save_button);

// Edit note dialog
		$oTblProps = new we_html_table(['width' => '100%'], 5, 2);
		$oTblProps->setCol(0, 0, g_l('cockpit', '[valid]') . '&nbsp;', ['class' => 'middlefont']);
		$oTblProps->setCol(0, 1, $oTblPeriod->getHTML(), ['colspan' => '2', 'style' => 'text-align:right']);
		$oTblProps->setCol(1, 0, g_l('cockpit', '[prio]'), ['class' => 'middlefont',]);
		$oTblProps->setCol(1, 1, $oTblPrio->getHTML());
		$oTblProps->setCol(2, 0, g_l('cockpit', '[title]'), ['class' => 'middlefont',]);
		$oTblProps->setCol(2, 1, we_html_tools::textInput('props_title', '', 255, [], 'text', '100%'));
		$oTblProps->setCol(3, 0, g_l('cockpit', '[note]'), ['class' => 'middlefont', 'style' => 'vertical-align:top;']);
		$oTblProps->setCol(3, 1, we_html_element::textArea([
					'name' => 'props_text',
					'id' => 'previewCode',
					'style' => 'width:100%;height:70px;',
					'class' => 'wetextinput',
						], ""));
		$oTblProps->setCol(4, 0, $buttons, ['colspan' => '2']);

// Button: add note
		$oTblBtnProps = we_html_button::create_button('fa:btn_add_note,fa-plus,fa-lg fa-newspaper-o', "javascript:displayNote();");

// Table with the note list
		$oPad = new we_html_table([
			'style' => 'table-layout:fixed;width:100%;padding-top:6px;padding-bottom:6px;background-color:white;',
			'class' => 'default'
				], 1, 1);

		$oPad->setCol(0, 0, we_html_element::div(['id' => 'notices'], self::getNoteList($title, $q_sort, $bDate, $bDisplay)), ['colspan' => '3', 'class' => 'cl_notes']);

		$notepad = $oPad->getHTML() .
				we_html_element::div(['id' => 'props'], $oTblProps->getHTML()) .
				we_html_element::div(['id' => 'view'], $oTblBtnProps);

		$isHome = ($command === 'home');
		$jsCmd = new we_base_jsCmd();
		$jsCmd->addCmd('initPreview', [
			'id' => we_base_request::string('we_cmd', '', 5),
			'type' => 'pad',
			'tb' => ($isHome ? $title : g_l('cockpit', '[notes]') . ' - ' . $title )
		]);

		echo we_html_tools::getHtmlTop(g_l('cockpit', '[notepad]'), we_html_element::cssLink(CSS_DIR . 'pad.css') .
				we_html_element::jsScript(JS_DIR . 'widgets/pad.js', '', [
					'id' => 'loadVarWidget',
					'data-widget' => setDynamicVar([
						'sObjId' => we_base_request::string('we_cmd', 'm_0', 5),
						'sTb' => ($isHome ? $title : g_l('cockpit', '[notes]') . ' - ' . $title ),
						'sType' => 'pad',
						'sInitProps' => ($isHome ? $sInitProps : '')
			])]) .
				$jsCmd->getCmds(), we_html_element::body(
						[
							'onload' => "toggleTblValidity();"
						], we_html_element::form(['style' => 'display:inline;'], we_html_element::div(
										['id' => 'pad'], $notepad .
										we_html_element::hidden('mark', '')
		))));
	}

}
