<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d938a246a5b9 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-03-29 12:22 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_widget_wfl extends we_widget_base{//Workflow

	public function __construct(string $curID = '', array $aProps = []){
		if(!we_modules_info::isActive(we_modules_info::WORKFLOW)){
			throw new we_exception_widget();
		}
	}

	/**
	 * @return array{string,array{string,string}}
	 */
	public function getInsertDiv(int $iCurrId, array $aProps, we_base_jsCmd $jsCmd): array{
		$oTblCont = new we_html_table([], 0, 2);
		foreach(array_filter([FILE_TABLE, we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE]) ? OBJECT_FILES_TABLE : '']) as $table){
			'@phan-var string $table';
			$wsArr = we_users_util::get_ws($table);
			//get workspace query
			if($wsArr){
				$childs = array_values($wsArr);
				we_base_util::readChilds($wsArr, $childs, $table, true, '', 'ContentType', we_base_ContentTypes::FOLDER_TYPES);
				$wsQuery = ' t.ParentID IN(' . implode(',', $childs) . ') ';
			} else {
				$wsQuery = '';
			}
			$all = DB_WE::$inst->getAllq('SELECT
	t.ContentType,
	t.ID,
	t.Path,
	t.IsPublished
FROM
	' . WORKFLOW_TASK_TABLE . ' task JOIN
	' . WORKFLOW_DOC_TASK_TABLE . ' dtask ON task.ID=dtask.workflowTaskID AND dtask.Status="' . we_workflow_documentTask::STATUS_NEW . '" JOIN
	' . WORKFLOW_DOC_STEP_TABLE . ' s ON dtask.documentStepID=s.ID JOIN
	' . WORKFLOW_DOC_TABLE . ' d ON d.DocumentTable="' . stripTblPrefix($table) . '" AND s.workflowDocID=d.ID AND d.Status="' . we_workflow_document::STATUS_ACTIVE . '" JOIN
	' . $table . ' t ON t.ID=d.documentID
WHERE
task.userID=' . we_users_user::getID() . ' AND
	(
		t.ContentType IN ("' . we_base_ContentTypes::WEDOCUMENT . '","' . we_base_ContentTypes::HTML . '","' . we_base_ContentTypes::OBJECT_FILE . '")
	)' .
					($wsQuery ? ' AND (' . $wsQuery . ') ' : '') . '
ORDER BY
	t.Path
LIMIT 100');

			foreach($all as $cur){
				$wfDat = '';
				if(!empty($cur['wforder'])){
					try{
						$step = we_workflow_utility::findLastActiveStep($cur['ID'], $table) + 1;
						$steps = we_workflow_utility::getNumberOfSteps($cur['ID'], $table);
						$wfDat = $step . '&nbsp;' . g_l('exchange', '[step_of]') . '&nbsp;' . $steps . '&nbsp;<i class="fa fa-lg fa-circle" style="color:#006DB8;"></i>';
					}catch(we_exception_base $e){

					}
				}
				$oTblCont->addRow(new we_html_tr([
							new we_html_td('', ['class' => 'upbIcon', 'data-contenttype' => $cur['ContentType']]),
							new we_html_td($cur['Path'] . $wfDat, ['class' => 'upbEntry middlefont ' . ($cur['IsPublished'] ? 'changed' : 'notpublished'), 'onclick' => "WE().layout.weEditorFrameController.openDocument('" . $table . "'," . $cur['ID'] . ",'" . $cur['ContentType'] . "')", 'title' => $cur['Path']])]));
			}
		}
		$cfg = self::getDefaultConfig();
		if($oTblCont->isEmpty()){
			$oTblCont->addRow(new we_html_tr([new we_html_td(), new we_html_td(g_l('modules_workflow', '[noDocuments]'))]));
		}

		$oTblDiv = we_html_element::div(['id' => 'm_' . $iCurrId . '_inline',
					'style' => 'height:' . ($cfg['height'] - 25) . 'px;overflow:auto;'
						], $oTblCont->getHtml());

		return [$oTblDiv, [g_l('menu_global', '[widget][workflow]'), '']];
	}

	/**
	 * @return array<string,mixed>
	 */
	public static function getDefaultConfig(): array{
		return [
			'width' => self::WIDTH_SMALL,
			'expanded' => 0,
			'height' => 210,
			'res' => 0,
			'cls' => 'orange',
			'csv' => '',
			'dlgHeight' => 140,
			'isResizable' => true
		];
	}

	public static function showDialog(): void{
		[$jsFile, $oSelCls] = self::getDialogPrefs();

		$parts = (new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine('', $oSelCls->getHTML()));


		$sTblWidget = (new we_html_multiIconBox($parts, 'wflProps'))
				->marginLeft()
				->buttons(we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:save();"), we_html_button::create_button(we_html_button::PREVIEW, "javascript:preview();"), we_html_button::create_button(we_html_button::CLOSE, "javascript:exit_close();")))
				->headline(g_l('menu_global', '[widget][workflow]'))
				->get();

		echo we_html_tools::getHtmlTop(g_l('menu_global', '[widget][workflow]'), $jsFile, we_html_element::body(
						['class' => 'weDialogBody', 'onload' => "init();"
						], we_html_element::form([], $sTblWidget)));
	}

	public function showPreview(): void{

	}

}
