<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 24bf12724cb1 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-09-03 21:17 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Code Snipptes are used in templates inside webEdition
 *
 * @see Parser.php
 * @see dtd:http://docs.oasis-open.org/dita/v1.0.1/dtd/topic.dtd
 *
 */
class we_wizard_codeSnippet{
	/** @var string Name of the Snippet	 */
	private $title = '';

	/** @var string Description of the snippet	 */
	private $shortdesc = '';

	/** @var string Author of the snippet	 */
	private $author = '';

	/** @var string Snippet code */
	private $code = '';

	/** @var string */
	public $folder = '';

	/**
	 * initialize the snippet from an xml file
	 *
	 * @param string $file
	 */
	public function __construct(string $file){
		$this->readJSON($file);
	}

	private function readJSON(string $file): void{
		$data = we_unserialize(we_base_file::load($file));
		foreach($data as $key => $d){
			assert(is_string($key));
			$this->{$key} = is_array($d) ? g_l($d[0], $d[1]) : $d;
		}
	}

	private function updateCode(): string{
		$ret = $this->code;
		$matches = [];
		if(preg_match_all('|__GL\(([^)]+)\)__|', $ret, $matches)){
			foreach($matches[1] as $match){
				$ret = str_replace('__GL(' . $match . ')__', g_l('snippet', $match), $ret);
			}
		}
		return $ret;
	}

	/**
	 * get the snippet name
	 *
	 * @return string
	 */
	function getName(): string{
		return $this->title;
	}

	/**
	 * get the snippet description
	 *
	 * @return string
	 */
	function getDescription(): string{
		return $this->shortdesc;
	}

	/**
	 * get the snippet author
	 *
	 * @return string
	 */
	function getAuthor(): string{
		return $this->author;
	}

	/**
	 * get the snippet code
	 *
	 * @return string
	 */
	function getCode(): string{
		return $this->updateCode();
	}

}
