<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: ef93478307da $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-02 17:13 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * WorkfFlow Document Task definition
 *
 * This class describe document task in workflow process
 *
 */
class we_workflow_documentTask extends we_workflow_base{
	const STATUS_NEW = 'new';
	const STATUS_APPROVED = 'approved';
	const STATUS_CANCELED = 'canceled';

	/** @var int workflow document task ID */
	var $ID = 0;

	/** @var int workflow document step ID */
	var $documentStepID = 0;

	/** @var int workflow task ID */
	var $workflowTaskID = 0;

	/** @var ?DateTime date when task is done */
	protected $Mod_Date = null;

	/** @var string Status of document task */
	var $Status = self::STATUS_NEW;

	/**
	 * Default Constructor
	 */
	function __construct(int $wfDocumentTask = 0){
		parent::__construct(WORKFLOW_DOC_TASK_TABLE);

		$this->persistent_slots = [
			'ID' => we_base_request::INT,
			'documentStepID' => we_base_request::INT,
			'workflowTaskID' => we_base_request::INT,
			'Mod_Date' => we_base_request::DATE_TIME,
			'Status' => we_base_request::STRING,
		];

		if($wfDocumentTask){
			$this->load($wfDocumentTask);
		}
	}

	public function approve(): void{
		$this->Status = self::STATUS_APPROVED;
		$this->Mod_Date = new DateTime();
	}

	public function decline(): void{
		$this->Status = self::STATUS_CANCELED;
		$this->Mod_Date = new DateTime();
	}

	//--------------------------------STATIC FUNCTIONS ------------------------------

	/**
	 * returns all tasks for workflow step
	 * @param int $workflowDocumentStep
	 * @return we_workflow_documentTask[]
	 */
	static function getAllTasks(int $workflowDocumentStep): array{
		$all = DB_WE::$inst->getAllq('SELECT ID FROM ' . WORKFLOW_DOC_TASK_TABLE . ' WHERE documentStepID=' . $workflowDocumentStep . ' ORDER BY ID', true);

		$docTasks = [];
		foreach($all as $cur){
			$docTasks[] = new self((int) $cur);
		}
		return $docTasks;
	}

	/**
	 * creates all tasks for workflow step
	 *
	 */
	public static function createAllTasks(int $workflowStepID): array{
		$all = DB_WE::$inst->getAllq('SELECT ID FROM ' . WORKFLOW_TASK_TABLE . ' WHERE stepID=' . $workflowStepID . ' ORDER BY ID', true);
		'@phan-var int[] $all';
		$docTasks = [];
		foreach($all as $cur){
			$docTasks[] = self::createTask($cur);
		}
		return $docTasks;
	}

	/**
	 * Create task
	 */
	private static function createTask(int $id): we_workflow_documentTask{
		$docTask = new we_workflow_documentTask();
		$docTask->workflowTaskID = $id;
		return $docTask;
	}

}
