/* global top, WE */

/**
 * webEdition CMS
 *
 * $Rev: 36ad0e3aa620 $
 * $Author: Lukas Imhof $
 * $Date: 2021-09-06 19:26 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
var tmp = WE().util.getDynamicVar(document, 'loadVarSelectors', 'data-selector');
//FIMXE: remove when selectorbody is not an iframe anymore
var fileSelect = Object.prototype.toString.call(tmp) === '[object Array]' ? fileSelect : tmp;
WE().util.loadConsts(document, "g_l.fileselector");
WE().util.loadConsts(document, "selectors");

var entries = [];
var clickCount = 0;
WE().util.dialogTop(window).metaKeys = {
	ctrl: false,
	shift: false,
	doubleClick: false,
	inputClick: false,
	doubleTout: null
};

function applyOnEnter(evt) {
	var _elemName = "target";
	if (evt.srcElement !== undefined) { // IE
		_elemName = "srcElement";
	}

	if (!(evt[_elemName].tagName === "SELECT" ||
					(evt[_elemName].tagName === "INPUT" && evt[_elemName].name !== "fname")
					)) {
		WE().util.dialogTop(window).press_ok_button();
		return true;
	}

}
function closeOnEscape() {
	WE().util.dialogTop(window).exit_close();
}

function goBackDir() {
	setDir(WE().util.dialogTop(window).fileSelect.data.parentID);
}

function getEntry(id) {
	for (let entry of WE().util.dialogTop(window).entries) {
		if (entry.ID == id) {
			return entry;
		}
	}
	return {
		ID: 0,
		icon: "",
		text: "/",
		isFolder: 1,
		path: "/"
	};
}

function clearEntries() {
	entries = [];
}

function exit_close() {
	WE().util.dialogTop(window).doClose();
}

function doClick(id, ct) {
	if (ct == 1) {
		if (WE().util.dialogTop(window).metaKeys.doubleClick) {
			setDir(id);
			window.setTimeout(function () {
				WE().util.dialogTop(window).metaKeys.doubleClick = false;
			}, 400);
		}
	} else if (WE().util.dialogTop(window).fileSelect.options.multiple) {
		if (WE().util.dialogTop(window).metaKeys.shift) {
			var oldid = WE().util.dialogTop(window).fileSelect.data.currentID;
			var currendPos = getPositionByID(id);
			var firstSelected = getFirstSelected();

			if (currendPos > firstSelected) {
				selectFilesFrom(firstSelected, currendPos);
			} else if (currendPos < firstSelected) {
				selectFilesFrom(currendPos, firstSelected);
			} else {
				selectFile(id);
			}
			WE().util.dialogTop(window).fileSelect.data.currentID = oldid;
		} else if (!WE().util.dialogTop(window).metaKeys.ctrl) {
			selectFile(id);
		} else if (isFileSelected(id)) {
			unselectFile(id);
		} else {
			selectFile(id);
		}
	} else {
		selectFile(id);

	}
	if (WE().util.dialogTop(window).metaKeys.ctrl) {
		WE().util.dialogTop(window).metaKeys.ctrl = 0;
	}
	if (WE().util.dialogTop(window).metaKeys.shift) {
		WE().util.dialogTop(window).metaKeys.shift = 0;
	}
}

function setDir(id) {
	var e = getEntry(id);
	var fspath = top.document.getElementById('fspath');
	WE().util.dialogTop(window).fileSelect.data.currentID = id;
	WE().util.dialogTop(window).fileSelect.data.currentDir = id;
	WE().util.dialogTop(window).fileSelect.data.currentPath = e.path;
	WE().util.dialogTop(window).fileSelect.data.currentText = e.text;
	WE().util.dialogTop(window).document.getElementsByName("fname")[0].value = e.text;
	WE().util.dialogTop(window).fscmd.location.replace(WE().util.dialogTop(window).queryString(WE().consts.selectors.CMD, id));
	if (fspath) {
		fspath.innerHTML = e.path;
	}
}

function setRootDir() {
	setDir(WE().util.dialogTop(window).fileSelect.options.rootDirID);
}

function selectFile(id) {
	var a = WE().util.dialogTop(window).document.getElementsByName("fname")[0];
	var fspath = top.document.getElementById('fspath');

	if (id) {
		var e = getEntry(id);

		if (a.value != e.text &&
						a.value.indexOf(e.text + ",") === -1 &&
						a.value.indexOf("," + e.text + ",") === -1 &&
						a.value.indexOf("," + e.text + ",") === -1) {
			a.value = a.value ? (a.value + "," + e.text) : e.text;
			if (fspath) {
				fspath.innerHTML = e.path;
			}
		}
		if (WE().util.dialogTop(window).fsbody.document.getElementById("line_" + id)) {
			WE().util.dialogTop(window).fsbody.document.getElementById("line_" + id).classList.add("selected");
		}
		WE().util.dialogTop(window).fileSelect.data.currentPath = e.path;
		WE().util.dialogTop(window).fileSelect.data.currentID = id;
	} else {
		a.value = "";
		WE().util.dialogTop(window).fileSelect.data.currentPath = "";
	}
}

function addEntry(ID, text, isFolder, path, ct) {
	entries.push({
		ID: ID,
		text: text,
		isFolder: isFolder,
		path: path,
		contentType: ct
	});
}

function writeBody(d) {
	let body = '';
	for (let entry of entries) {
		body += '<tr' + ((entry.ID == WE().util.dialogTop(window).fileSelect.data.currentID) ? ' class="selected"' : '') + ' id="line_' + entry.ID + '"' + ' onclick="return selectorOnClick(event,' + entry.ID + ');"' + (entry.isFolder ? ' ondblclick="return selectorOnDblClick(' + entry.ID + ');"' : '') + ' >' +
			'<td class="selector selectoricon">' + WE().util.getTreeIcon(entry.contentType, false) + '</td>' +
			'<td class="selector filename"  title="' + entry.text + '"><div class="cutText">' + entry.text + '</div></td>' +
						'<td class="selector remain"></td></tr>';
	}
	d.innerHTML = '<table class="selector sort" data-skipSort="0"><thead><tr>' + WE().util.dialogTop(window).fileSelect.options.selectorHeader + '</tr></thead><tbody>' + body + '</tbody></table>';
	window.fsbody.$("table.sort").each(WE().layout.applyTableSort);
}

function getFirstSelected() {
	for (let i = 0; i < entries.length; i++) {
		if (WE().util.dialogTop(window).fsbody.document.getElementById("line_" + entries[i].ID).classList.contains("selected")) {
			return i;
		}
	}
	return -1;
}

function unselectFile(id) {
	let e = getEntry(id);
	WE().util.dialogTop(window).fsbody.document.getElementById("line_" + id).style.classList.remove("selected");

	let foo = WE().util.dialogTop(window).document.getElementsByName("fname")[0].value.split(/,/);

	for (let text of foo) {
		if (text == e.text) {
			text = "";
			break;
		}
	}
	let str = "";
	for (let text of foo) {
		if (text) {
			str += text + ",";
		}
	}
	str = str.replace(/(.*),$/, "$1");
	WE().util.dialogTop(window).document.getElementsByName("fname")[0].value = str;
}


function selectFilesFrom(from, to) {
	unselectAllFiles();
	for (let i = from; i <= to; i++) {
		selectFile(entries[i].ID);
	}
}

function getPositionByID(id) {
	for (var i = 0; i < entries.length; i++) {
		if (entries[i].ID == id) {
			return i;
		}
	}
	return -1;
}

function isFileSelected(id) {
	return (WE().util.dialogTop(window).fsbody.document.getElementById("line_" + id).classList.contains("selected"));
}

function unselectAllFiles() {
	for (let entry of entries) {
		let elem = WE().util.dialogTop(window).fsbody.document.getElementById("line_" + entry.ID);
		if (elem) {
			elem.classList.remove("selected");
		}
	}
	WE().util.dialogTop(window).document.getElementsByName("fname")[0].value = "";
	WE().util.dialogTop(window).delBut(false);
	if (WE().util.dialogTop(window).showPreview) {
		WE().util.dialogTop(window).showPreview(0);
	}
}

function queryString(what, id) {
	return WE().util.dialogTop(window).fileSelect.options.formtarget +
					'&we_cmd[what]=' + what +
					'&we_cmd[table]=' + WE().util.dialogTop(window).fileSelect.options.table +
					'&we_cmd[id]=' + id +
					"&we_cmd[filter]=" + WE().util.dialogTop(window).fileSelect.data.currentType;
}

function fillIDs() {
	WE().util.dialogTop(window).fileSelect.data.allIDs = [];
	WE().util.dialogTop(window).fileSelect.data.allPaths = [];
	WE().util.dialogTop(window).fileSelect.data.allTexts = [];
	WE().util.dialogTop(window).fileSelect.data.allIsFolder = [];

	for (var i = 0; i < entries.length; i++) {
		if (isFileSelected(entries[i].ID)) {
			WE().util.dialogTop(window).fileSelect.data.allIDs.push(entries[i].ID);
			WE().util.dialogTop(window).fileSelect.data.allPaths.push(entries[i].path);
			WE().util.dialogTop(window).fileSelect.data.allTexts.push(entries[i].text);
			WE().util.dialogTop(window).fileSelect.data.allIsFolder.push(entries[i].isFolder);
		}
	}
	if (WE().util.dialogTop(window).fileSelect.data.currentID !== "" && WE().util.dialogTop(window).fileSelect.data.allIDs.indexOf(WE().util.dialogTop(window).fileSelect.data.currentID) === -1) {
		WE().util.dialogTop(window).fileSelect.data.allIDs.push(WE().util.dialogTop(window).fileSelect.data.currentID);
	}
	if (WE().util.dialogTop(window).fileSelect.data.currentPath !== "" && WE().util.dialogTop(window).fileSelect.data.allPaths.indexOf(WE().util.dialogTop(window).fileSelect.data.currentPath) === -1) {
		WE().util.dialogTop(window).fileSelect.data.allPaths.push(WE().util.dialogTop(window).fileSelect.data.currentPath);
		WE().util.dialogTop(window).fileSelect.data.allTexts.push(we_makeTextFromPath(WE().util.dialogTop(window).fileSelect.data.currentPath));
	}
}

function we_makeTextFromPath(path) {
	var position = path.lastIndexOf("/");
	if (position > -1 && position < path.length) {
		return path.substring(position + 1);
	}
	return "";
}

function weonclick(e) {
	if (e.altKey || e.metaKey || e.ctrlKey) {
		WE().util.dialogTop(window).metaKeys.ctrl = true;
	}
	if (e.shiftKey) {
		WE().util.dialogTop(window).metaKeys.shift = true;
	}
	if (WE().util.dialogTop(window).fileSelect.options.multiple) {
		if ((WE().util.dialogTop(window).metaKeys.shift === false) && (WE().util.dialogTop(window).metaKeys.ctrl === false)) {
			WE().util.dialogTop(window).unselectAllFiles();
		}
	} else {
		WE().util.dialogTop(window).unselectAllFiles();
	}
}

function elementSelected() {
	return WE().util.dialogTop(window).document.getElementsByName("fname")[0].value !== "";
}

function press_ok_button() {
	if (elementSelected()) {
		WE().util.dialogTop(window).exit_open();
	} else {
		WE().util.dialogTop(window).exit_close();
	}
}

function delBut(enable) {
	if (enable) {
		WE().layout.button.switch_button_state(document, "delete", "enabled");
		if (WE().util.dialogTop(window).fileSelect.options.userCanEditCat) {
			WE().layout.button.switch_button_state(document, "btn_function_trash", "enabled");
			WE().util.dialogTop(window).fileSelect.data.changeCatState = true;
		}
	} else {
		WE().layout.button.switch_button_state(document, "delete", "disabled");
		WE().layout.button.switch_button_state(document, "btn_function_trash", "disabled");
		WE().util.dialogTop(window).fileSelect.data.changeCatState = false;
	}
}

function startFrameset() {
	WE().util.dialogTop(window).document.getElementById('fspath').innerHTML = (WE().util.dialogTop(window).fileSelect.data.startPath === '' ? '/' : WE().util.dialogTop(window).fileSelect.data.startPath);
}

function rootDirButs(enable) {
	if (enable) {
		WE().layout.button.switch_button_state(document, "root_dir", "enabled");
		WE().layout.button.switch_button_state(document, "btn_fs_back", "enabled");
	} else {
		WE().layout.button.switch_button_state(document, "root_dir", "disabled");
		WE().layout.button.switch_button_state(document, "btn_fs_back", "disabled");
	}
	WE().util.dialogTop(window).fileSelect.data.rootDirButsState = enable;
}
function newFolderBut(enable) {
	if (enable) {
		WE().layout.button.switch_button_state(document, "btn_new_dir", "enabled");
	} else {
		WE().layout.button.switch_button_state(document, "btn_new_dir", "disabled");
	}
	WE().util.dialogTop(window).fileSelect.data.makefolderState = enable;
}

function NewBut(enable) {
	if (enable) {
		if (WE().util.dialogTop(window).fileSelect.options.userCanEditCat) {
			WE().layout.button.switch_button_state(document, "btn_new_dir", "enabled");
			WE().layout.button.switch_button_state(document, "btn_add_cat", "enabled");
		} else {
			WE().layout.button.switch_button_state(document, "btn_new_dir", "disabled");
			WE().layout.button.switch_button_state(document, "btn_add_cat", "disabled");
		}
	}
}

function newFileBut(enable) {
	if (enable) {
		WE().layout.button.switch_button_state(document, "btn_add_file", "enabled");
	} else {
		WE().layout.button.switch_button_state(document, "btn_add_file", "disabled");
	}
	WE().util.dialogTop(window).fileSelect.data.newFileState = enable;
}

function clearOptions() {
	var a = WE().util.dialogTop(window).document.getElementById("lookin");
	while (a.options.length) {
		a.options.remove(0);
	}
}
function addOption(txt, id) {
	var a = WE().util.dialogTop(window).document.getElementById("lookin");
	a.options[a.options.length] = new window.Option(txt, id);
	a.selectedIndex = (a.options.length > 0 ?
					a.options.length - 1 :
					0);

}
function selectIt() {
	var a = WE().util.dialogTop(window).document.getElementById("lookin");
	a.selectedIndex = a.options.length - 1;
}

function setview(view) {
	WE().util.dialogTop(window).fileSelect.options.view = view;
	var zoom = WE().util.dialogTop(window).document.getElementsByName("zoom")[0];
	switch (view) {
		case 'list':
			zoom.value = 100;
			if (zoom.onchange) {
				zoom.onchange();
			}
			zoom.disabled = true;
			zoom.style.display = "none";
			break;
		case 'icons':
			zoom.disabled = false;
			zoom.style.display = "inline";
			break;
	}
	WE().util.dialogTop(window).document.getElementById('list').style.display = (view === 'list' ? "none" : "table-cell");
	WE().util.dialogTop(window).document.getElementById('icons').style.display = (view === 'icons' ? "none" : "table-cell");

	WE().util.dialogTop(window).writeBody(WE().util.dialogTop(window).fsbody.document.body);
}

function we_cmd() {
	/*jshint validthis:true */
	let caller = (this && this.window === this ? this : window),
					args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments)),
			url = WE().util.getWe_cmdArgsUrl(args);
	switch (args[0]) {
		case 'clearEntries':
			WE().util.dialogTop(window).clearEntries();
			break;
		case 'addEntries':
			if (!args[1]) {
				break; 
			}
			for (let el of args[1]) {
				WE().util.dialogTop(window).addEntry.apply(window, el);
			}
			break;
		case 'writeOptions':
			WE().util.dialogTop(window).clearOptions();
			for (let el of args[1]) {
				WE().util.dialogTop(window).addOption.apply(window, el);
			}
			WE().util.dialogTop(window).selectIt();
			break;
		case 'writeBody':
			if (WE().util.dialogTop(window).fsbody.document.body) {
				WE().util.dialogTop(window).writeBody(WE().util.dialogTop(window).fsbody.document.body);
				//top.selectFile(top.fileSelect.data.currentID);
			}
			break;
		case 'updateTreeEntry':
			{
				let ref = (WE().util.dialogTop(window).top.treeData ? WE().util.dialogTop(window).top : (WE().util.dialogTop(window).opener.top.opener.top.treeData ? WE().util.dialogTop(window).opener.top.opener.top : null));
			if (ref) {
				ref.treeData.updateEntry(args[1]);
			}
			}
			break;
		case 'makeNewTreeEntry':
			{
				let ref = (WE().util.dialogTop(window).opener.top.treeData ? WE().util.dialogTop(window).opener.top : (WE().util.dialogTop(window).opener.top.opener.top.treeData ? WE().util.dialogTop(window).opener.top.opener.top : null));
			if (ref) {
				ref.treeData.makeNewEntry(args[1]);
			}
			if (WE().util.dialogTop(window).fileSelect.options.canSelectDir) {
				WE().util.dialogTop(window).document.getElementsByName("fname")[0].value = WE().util.dialogTop(window).fileSelect.data.currentText;
			}
			}
			break;
		case 'updateSelectData':
			var obj = args[1];
			for (let i in obj) {
				WE().util.dialogTop(window).fileSelect.data[i] = obj[i];
				if (obj === 'currentText') {
					WE().util.dialogTop(window).document.getElementsByName("fname")[0].value = WE().util.dialogTop(window).fileSelect.data.currentText;
				}
			}
			break;
		case 'setButtons':
			for (let enable of args[1]) {
				switch (enable[0]) {
					case 'NewFolderBut':
						newFolderBut(enable[1]);
						break;
					case 'NewFileBut':
						newFileBut(enable[1]);
						break;
					case 'RootDirButs':
						rootDirButs(enable[1]);
						break;
					case 'DelBut':
						delBut(enable[1]);
				}
			}
			break;
		case 'setLookinDir':
			WE().util.dialogTop(window).setDir(WE().util.dialogTop(window).document.getElementById("lookin").value);
			break;
		case 'updateCatChooserButton':
			WE().util.dialogTop(window).frames.fsvalues.document.we_form.elements.FolderID.value = args[1];
			WE().util.dialogTop(window).frames.fsvalues.document.we_form.elements.FolderIDPath.value = args[2];
			break;
		case 'newCatSuccess':
			WE().util.dialogTop(window).hot = true; // this is hot for category edit!!

			if (WE().util.dialogTop(window).fileSelect.data.currentID) {
				WE().util.dialogTop(window).delBut(false);
				WE().util.dialogTop(window).showPref(WE().util.dialogTop(window).fileSelect.data.currentID);
			}
			break;
		case 'unselectAllFiles':
			WE().util.dialogTop(window).unselectAllFiles();
			break;
		case 'postRenameCat':
			WE().util.dialogTop(window).hot = true; // this is hot for category edit!!
			if (WE().util.dialogTop(window).fileSelect.data.currentID) {
				WE().util.dialogTop(window).delBut(true);
				WE().util.dialogTop(window).showPref(WE().util.dialogTop(window).fileSelect.data.currentID);
			}
			break;
		case 'selector_insertFromUploader':
			var importedDoc = args[1];
			WE().util.dialogTop(window).reloadDir();
			WE().util.dialogTop(window).unselectAllFiles();
			WE().util.dialogTop(window).addEntry(importedDoc.currentID, importedDoc.currentText, false, importedDoc.currentPath, importedDoc.currentType);
			WE().util.dialogTop(window).doClick(importedDoc.currentID);
			window.setTimeout(WE().util.dialogTop(window).selectFile, 200, importedDoc.currentID);
			break;
		case "we_selector_file":
			(new (WE().layout.overlay)(caller, 'self', url, "we_selector", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		default:
			WE().util.dialogTop(window).opener.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
	}
}

function selectorOnClick(event, id) {
	weonclick(event);
	WE().util.dialogTop(window).metaKeys.doubleTout = setTimeout(function () {
		if (!WE().util.dialogTop(window).metaKeys.doubleClick) {
			WE().util.dialogTop(window).doClick(id, 0);
		} else {
			WE().util.dialogTop(window).metaKeys.doubleClick = false;
		}
	}, 300);
	return true;
}

function selectorOnDblClick(id) {
	WE().util.dialogTop(window).metaKeys.doubleClick = true;
	window.clearTimeout(WE().util.dialogTop(window).metaKeys.doubleTout);
	WE().util.dialogTop(window).doClick(id, 1);
	return true;
}

function exit_open() {
	var setTabsCurPath,
		suggestID = '';

	if (WE().util.dialogTop(window).fileSelect.data.JSIDName) {
		var elemID = WE().util.dialogTop(window).opener.document.we_form.elements[WE().util.dialogTop(window).fileSelect.data.JSIDName];
		elemID.value = WE().util.dialogTop(window).fileSelect.data.currentID;
		if (elemID.id && elemID.id.search('yuiAcResult') === 0) {
			suggestID = elemID.id.substr(11);
		}
	}
	if (WE().util.dialogTop(window).fileSelect.data.JSTextName) {
		var elemText = WE().util.dialogTop(window).opener.document.we_form.elements[WE().util.dialogTop(window).fileSelect.data.JSTextName];
		elemText.value = WE().util.dialogTop(window).fileSelect.data.currentID ? WE().util.dialogTop(window).fileSelect.data.currentPath : "";
		if (elemID.id && elemID.id.search('yuiAcInput') === 0) {
			suggestID = elemText.id.substr(10);
		}

		if ((WE().util.dialogTop(window).parent !== undefined) && (WE().util.dialogTop(window).parent.frames.editHeader !== undefined)) {
			if (WE().util.dialogTop(window).fileSelect.data.currentType !== "") {
				switch (WE().util.dialogTop(window).fileSelect.data.currentType) {
					case "noalias":
						setTabsCurPath = "@" + WE().util.dialogTop(window).fileSelect.data.currentText;
						break;
					default:
						setTabsCurPath = WE().util.dialogTop(window).fileSelect.data.currentPath;
				}
				if (getEntry(WE().util.dialogTop(window).fileSelect.data.currentID).isFolder) {
					WE().util.dialogTop(window).opener.parent.frames.editHeader.weTabs.setTitlePath("", setTabsCurPath);
				} else {
					WE().util.dialogTop(window).opener.parent.frames.editHeader.weTabs.setTitlePath(setTabsCurPath);
				}
			}
		}
		WE().layout.weSuggest.checkRequired(WE().util.dialogTop(window).opener, WE().util.dialogTop(window).opener.document.we_form.elements[WE().util.dialogTop(window).fileSelect.data.JSTextName].id);
	}
	if (WE().util.dialogTop(window).fileSelect.data.JSIDName) {
		var el = WE().util.dialogTop(window).opener.document.we_form.elements[WE().util.dialogTop(window).fileSelect.data.JSIDName];
		if (el.length !== undefined && el.length > 1) {
			WE().util.dialogTop(window).doClose();
			WE().util.showMessage(WE().consts.g_l.alert.templateDuplicateEntry, WE().consts.message.WE_MESSAGE_ERROR, window);
		}
	}
	// check is required for simple id-selectors on tag wizzard (with no hidden result field an no suggestor, i.e no data-xyz attributes)
	// FIXME: implementing fullflegded selector/suggestors in tag wizzard too, makes this obsolete
	if (WE().util.dialogTop(window).fileSelect.data.JSIDName &&
					!WE().util.dialogTop(window).fileSelect.data.JSTextName &&
					!WE().util.dialogTop(window).opener.document.we_form.elements[WE().util.dialogTop(window).fileSelect.data.JSIDName].getAttribute("data-result") &&
					WE().util.dialogTop(window).opener.document.we_form.elements[WE().util.dialogTop(window).fileSelect.data.JSIDName].getAttribute("required")) {
		WE().layout.weSuggest.checkRequired(WE().util.dialogTop(window).opener, WE().util.dialogTop(window).opener.document.we_form.elements[WE().util.dialogTop(window).fileSelect.data.JSIDName].id);
	}

	// if selector is used in combination with suggestor and we know sugestorID we call sugestors postporcess
	if (suggestID) {
		WE().util.dialogTop(window).opener.we_cmd('we_suggest_postprocessSelection', WE().util.dialogTop(window).fileSelect.data, suggestID);
	}

	// if not in combination with suggestor or if using custom selector callback: no onSelect is called automatically
	if (WE().util.dialogTop(window).fileSelect.data.JSCommand) {
		fillIDs();
		var tmp = WE().util.dialogTop(window).fileSelect.data.JSCommand.split(',');
		tmp.splice(1, 0, WE().util.dialogTop(window).fileSelect.data);
		WE().util.dialogTop(window).opener.we_cmd.apply(WE().util.dialogTop(window).opener, tmp);
	}

	WE().util.dialogTop(window).doClose();
}

WE().util.dialogTop(window).focus();
