/* global WE, top */

/**
 * webEdition CMS
 *
 * $Rev: 62067699fe89 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-09-01 13:04 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

var weOrderContainer = function (id) {
	this.container = id;
	this.elements = [];
	this.position = [];

	this.init = function () {
		this.setButtonsDisabled();
		//this.reinitWysiwygEditors();
	};

	this.processCommand = function (win, cmd, id, afterid) {
		switch (cmd) {
			case 'add':
				this.add(win.document, id, afterid);
				break;
			case 'reload':
				this.reload(win.document, id, win);
				break;
			case 'delete':
			case 'del':
				this.del(id);
				break;
			case 'moveup':
			case 'up':
				this.up(id);
				break;
			case 'movedown':
			case 'down':
				this.down(id);
		}
		this.setButtonsDisabled();
	};

	this.add = function (doc, id, afterid, isObjectFile) {
		let child,
			node = null,
			div = null,
			pos = this.position.length,
			element = [];

		element.id = id;
		this.elements[this.elements.length] = element;

		if (afterid) {
			for (let i = 0; i < this.position.length; i++) {
				if (this.position[i] === afterid) {
					pos = i + 1;
				}
			}
		}

		// hinten anhängen
		if (pos >= this.position.length) {
			pos = this.position.length;
			this.position.push(element.id);
			// vorne einfügen
		} else if (pos <= 0) {
			pos = 0;
			this.position.reverse();
			this.position.push(element.id);
			this.position.reverse();

			// einfügen
		} else {
			for (i = this.position.length; i > pos; i--) {
				this.position[i] = this.position[(i - 1)];
			}
			this.position[pos] = element.id;

		}

		if (doc === document) {
			if (isObjectFile) {
				return;
			}
			node = doc.getElementById(id);
			div = node;
		} else {
			div = doc.getElementById("orderContainer").childNodes[0];
		}

		if (this.position.length === 1 || pos >= this.position.length - 1) {
			document.getElementById(this.container).appendChild(div);
		} else {
			child = document.getElementById(this.container).childNodes;
			document.getElementById(this.container).insertBefore(div, child[pos]);
		}
	};

	this.reload = function (doc, id, win /*, selectedId, selectedValue*/) {
		let found = false;

		for (let i = 0; i < this.position.length; i++) {
			if (this.position[i] === id) {
				found = true;
				break;
			}
		}

		if (!found) {
			return;
		}
		let div = (doc === document ?
			doc.getElementById(id) :
			this.createDIV(document.importNode(doc.getElementById(id), true))
			);

		document.getElementById(id).innerHTML = div.innerHTML;

		let wysiwygConfigs;
		if ((wysiwygConfigs = WE().util.getDynamicVar(win.document, 'loadVar_tinyConfigs', 'data-configurations'))) {
			window.tinyMceRawConfigurations = window.tinyMceRawConfigurations ? window.tinyMceRawConfigurations : {};
			let config;
			for (let i = 0; i < wysiwygConfigs.length; i++) {
				config = wysiwygConfigs[i];
				window.tinyMceRawConfigurations[config.weFieldName] = config;
				if (config.weEditorType === 'inlineTrue') {
					WE().layout.we_tinyMCE.functions.initEditor(window, config);
				}
			}
		}
		we_cmd('reinitSuggest');
	};

	this.del = function (id) {
		for (let i = 0; i < this.elements.length; i++) {
			if (this.elements[i].id === id) {
				this.elements.splice(i, 1);
				i = this.elements.length;
			}
		}

		for (let i = 0; i < this.position.length; i++) {
			if (this.position[i] === id) {
				this.position.splice(i, 1);
				i = this.position.length;
			}
		}

		let node = document.getElementById(id);
		document.getElementById(this.container).removeChild(node);
	};


	this.up = function (id) {
		let up = null,
			down = null;

		for (let i = 1; i < this.position.length; i++) {
			if (this.position[i] === id) {
				up = document.getElementById(this.position[i]);
				down = document.getElementById(this.position[(i - 1)]);
				let temp = this.position[(i - 1)];
				this.position[(i - 1)] = this.position[i];
				this.position[i] = temp;
				i = this.position.length;
			}
		}

		if (up !== null && down !== null) {
			document.getElementById(this.container).removeChild(up);
			document.getElementById(this.container).insertBefore(up, down);
		}

		//this.reinitWysiwygEditors();
	};


	this.down = function (id) {
		let up = null,
			down = null;

		for (let i = 0; i < this.position.length - 1; i++) {
			if (this.position[i] === id) {
				up = document.getElementById(this.position[i + 1]);
				down = document.getElementById(this.position[i]);
				let temp = this.position[(i + 1)];
				this.position[(i + 1)] = this.position[i];
				this.position[i] = temp;
				i = this.position.length;
			}
		}

		if (up !== null && down !== null) {
			document.getElementById(this.container).removeChild(up);
			document.getElementById(this.container).insertBefore(up, down);
		}

		//this.reinitWysiwygEditors();
	};

	this.setButtonsDisabled = function () {
		for (let i = 0; i < this.position.length; i++) {
			let id = this.position[i].replace(/entry_/, '');

			WE().layout.button.disable(document, "btn_direction_up_" + id, false);
			WE().layout.button.disable(document, "btn_direction_down_" + id, false);

			if (i === 0) {
				WE().layout.button.disable(document, "btn_direction_up_" + id, true);
			}
			if (i + 1 === this.position.length) {
				WE().layout.button.disable(document, "btn_direction_down_" + id, true);
			}
		}
	};

	this.reinitWysiwygEditors = function () {
		let configs = window.tinyMceRawConfigurations;
		if (configs) {
			for (let key in configs) {
				if (configs.hasOwnProperty(key) && typeof configs[key] === 'object' &&
					document.getElementById('we_' + window.doc.docName + '_input[' + key + ']') &&
					configs[key].weEditorType === 'inlineTrue') {
					WE().layout.we_tinyMCE.functions.initEditor(window, configs[key]);
				}
			}
		}
	};

	this.createDIV = function (node) {
		let
			div = document.createElement("div"),
			attr = document.createAttribute("id");

		attr.value = node.getAttribute("id");
		div.innerHTML = node.innerHTML;
		div.setAttributeNode(attr);

		return div;

	};

};

var orderContainer = new weOrderContainer("orderContainer");
$(function () {
	orderContainer.init();
});
